    //
//  AppDelegate.m
//  plus
//
//  Created by Виктор Георгиев on 7/8/16.
//  Copyright © 2016 Outsourcing IT Services Ltd. All rights reserved.
//

#import "AppDelegate.h"

#import "MainMenuViewController.h"
#import "SubcategoriesViewController.h"
#import "CategoriesViewController.h"
#import "CatalogueViewController.h"
#import "PlaceViewController.h"

#import "URLRequests.h"
#import "Utils.h"
#import "AnimationTransitioning.h"

#import <FBSDKCoreKit/FBSDKCoreKit.h>

@interface AppDelegate ()

@end

@implementation AppDelegate

- (BOOL)application:(UIApplication *)application didFinishLaunchingWithOptions:(NSDictionary *)launchOptions {
    ((UINavigationController *)self.window.rootViewController).delegate = self;
    
    CGFloat osVersion = [[UIDevice currentDevice].systemVersion floatValue];
    if(osVersion >= 8.0) {
        UIUserNotificationSettings *settings = [UIUserNotificationSettings settingsForTypes:UIUserNotificationTypeBadge | UIUserNotificationTypeSound | UIUserNotificationTypeAlert categories:nil];
        [application registerUserNotificationSettings:settings];
    } else {
        //[application registerForRemoteNotificationTypes: (UIRemoteNotificationTypeBadge | UIRemoteNotificationTypeAlert | UIRemoteNotificationTypeSound)];
    }
    
    NSDictionary *userInfo = [launchOptions objectForKey:UIApplicationLaunchOptionsRemoteNotificationKey];
    if(userInfo != nil) {
        [self application:application didReceiveRemoteNotification:userInfo];
    }
    
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    if([userDefaults integerForKey:@"userId"] == 0) {
        [URLRequests setUser:@"bg" facebookId:nil email:nil firstname:nil lastname:nil receive:^(NSData *data) {
            NSError *error;
            NSDictionary *dictionary = [NSJSONSerialization JSONObjectWithData:data options:0 error:&error];
            int userId = [[dictionary objectForKey:@"id"] intValue];
            [userDefaults setInteger:userId forKey:@"userId"];
            [userDefaults synchronize];
        }];
    }
    
    [userDefaults setValue:@"" forKey:@"currentFilters"];
    [userDefaults setValue:@"" forKey:@"mapFilters"];
    [userDefaults setValue:@"" forKey:@"brandFilters"];
    
    NSString *currentAppVersion = [[NSBundle mainBundle] objectForInfoDictionaryKey:@"CFBundleShortVersionString"];
    NSString *buildAppVersion = [[NSBundle mainBundle] objectForInfoDictionaryKey:(NSString*)kCFBundleVersionKey];
    NSString *combinedVersion = [NSString stringWithFormat:@"%@+%@", currentAppVersion, buildAppVersion];
    NSString *previousVersion = [userDefaults objectForKey:@"appVersion"];
    if (!previousVersion) {
        [userDefaults setObject:combinedVersion forKey:@"appVersion"];
        [userDefaults setObject:[NSArray arrayWithObjects:@"bg", @"en", nil] forKey:@"AppleLanguages"];
    } else if ([previousVersion isEqualToString:combinedVersion]) {
        NSFetchRequest *fetchRequest = [NSFetchRequest fetchRequestWithEntityName:@"Filter"];
        NSError *error;
        NSArray *items = [self.managedObjectContext executeFetchRequest:fetchRequest error:&error];
        for (NSManagedObject *managedObject in items) {
            [self.managedObjectContext deleteObject:managedObject];
        }
    } else {
        NSURL *storeURL = [[self applicationDocumentsDirectory] URLByAppendingPathComponent:@"plus.sqlite"];
        NSFileManager *fileManager = [[NSFileManager alloc] init];
        NSError *error;
        [fileManager removeItemAtURL:storeURL error:&error];
        
        storeURL = [[self applicationDocumentsDirectory] URLByAppendingPathComponent:@"plus.sqlite-shm"];
        [fileManager removeItemAtURL:storeURL error:&error];
        
        storeURL = [[self applicationDocumentsDirectory] URLByAppendingPathComponent:@"plus.sqlite-wal"];
        [fileManager removeItemAtURL:storeURL error:&error];

        [userDefaults setObject:combinedVersion forKey:@"appVersion"];
    }
    
    [userDefaults synchronize];
    
    [[FBSDKApplicationDelegate sharedInstance] application:application didFinishLaunchingWithOptions:launchOptions];
    
    NSDictionary *notification = [launchOptions objectForKey:UIApplicationLaunchOptionsRemoteNotificationKey];
    if (notification) {
        self.hasNotification = YES;
    }
    
    return YES;
}

- (BOOL)application:(UIApplication *)application openURL:(NSURL *)url sourceApplication:(NSString *)sourceApplication annotation:(id)annotation {
    return [[FBSDKApplicationDelegate sharedInstance] application:application openURL:url sourceApplication:sourceApplication annotation:annotation];
}

- (void)applicationWillResignActive:(UIApplication *)application {
    // Sent when the application is about to move from active to inactive state. This can occur for certain types of temporary interruptions (such as an incoming phone call or SMS message) or when the user quits the application and it begins the transition to the background state.
    // Use this method to pause ongoing tasks, disable timers, and throttle down OpenGL ES frame rates. Games should use this method to pause the game.
}

- (void)applicationDidEnterBackground:(UIApplication *)application {
    // Use this method to release shared resources, save user data, invalidate timers, and store enough application state information to restore your application to its current state in case it is terminated later.
    // If your application supports background execution, this method is called instead of applicationWillTerminate: when the user quits.
}

- (void)applicationWillEnterForeground:(UIApplication *)application {
    if([Utils getLocation] == nil) {
        [((MainMenuViewController *)[((UINavigationController *)self.window.rootViewController).viewControllers objectAtIndex:0]) downloadContent];
    }
}

- (void)applicationDidBecomeActive:(UIApplication *)application {
    [FBSDKAppEvents activateApp];
}

- (void)applicationWillTerminate:(UIApplication *)application {
    // Called when the application is about to terminate. Save data if appropriate. See also applicationDidEnterBackground:.
    // Saves changes in the application's managed object context before the application terminates.
    [self saveContext];
}

- (void)application:(UIApplication *)application didRegisterUserNotificationSettings:(UIUserNotificationSettings *)notificationSettings {
    //register to receive notifications
    [application registerForRemoteNotifications];
}

-(void)application:(UIApplication *)application didRegisterForRemoteNotificationsWithDeviceToken:(NSData *)deviceToken {
    [[NSUserDefaults standardUserDefaults] setObject:deviceToken forKey:@"deviceToken"];
    [[NSUserDefaults standardUserDefaults] synchronize];
}

-(void)application:(UIApplication *)application didFailToRegisterForRemoteNotificationsWithError:(NSError *)error {
    
}

-(void) application:(UIApplication *)application didReceiveRemoteNotification:(NSDictionary *)userInfo {
    MainMenuViewController *mainMenuViewController = (MainMenuViewController *) ((UINavigationController *)self.window.rootViewController).viewControllers.firstObject;
    if(application.applicationState == UIApplicationStateInactive || application.applicationState == UIApplicationStateBackground) {
        if(mainMenuViewController.brandsViewController.areBrandsVisible) {
            [mainMenuViewController performSegueWithIdentifier:@"NotificationSegue" sender:mainMenuViewController];
        }
    }
}

#pragma mark - Core Data stack

@synthesize managedObjectContext = _managedObjectContext;
@synthesize managedObjectModel = _managedObjectModel;
@synthesize persistentStoreCoordinator = _persistentStoreCoordinator;

- (NSURL *)applicationDocumentsDirectory {
    // The directory the application uses to store the Core Data store file. This code uses a directory named "bg.unicredit.plus" in the application's documents directory.
    return [[[NSFileManager defaultManager] URLsForDirectory:NSDocumentDirectory inDomains:NSUserDomainMask] lastObject];
}

- (NSManagedObjectModel *)managedObjectModel {
    // The managed object model for the application. It is a fatal error for the application not to be able to find and load its model.
    if (_managedObjectModel != nil) {
        return _managedObjectModel;
    }
    NSURL *modelURL = [[NSBundle mainBundle] URLForResource:@"plus" withExtension:@"momd"];
    _managedObjectModel = [[NSManagedObjectModel alloc] initWithContentsOfURL:modelURL];
    return _managedObjectModel;
}

- (NSPersistentStoreCoordinator *)persistentStoreCoordinator {
    // The persistent store coordinator for the application. This implementation creates and returns a coordinator, having added the store for the application to it.
    if (_persistentStoreCoordinator != nil) {
        return _persistentStoreCoordinator;
    }
    
    // Create the coordinator and store
    
    _persistentStoreCoordinator = [[NSPersistentStoreCoordinator alloc] initWithManagedObjectModel:[self managedObjectModel]];
    NSURL *storeURL = [[self applicationDocumentsDirectory] URLByAppendingPathComponent:@"plus.sqlite"];
    NSError *error = nil;
    NSString *failureReason = @"There was an error creating or loading the application's saved data.";
    if (![_persistentStoreCoordinator addPersistentStoreWithType:NSSQLiteStoreType configuration:nil URL:storeURL options:nil error:&error]) {
        // Report any error we got.
        NSMutableDictionary *dict = [NSMutableDictionary dictionary];
        dict[NSLocalizedDescriptionKey] = @"Failed to initialize the application's saved data";
        dict[NSLocalizedFailureReasonErrorKey] = failureReason;
        dict[NSUnderlyingErrorKey] = error;
        error = [NSError errorWithDomain:@"YOUR_ERROR_DOMAIN" code:9999 userInfo:dict];
        // Replace this with code to handle the error appropriately.
        // abort() causes the application to generate a crash log and terminate. You should not use this function in a shipping application, although it may be useful during development.
        NSLog(@"Unresolved error %@, %@", error, [error userInfo]);
        abort();
    }
    
    return _persistentStoreCoordinator;
}


- (NSManagedObjectContext *)managedObjectContext {
    // Returns the managed object context for the application (which is already bound to the persistent store coordinator for the application.)
    if (_managedObjectContext != nil) {
        return _managedObjectContext;
    }
    
    NSPersistentStoreCoordinator *coordinator = [self persistentStoreCoordinator];
    if (!coordinator) {
        return nil;
    }
    _managedObjectContext = [[NSManagedObjectContext alloc] initWithConcurrencyType:NSMainQueueConcurrencyType];
    [_managedObjectContext setPersistentStoreCoordinator:coordinator];
    return _managedObjectContext;
}

#pragma mark - Core Data Saving support

- (void)saveContext {
    NSManagedObjectContext *managedObjectContext = self.managedObjectContext;
    if (managedObjectContext != nil) {
        NSError *error = nil;
        if ([managedObjectContext hasChanges] && ![managedObjectContext save:&error]) {
            // Replace this implementation with code to handle the error appropriately.
            // abort() causes the application to generate a crash log and terminate. You should not use this function in a shipping application, although it may be useful during development.
            NSLog(@"Unresolved error %@, %@", error, [error userInfo]);
            abort();
        }
    }
}

-(id<UIViewControllerAnimatedTransitioning>) navigationController:(UINavigationController *)navigationController animationControllerForOperation:(UINavigationControllerOperation)operation fromViewController:(UIViewController *)fromVC toViewController:(UIViewController *)toVC {
    AnimationTransitioning *animationTransitioning = [[AnimationTransitioning alloc] init];
    if(operation == UINavigationControllerOperationPush) {
        if([toVC isKindOfClass:[CatalogueViewController class]]) {
            animationTransitioning = nil;
        }
    } else if(operation == UINavigationControllerOperationPop) {
        if([fromVC isKindOfClass:[CatalogueViewController class]]) {
            animationTransitioning = nil;
        }
    }
    return animationTransitioning;

}

@end
