//
//  PlacesViewController.m
//  plus
//
//  Created by Виктор Георгиев on 7/21/16.
//  Copyright © 2016 Outsourcing IT Services Ltd. All rights reserved.
//

#import "AppDelegate.h"

#import "MainMenuViewController.h"
#import "BrandsViewController.h"
#import "SingleBrandViewController.h"
#import "PlaceViewController.h"
#import "CatalogueViewController.h"
#import "MapViewController.h"
#import "ContentHolderViewController.h"
#import "WebVieWController.h"
#import "CategoriesViewController.h"

#import "BrandCollectionViewCell.h"

#import "URLRequests.h"
#import "ItemCategory.h"
#import "Brand.h"
#import "Utils.h"
#import "Filter.h"
#import "BrandName.h"
#import "AnimationTransitioning.h"

@interface BrandsViewController ()

@property (strong, nonatomic) NSUserDefaults *userDefaults;

@property (strong, nonatomic) CLLocation *lastKnownLocation;

@property (weak, nonatomic) IBOutlet UIButton *backButton;

@property (weak, nonatomic) IBOutlet UILabel *noResultsLabel;

@property (assign, nonatomic) int brandsPage;

@property (assign, nonatomic) long brandsCount;

@property (assign, nonatomic) float osVersion;

@property (assign, nonatomic) BOOL timerHasFired;

@end

@implementation BrandsViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    self.mainMenuViewController = (MainMenuViewController *)[((ContentHolderViewController *)[self.navigationController.viewControllers objectAtIndex:0]).mainNavigationController.viewControllers objectAtIndex:0];
    self.contentHolderViewController = (ContentHolderViewController *)[self.navigationController.viewControllers objectAtIndex:0];
    
    self.userDefaults = [NSUserDefaults standardUserDefaults];
    
    self.areBrandsVisible = YES;
    self.brandsPage = 1;
    self.shouldAddBrands = YES;
    
    self.locationManager = [[CLLocationManager alloc] init];
    self.locationManager.delegate = self;
    self.locationManager.distanceFilter = kCLDistanceFilterNone;
    self.locationManager.desiredAccuracy = kCLLocationAccuracyBest;
    self.osVersion = [[UIDevice currentDevice].systemVersion floatValue];
    
    self.shouldDisplayAnimation = NO;
    
    if([CLLocationManager authorizationStatus] == kCLAuthorizationStatusAuthorizedWhenInUse) {
        if(self.osVersion >= 9) {
            [self.locationManager requestLocation];
            [self.contentHolderViewController.mapViewController.locationManager requestLocation];

        } else {
            [self.locationManager startUpdatingLocation];
            [self.contentHolderViewController.mapViewController.locationManager startUpdatingLocation];
        }
    } else {
        if(self.osVersion >= 8) {
            [self.locationManager requestWhenInUseAuthorization];
        } else {
            [self.locationManager startUpdatingLocation];
        }
    }
    
    self.noResultsLabel.text = [Utils translate:@"No results"];
    
    [self getBrandsList];

    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(changeLanguage:) name:@"LanguageNotification" object:nil];
}

-(void)viewDidAppear:(BOOL)animated {
    [super viewDidAppear:animated];
    
    CGRect rect = self.backButton.bounds;
    UIBezierPath *maskPath = [UIBezierPath bezierPathWithRoundedRect:rect byRoundingCorners:UIRectCornerTopRight cornerRadii:CGSizeMake(10, 10)];
    CAShapeLayer *maskLayer = [CAShapeLayer layer];
    maskLayer.fillColor = [Utils colorFromHexString:@"#46ADC6"].CGColor;
    maskLayer.frame = rect;
    maskLayer.path = maskPath.CGPath;
    [self.backButton.layer insertSublayer:maskLayer atIndex:0];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
}

- (IBAction)onBackPressed:(id)sender {
    self.keyword = nil;
    [self showSearchResults];
    self.shouldDisplayAnimation = YES;
}

- (void)locationManager:(CLLocationManager *)manager didChangeAuthorizationStatus:(CLAuthorizationStatus)status {
    if(status == kCLAuthorizationStatusAuthorizedWhenInUse) {
        if(self.osVersion >= 9) {
            [self.locationManager requestLocation];
            [self.contentHolderViewController.mapViewController.locationManager requestLocation];
            
        } else {
            [self.locationManager startUpdatingLocation];
            [self.contentHolderViewController.mapViewController.locationManager startUpdatingLocation];
        }
    }
}

-(void)locationManager:(CLLocationManager *)manager didUpdateLocations:(NSArray<CLLocation *> *)locations {
    self.lastKnownLocation = [locations lastObject];
    if(self.shouldAddBrands) {
        [self getBrands];
    }
}

-(void)locationManager:(CLLocationManager *)manager didFailWithError:(NSError *)error {
    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:@"Please turn on your GPS" message:@"" preferredStyle:UIAlertControllerStyleAlert];
    [alertController addAction:[UIAlertAction actionWithTitle:@"Cancel" style:UIAlertActionStyleDefault handler:^(UIAlertAction *action) {
        [self dismissViewControllerAnimated:YES completion:^{
        }];
    }]];
    
    [alertController addAction:[UIAlertAction actionWithTitle:@"OK" style:UIAlertActionStyleCancel handler:^(UIAlertAction *action) {
        if(self.osVersion >= 9) {
            [self.locationManager requestLocation];
            [self.contentHolderViewController.mapViewController.locationManager requestLocation];
            
        } else {
            [self.locationManager startUpdatingLocation];
            [self.contentHolderViewController.mapViewController.locationManager startUpdatingLocation];
        }
    }]];
    
    [self.mainMenuViewController presentViewController:alertController animated:YES completion:nil];
}

-(void)getBrands {
    NSString *filter = (NSString *)[self.userDefaults objectForKey:@"brandFilters"];
    if([filter isEqualToString:@""]) {
        filter = nil;
    }
    NSString *keyword = nil;
    if(self.keyword != nil && ![self.keyword isEqualToString:@""]) {
        keyword = self.keyword;
        self.backButton.hidden = NO;
    } else {
        self.backButton.hidden = YES;
        self.noResultsLabel.hidden = YES;
    }
    
    if(!self.shouldDisplayAnimation) {
        dispatch_after(dispatch_time(DISPATCH_TIME_NOW, 0 * NSEC_PER_SEC), dispatch_get_main_queue(), ^{
            self.timerHasFired = YES;
            if(self.brandsArray != nil) {
                [Utils dismissAnimation];
                [UIView animateWithDuration:0.5 animations:^{
                    self.mainMenuViewController.animatedSplashScreen.alpha = 0;
                } completion:^(BOOL finished) {
                    self.mainMenuViewController.animatedSplashScreen.hidden = YES;
                    if(self.mainMenuViewController.newNotificationsCount > 0) {
                        [self.mainMenuViewController startNotificationAnimation];
                    }
                }];
            }
        });
        self.timerHasFired = NO;
    }
    //        if(!self.shouldDisplayAnimation) {
    //            [Utils displayAnimation];
    //        }
    [URLRequests getBrands:[self.lastKnownLocation coordinate] language:[Utils currentLanguage] page:self.brandsPage filter:filter keyword:keyword displayAnimation:self.shouldDisplayAnimation receive:^(NSArray<Brand *> *items, long brandsCount) {
        if(self.brandsPage > 1) {
            self.brandsArray = [self.brandsArray arrayByAddingObjectsFromArray:items];
        } else {
            self.brandsArray = items;
        }
        if((!self.shouldDisplayAnimation && self.timerHasFired) || items.count == 0) {
            [Utils dismissAnimation];
        }
        if(self.brandsArray.count == 0) {
            self.noResultsLabel.hidden = NO;
        } else if(self.brandsArray.count > 0) {
            self.noResultsLabel.hidden = YES;
        }
        self.brandsCount = brandsCount;
        [self.brandsCollectionView reloadData];
        if([self.mainMenuViewController.searchTextField isFirstResponder]) {
            [self.mainMenuViewController.searchTextField resignFirstResponder];
        }
        if(self.timerHasFired) {
            [UIView animateWithDuration:0.5 animations:^{
                self.mainMenuViewController.animatedSplashScreen.alpha = 0;
            } completion:^(BOOL finished) {
                self.mainMenuViewController.animatedSplashScreen.hidden = YES;
                if(self.mainMenuViewController.newNotificationsCount > 0) {
                    [self.mainMenuViewController startNotificationAnimation];
                }
                if(((AppDelegate *)[UIApplication sharedApplication].delegate).hasNotification) {
                    [self.mainMenuViewController performSegueWithIdentifier:@"NotificationSegue" sender:self.mainMenuViewController];
                    [Utils displayAnimation];
                    ((AppDelegate *)[UIApplication sharedApplication].delegate).hasNotification = NO;
                }
            }];
        }
    }];
    
    self.shouldAddBrands = NO;
}

-(void)getBrandsList {
    [URLRequests getBrandsList:^(BOOL success, NSArray<BrandName *> *brandNamesArray) {
        if(success) {
            self.mainMenuViewController.brandNamesArray = brandNamesArray;
        }
    }];
}

-(void)showSearchResults {
    self.brandsPage = 1;
    self.shouldAddBrands = YES;
//    if(self.osVersion >= 9) {
//        [self.locationManager requestLocation];
//    } else {
//        [self.locationManager startUpdatingLocation];
//    }
    [self getBrands];
}

- (NSInteger)collectionView:(UICollectionView *)collectionView numberOfItemsInSection:(NSInteger)section {
    return self.brandsArray.count;
}

- (CGSize)collectionView:(UICollectionView *)collectionView layout:(UICollectionViewLayout *)collectionViewLayout sizeForItemAtIndexPath:(NSIndexPath *)indexPath {
    CGRect screenBounds = [[UIScreen mainScreen] bounds];
    CGSize size = CGSizeMake((screenBounds.size.width - 45) / 2, 177);
    return size;
}

//- (UICollectionReusableView *)collectionView:(UICollectionView *)theCollectionView viewForSupplementaryElementOfKind:(NSString *)kind atIndexPath:(NSIndexPath *)theIndexPath {
//    UICollectionReusableView *view = nil;
//    if(kind  == UICollectionElementKindSectionFooter) {
//        view = [self.brandsCollectionView dequeueReusableSupplementaryViewOfKind:kind withReuseIdentifier:@"Footer" forIndexPath:theIndexPath];
//        view.backgroundColor = [UIColor whiteColor];
//        ((UILabel *)[view viewWithTag:1]).text = [Utils translate:@"Show more"];
//    }
//    return view;
//}

- (UICollectionViewCell *)collectionView:(UICollectionView *)collectionView cellForItemAtIndexPath:(NSIndexPath *)indexPath {
    BrandCollectionViewCell *cell = (BrandCollectionViewCell *)[collectionView dequeueReusableCellWithReuseIdentifier:@"Cell" forIndexPath:indexPath];
    cell.brandsViewController = self;
    //cell.layer.shouldRasterize = YES;
    //cell.layer.rasterizationScale = [UIScreen mainScreen].scale;
    Brand *brand = (Brand *) [self.brandsArray objectAtIndex:indexPath.item];
    cell.locationId = brand.brandId;
    cell.brandsCollectionView = self.brandsCollectionView;
    cell.isBrand = brand.isBrand;
    cell.loadIndicator.hidesWhenStopped = YES;
    if(brand.brandImage != nil && ![brand.brandImage isEqualToString:@""]) {
        cell.brandNameLabelHeightConstraint.constant = 15;
        cell.brandNameLabelVerticalConstraint.constant = 8;
        cell.brandHeaderLabel.hidden = YES;
        cell.brandImageView.image = nil;
        [cell.loadIndicator startAnimating];
        [URLRequests loadImage:brand.brandImage withFolder:nil receive:^(UIImage *image) {
            for(NSIndexPath *indexPathForImage in [collectionView indexPathsForVisibleItems]) {
                if([[self.brandsArray objectAtIndex:indexPathForImage.item].brandImage isEqualToString:brand.brandImage]) {
                    ((BrandCollectionViewCell *)[collectionView cellForItemAtIndexPath:indexPathForImage]).brandImageView.image = image;
                    [cell.loadIndicator stopAnimating];
                }
            }
        }];
        if(![brand.brandColor isKindOfClass:[NSNull class]] && ![brand.brandColor isEqualToString:@""] && ![brand.brandColor isEqualToString:@"null"]) {
            cell.brandImageView.backgroundColor = [Utils colorFromHexString:[NSString stringWithFormat:@"#%@", brand.brandColor]];
        } else {
            cell.brandImageView.backgroundColor = [UIColor clearColor];
        }
    } else {
        cell.brandImageView.image = [UIImage imageNamed:@"no-logo"];
        cell.brandImageView.backgroundColor = [Utils colorFromHexString:@"#E6E7E8"];
        [cell.loadIndicator stopAnimating];
        //cell.brandHeaderLabel.text = brand.brandName;
        //cell.brandHeaderLabel.hidden = NO;
        cell.brandNameLabelHeightConstraint.constant = 15;
        cell.brandNameLabelVerticalConstraint.constant = 8;
    }
    
    if(!brand.isFollowed) {
        [cell.brandFollowButton setImage:[UIImage imageNamed:@"add_to_favs"] forState:UIControlStateNormal];
        cell.followed = 0;
    } else {
        [cell.brandFollowButton setImage:[UIImage imageNamed:@"remove_from_favs"] forState:UIControlStateNormal];
        cell.followed = 1;
    }
    
    if(brand.brandIsNewBrand) {
        cell.brandPromoLabel.text = [Utils translate:@"New brand"];
        CGFloat width = [cell.brandPromoLabel.text boundingRectWithSize:CGSizeMake(CGFLOAT_MAX, cell.brandPromoView.frame.size.height) options:NSStringDrawingUsesLineFragmentOrigin attributes:@{NSFontAttributeName : cell.brandPromoLabel.font} context:nil].size.width;
        cell.brandPromoViewWidthConstraint.constant = width + 10;
        cell.brandPromoView.backgroundColor = [Utils colorFromHexString:@"#00D3B3"];
        cell.brandPromoView.hidden = NO;
    } else if(brand.brandIsNewLocation) {
        cell.brandPromoLabel.text = [Utils translate:@"New location"];
        CGFloat width = [cell.brandPromoLabel.text boundingRectWithSize:CGSizeMake(CGFLOAT_MAX, cell.brandPromoView.frame.size.height) options:NSStringDrawingUsesLineFragmentOrigin attributes:@{NSFontAttributeName : cell.brandPromoLabel.font} context:nil].size.width;
        cell.brandPromoViewWidthConstraint.constant = width + 10;
        cell.brandPromoView.backgroundColor = [Utils colorFromHexString:@"#00D3B3"];
        cell.brandPromoView.hidden = NO;
    } else if(brand.hasPromotion) {
        cell.brandPromoLabel.text = [Utils translate:@"Promo"];
        CGFloat width = [cell.brandPromoLabel.text boundingRectWithSize:CGSizeMake(CGFLOAT_MAX, cell.brandPromoView.frame.size.height) options:NSStringDrawingUsesLineFragmentOrigin attributes:@{NSFontAttributeName : cell.brandPromoLabel.font} context:nil].size.width;
        cell.brandPromoViewWidthConstraint.constant = width + 10;
        cell.brandPromoView.backgroundColor = [Utils colorFromHexString:@"#E61D25"];
        cell.brandPromoView.hidden = NO;
        
    } else {
        cell.brandPromoView.hidden = YES;
    }
    
    cell.brandNameLabel.text = brand.brandName;
    
    NSArray<ItemCategory *> *categoriesArray = [Utils getCategories];
    for(ItemCategory *itemCategory in categoriesArray) {
        if(brand.brandCategoryId.intValue == itemCategory.itemId.intValue) {
            cell.brandCategoryLabel.text = itemCategory.itemName;
            break;
        } else {
            BOOL found = NO;
            for(ItemCategory *itemSubcategiory in itemCategory.itemSubcategories) {
                if(brand.brandCategoryId.intValue == itemSubcategiory.itemId.intValue) {
                    cell.brandCategoryLabel.text = itemSubcategiory.itemName;
                    found = YES;
                    break;
                }
            }
            if(found) {
                break;
            }
        }
    }
    
    cell.brandLocationsCountLabel.text = [NSString stringWithFormat:@"%d", brand.brandCount.intValue];
    if(brand.brandCount.longValue > 0) {
        cell.brandLocationsCountImageView.hidden = NO;
        cell.brandLocationsCountLabel.hidden = NO;
    } else {
        cell.brandLocationsCountImageView.hidden = YES;
        cell.brandLocationsCountLabel.hidden = YES;
    }
    
    CLLocation *brandLocation = [[CLLocation alloc] initWithLatitude:brand.brandLatitude.floatValue longitude:brand.brandLongitude.floatValue];
    CLLocationDistance distance = [self.lastKnownLocation distanceFromLocation:brandLocation];
    if(distance < 1000) {
        cell.brandDistanceLabel.text = [NSString stringWithFormat:@"%d %@", (int)distance, [Utils translate:@"Meter"]];
    } else {
        cell.brandDistanceLabel.text = [NSString stringWithFormat:@"%.2f %@", distance / 1000, [Utils translate:@"Kilometer"]];
    }
    CGFloat labelWidth = [cell.brandDistanceLabel.text boundingRectWithSize:CGSizeMake(CGFLOAT_MAX, 20) options:NSStringDrawingUsesLineFragmentOrigin attributes: @{NSFontAttributeName : cell.brandDistanceLabel.font} context:nil].size.width;
    cell.brandDistanceLabelWidthConstraint.constant = labelWidth;

    return cell;
}

- (void)collectionView:(UICollectionView *)collectionView willDisplayCell:(UICollectionViewCell *)cell forItemAtIndexPath:(NSIndexPath *)indexPath {
    Brand *brand = (Brand *) [self.brandsArray objectAtIndex:indexPath.item];
    UIImage *image = [URLRequests getImage:brand.brandImage];
    if(image != nil) {
        BrandCollectionViewCell *brandCell = (BrandCollectionViewCell *)cell;
        brandCell.brandImageView.image = image;
        [brandCell.loadIndicator stopAnimating];
    }
    
    if(self.brandsArray.count - indexPath.item <= 4) {
        if(!self.shouldAddBrands && self.brandsCount >= 24) {
            [self showMore:self];
        }
    }
    
    if(self.brandsArray.count - 1 == indexPath.item && self.brandsArray.count != self.brandsCount) {
        [Utils displayAnimation];
    }
}

- (void)collectionView:(UICollectionView *)collectionView didSelectItemAtIndexPath:(NSIndexPath *)indexPath {
    Brand *brand = (Brand *) [self.brandsArray objectAtIndex:indexPath.item];
    if(brand.isBrand) {
        [self performSegueWithIdentifier:@"BrandSegue" sender:self];
    } else if(!brand.isBrand) {
        [self performSegueWithIdentifier:@"PlaceSegue" sender:self];
    }
}

- (IBAction)showMore:(id)sender {
    self.brandsPage++;
    self.shouldAddBrands = YES;
    self.shouldDisplayAnimation = YES;
    if(self.osVersion >= 9) {
        [self.locationManager requestLocation];
    } else {
        [self.locationManager startUpdatingLocation];
    }
}

-(void)setFilters {
    if(self.areBrandsVisible) {
        NSString *currentFilters = (NSString *)[self.userDefaults valueForKey:@"currentFilters"];
        NSString *savedFilters = (NSString *)[self.userDefaults valueForKey:@"brandFilters"];
        if(![savedFilters isEqualToString:currentFilters]) {
            [self.userDefaults setValue:currentFilters forKey:@"brandFilters"];
            [self.userDefaults synchronize];
            [self.locationManager requestLocation];
            self.shouldAddBrands = YES;
            self.brandsPage = 1;
            self.shouldDisplayAnimation = YES;
        }
    }
}

-(void)changeLanguage:(NSNotification *)notification {
    self.brandsPage = 1;
    [URLRequests getBrands:[self.lastKnownLocation coordinate] language:[Utils currentLanguage] page:self.brandsPage filter:nil keyword:nil displayAnimation:NO receive:^(NSArray<Brand *> *items, long brandsCount) {
        self.brandsArray= items;
        self.brandsCount = brandsCount;
        [self.brandsCollectionView reloadData];
    }];
}

-(void)showMap {
    [UIView animateWithDuration:0.3 animations:^{
        self.contentHolderViewController.contentScrollView.contentOffset = CGPointMake([UIScreen mainScreen].bounds.size.width, self.contentHolderViewController.contentScrollView.contentOffset.y);
    }];
}

#pragma mark - Navigation

- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    if([self.mainMenuViewController.searchTextField isFirstResponder]) {
        [self.mainMenuViewController.searchTextField resignFirstResponder];
    }

    if([[segue destinationViewController] isKindOfClass:[PlaceViewController class]]) {
        NSNumber *item = ((Brand *)[self.brandsArray objectAtIndex:[self.brandsCollectionView indexPathsForSelectedItems][0].row]).brandId;
        PlaceViewController *placeViewController = (PlaceViewController *)[segue destinationViewController];
        placeViewController.placeId = item;
        placeViewController.headerFieldType = @"search";
        placeViewController.isFromMainDrawer = NO;
    } else if([[segue destinationViewController] isKindOfClass:[SingleBrandViewController class]]) {
        SingleBrandViewController *singleBrandViewController = (SingleBrandViewController *)[segue destinationViewController];
        singleBrandViewController.isFromMainDrawer = NO;
        singleBrandViewController.headerFieldType = @"search";
        if(!self.loadFromSearchResults) {
            Brand *brand = (Brand *)[self.brandsArray objectAtIndex:[self.brandsCollectionView indexPathsForSelectedItems][0].row];
            singleBrandViewController.brandId = brand.brandId;
        } else {
            singleBrandViewController.brandId = self.brandId;
            self.loadFromSearchResults = NO;
        }
    } else if([[segue destinationViewController] isKindOfClass:[WebViewController class]]) {
        WebViewController *webViewController = (WebViewController *)[segue destinationViewController];
        NSURL *url = [NSURL URLWithString:@"http://www.unicreditbulbank.bg/bg/index.htm"];
        NSURLRequest *request = [NSURLRequest requestWithURL:url];
        webViewController.request = request;
        webViewController.isFromMainDrawer = NO;
        webViewController.onDismiss = ^{
            [self.mainMenuViewController changeHeader:nil withType:@"search"];
        };
    }
}

@end
