//
//  PlacesCollectionViewCell.m
//  plus
//
//  Created by Виктор Георгиев on 7/15/16.
//  Copyright © 2016 Outsourcing IT Services Ltd. All rights reserved.
//

#define DURATION 0.5f
#define NEW 0
#define PROMOTIONS 1
#define FAVOURITES 2
#define SHOPPING_CENTERS 3

#import "CatalogueCollectionViewCell.h"

#import "CatalogueViewController.h"
#import "MainMenuViewController.h"

#import "URLRequests.h"
#import "Promotion.h"
#import "NewPlace.h"
#import "Brand.h"
#import "ShoppingCenter.h"

#import <FBSDKCoreKit/FBSDKCoreKit.h>

@implementation CatalogueCollectionViewCell

- (IBAction)onFollowPressed:(id)sender {
    if(self.isBrand) {
        [URLRequests followBrand:self.followed withLocationId:self.locationId.intValue receive:^(BOOL success) {
            if(success) {
                if(self.followed == 1) {
                    self.followed = 0;
                    [self unfollowAnimation:self.catalogueFollowButton];
                    [self changeFollowed:self.followed];
                } else {
                    self.followed = 1;
                    [self followAnimation:self.catalogueFollowButton];
                    [self changeFollowed:self.followed];
                }
            }
        }];
    } else {
        [URLRequests followPlace:self.followed withLocationId:self.locationId.intValue receive:^(BOOL success) {
            if(success) {
                if(self.followed == 1) {
                    self.followed = 0;
                    [self unfollowAnimation:self.catalogueFollowButton];
                    [self changeFollowed:self.followed];
                } else {
                    self.followed = 1;
                    [self followAnimation:self.catalogueFollowButton];
                    [self changeFollowed:self.followed];
                }
            }
        }];
    }
}

-(void)changeFollowed:(int)followed {
    MainMenuViewController *mainMenuViewController = (MainMenuViewController *)[self.catalogueViewController.catalogueControlViewController.navigationController.viewControllers objectAtIndex:0];
    for(Brand *brand in mainMenuViewController.brandsViewController.brandsArray) {
        if(brand.brandId.intValue == self.locationId.intValue) {
            if(followed == 0) {
                brand.isFollowed = NO;
            } else {
                brand.isFollowed = YES;
            }
            [mainMenuViewController.brandsViewController.brandsCollectionView reloadData];
        }
    }
    
    if(self.catalogueViewController.catalogueType == NEW || self.catalogueViewController.catalogueType == FAVOURITES) {
        for(NewPlace *newPlace in self.catalogueViewController.newsArray) {
            if(newPlace.placeId.intValue == self.locationId.intValue) {
                if(followed == 0) {
                    newPlace.placeIsFollowed = NO;
                } else {
                    newPlace.placeIsFollowed = YES;
                }
                break;
            }
        }
    } else if(self.catalogueViewController.catalogueType == PROMOTIONS) {
        for(Promotion *promotion in self.catalogueViewController.promotionsArray) {
            if(promotion.promotionId.intValue == self.locationId.intValue) {
                promotion.promotionFollowed = [NSNumber numberWithInt:followed];
                break;
            }
        }
    } else if(self.catalogueViewController.catalogueType == SHOPPING_CENTERS) {
        for(ShoppingCenter *shoppingCenter in self.catalogueViewController.shoppingCentersArray) {
            if(shoppingCenter.shoppingCenterId.intValue == self.locationId.intValue) {
                shoppingCenter.shoppingCenterFollowed = [NSNumber numberWithInt:followed];
                break;
            }
        }
    }
}

- (void)animationDidStop:(CAAnimation *)theAnimation finished:(BOOL)flag {
    if(self.followed) {
        [self.catalogueFollowButton setImage:[UIImage imageNamed:@"remove_from_favs"] forState:UIControlStateNormal];
    } else {
        [self.catalogueFollowButton setImage:[UIImage imageNamed:@"add_to_favs"] forState:UIControlStateNormal];
    }
}

-(void)followAnimation:(UIButton *)button {
    
    CABasicAnimation *rotate = [CABasicAnimation animationWithKeyPath:@"transform.rotation"];
    rotate.fromValue = [NSNumber numberWithFloat:0];
    rotate.toValue = [NSNumber numberWithFloat:-6.5];
    rotate.duration = DURATION;
    rotate.repeatCount = 1;
    CABasicAnimation *zoomOut = [CABasicAnimation animationWithKeyPath:@"transform.scale"];
    zoomOut.fromValue = [NSNumber numberWithFloat:1];
    zoomOut.toValue = [NSNumber numberWithFloat:0.2];
    zoomOut.duration = DURATION / 2;
    zoomOut.repeatCount = 1;
    CABasicAnimation *zoomIn = [CABasicAnimation animationWithKeyPath:@"transform.scale"];
    zoomIn.fromValue = [NSNumber numberWithFloat:0.2];
    zoomIn.toValue = [NSNumber numberWithFloat:1];
    zoomIn.duration = DURATION / 4;
    zoomIn.beginTime = CACurrentMediaTime() + DURATION * 3 / 4;
    zoomIn.repeatCount = 1;
    CABasicAnimation *fadeOut = [CABasicAnimation animationWithKeyPath:@"opacity"];
    fadeOut.delegate = self;
    fadeOut.fromValue = [NSNumber numberWithFloat:1.0];
    fadeOut.toValue = [NSNumber numberWithFloat:0.0];
    fadeOut.duration = DURATION / 4;
    fadeOut.repeatCount = 1;
    CABasicAnimation *continueFade = [CABasicAnimation animationWithKeyPath:@"opacity"];
    continueFade.fromValue = [NSNumber numberWithFloat:0];
    continueFade.toValue = [NSNumber numberWithFloat:0.0];
    continueFade.beginTime = CACurrentMediaTime() + DURATION / 4;
    continueFade.duration = DURATION / 2;
    continueFade.repeatCount = 1;
    
    [button.layer addAnimation:rotate forKey:nil];
    [button.layer addAnimation:zoomOut forKey:nil];
    [button.layer addAnimation:zoomIn forKey:nil];
    [button.layer addAnimation:fadeOut forKey:nil];
    [button.layer addAnimation:continueFade forKey:nil];
}

-(void)unfollowAnimation:(UIButton *)button {
    CABasicAnimation *zoomOut = [CABasicAnimation animationWithKeyPath:@"transform.scale"];
    zoomOut.delegate = self;
    zoomOut.fromValue = [NSNumber numberWithFloat:1];
    zoomOut.toValue = [NSNumber numberWithFloat:0.05];
    zoomOut.duration = DURATION / 2;
    zoomOut.repeatCount = 1;
    CABasicAnimation *zoomIn = [CABasicAnimation animationWithKeyPath:@"transform.scale"];
    zoomIn.fromValue = [NSNumber numberWithFloat:0.05];
    zoomIn.toValue = [NSNumber numberWithFloat:1];
    zoomIn.duration = DURATION / 2  + DURATION / 10;
    zoomIn.beginTime = CACurrentMediaTime() + DURATION / 2 - DURATION / 10;
    zoomIn.repeatCount = 1;
    
    [button.layer addAnimation:zoomOut forKey:nil];
    [button.layer addAnimation:zoomIn forKey:nil];
}


@end
