//
//  CatalogueViewController.m
//  plus
//
//  Created by Виктор Георгиев on 7/18/16.
//  Copyright © 2016 Outsourcing IT Services Ltd. All rights reserved.
//

#define NEW 0
#define PROMOTIONS 1
#define FAVOURITES 2
#define SHOPPING_CENTERS 3

#import "CatalogueViewController.h"
#import "PlaceViewController.h"
#import "ShoppingCenterViewController.h"
#import "SingleBrandViewController.h"

#import "CatalogueCollectionViewCell.h"

#import "ShoppingCenter.h"
#import "Brand.h"
#import "NewPlace.h"
#import "Promotion.h"
#import "ItemCategory.h"
#import "AnimationTransitioning.h"

#import "URLRequests.h"
#import "Utils.h"

@interface CatalogueViewController ()

@property (strong, nonatomic) NSNumber *placeId;
@property (strong, nonatomic) NSNumber *brandId;
@property (strong, nonatomic) NSNumber *shoppingCenterId;

@property (strong, nonatomic) NSString *headerString;

@property (weak, nonatomic) IBOutlet UILabel *noResultsLabel;

@property (assign, nonatomic) BOOL initialLaunch;

@property (assign, nonatomic) CGFloat osVersion;

@end

@implementation CatalogueViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    
    self.osVersion = [[UIDevice currentDevice].systemVersion floatValue];

    self.initialLaunch = YES;
    
    self.navigationController.delegate = self;
}

-(void)viewDidAppear:(BOOL)animated {
    [super viewDidAppear:animated];
    
    if(self.initialLaunch) {
        if(self.catalogueType == NEW) {
            [URLRequests getNew:^(NSArray<NewPlace *> *newPlacesArray) {
                self.newsArray = newPlacesArray;
                if(self.newsArray.count == 0) {
                    self.noResultsLabel.text = [Utils translate:@"No results"];
                    self.noResultsLabel.hidden = NO;
                }
                [self.catalogueCollectionView reloadData];
            }];
        } else if(self.catalogueType == PROMOTIONS) {
            [URLRequests getPromotions:[[Utils getLocation] coordinate] page:0 receive:^(NSDictionary *mainDict, NSArray *items) {
                self.promotionsArray = (NSArray<Promotion *> *)items;
                if(self.promotionsArray.count == 0) {
                    self.noResultsLabel.text = [Utils translate:@"No results"];
                    self.noResultsLabel.hidden = NO;
                }
                [self.catalogueCollectionView reloadData];
            }];
        } else if(self.catalogueType == FAVOURITES) {
            [URLRequests getFollowed:^(NSArray<NewPlace *> *items) {
                self.newsArray = items;
                if(self.newsArray.count == 0) {
                    self.noResultsLabel.text = [Utils translate:@"No results"];
                    self.noResultsLabel.hidden = NO;
                }
                [self.catalogueCollectionView reloadData];
            }];
        } else if(self.catalogueType == SHOPPING_CENTERS) {
            [URLRequests getShoppingCenters:[[Utils getLocation] coordinate] page:0 receive:^(NSDictionary *mainDict, NSArray *items) {
                self.shoppingCentersArray = (NSArray<ShoppingCenter *> *)items;
                if(self.shoppingCentersArray.count == 0) {
                    self.noResultsLabel.text = [Utils translate:@"No results"];
                    self.noResultsLabel.hidden = NO;
                }
                [self.catalogueCollectionView reloadData];
            }];
        }
        self.initialLaunch = NO;
    }
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
}

- (NSInteger)collectionView:(UICollectionView *)collectionView numberOfItemsInSection:(NSInteger)section {
    if(self.catalogueType == NEW || self.catalogueType == FAVOURITES) {
        return self.newsArray.count;
    } else if(self.catalogueType == PROMOTIONS) {
        return self.promotionsArray.count;
    } else if(self.catalogueType == SHOPPING_CENTERS) {
        return self.shoppingCentersArray.count;
    }
    return 0;
}

- (CGSize)collectionView:(UICollectionView *)collectionView layout:(UICollectionViewLayout *)collectionViewLayout sizeForItemAtIndexPath:(NSIndexPath *)indexPath {
    CGRect screenBounds = [[UIScreen mainScreen] bounds];
    CGSize size = CGSizeMake((screenBounds.size.width - 45) / 2, 177);
    return size;
}

- (UICollectionViewCell *)collectionView:(UICollectionView *)collectionView cellForItemAtIndexPath:(NSIndexPath *)indexPath {
    CatalogueCollectionViewCell *cell = (CatalogueCollectionViewCell *)[collectionView dequeueReusableCellWithReuseIdentifier:@"Cell" forIndexPath:indexPath];
    cell.catalogueViewController = self;
    if(self.catalogueType == NEW || self.catalogueType == FAVOURITES) {
        NewPlace *newPlace = (NewPlace *)[self.newsArray objectAtIndex:indexPath.row];
        if(newPlace.placeImage != nil && ![newPlace.placeImage isEqualToString:@""] && ![newPlace.placeImage isEqualToString:@"null"]) {
            cell.catalogueImageView.image = nil;
            cell.catalogueImageView.backgroundColor = [UIColor clearColor];
            [URLRequests loadImage:newPlace.placeImage withFolder:nil receive:^(UIImage *image) {
                for(NSIndexPath *indexPathForImage in [collectionView indexPathsForVisibleItems]) {
                    if([[self.newsArray objectAtIndex:indexPathForImage.item].placeImage isEqualToString:newPlace.placeImage]) {
                        ((CatalogueCollectionViewCell *)[collectionView cellForItemAtIndexPath:indexPathForImage]).catalogueImageView.image = image;
                        if(![newPlace.placeBackgroundColor isKindOfClass:[NSNull class]] && ![newPlace.placeBackgroundColor isEqualToString:@""] && ![newPlace.placeBackgroundColor isEqualToString:@"null"]) {
                            cell.catalogueImageView.backgroundColor = [Utils colorFromHexString:[NSString stringWithFormat:@"#%@", newPlace.placeBackgroundColor]];
                        } else {
                            cell.catalogueImageView.backgroundColor = [UIColor clearColor];
                        }
                    }
                }
            }];
        } else {
            //cell.catalogueHeaderLabel.text = newPlace.placeName;
            cell.catalogueHeaderLabel.hidden = YES;
            //cell.catalogueNameHeightConstraint.constant = 0;
            //cell.catalogueNameVerticalConstraint.constant = 0;
            cell.catalogueImageView.image = [UIImage imageNamed:@"no-logo"];
        }
//        if(newPlace.place.intValue == 1) {
//            cell.cataloguePromoLabel.text = [Utils translate:@"Promo"];
//            cell.cataloguePromoView.hidden = NO;
//        }
        
        if(newPlace.placeIsNewBrand) {
            cell.catalogueNewLabel.text = [Utils translate:@"New brand"];
            cell.catalogueNewView.hidden = NO;
        } else if(newPlace.placeIsNewLocation) {
            cell.catalogueNewLabel.text = [Utils translate:@"New location"];
            cell.catalogueNewView.hidden = NO;
        } else {
            cell.catalogueNewView.hidden = YES;
        }
        
        cell.locationId = newPlace.placeId;
        if(newPlace.placeIsFollowed || self.catalogueType == FAVOURITES) {
            [cell.catalogueFollowButton setImage:[UIImage imageNamed:@"remove_from_favs"] forState:UIControlStateNormal];
            cell.followed = 1;
        } else {
            [cell.catalogueFollowButton setImage:[UIImage imageNamed:@"add_to_favs"] forState:UIControlStateNormal];
            cell.followed = 0;
        }
        
        cell.catalogueNameLabel.text = newPlace.placeName;
        NSArray<ItemCategory *> *categoriesArray = [Utils getCategories];
        for(ItemCategory *itemCategory in categoriesArray) {
            if(newPlace.placeCategoryId.intValue == itemCategory.itemId.intValue) {
                cell.catalogueCategoryLabel.text = itemCategory.itemName;
                break;
            } else {
                BOOL found = NO;
                for(ItemCategory *itemSubcategiory in itemCategory.itemSubcategories) {
                    if(newPlace.placeCategoryId.intValue == itemSubcategiory.itemId.intValue) {
                        cell.catalogueCategoryLabel.text = itemSubcategiory.itemName;
                        found = YES;
                        break;
                    }
                }
                if(found) {
                    break;
                }
            }
        }
        
        if(newPlace.placeIsBrand) {
            cell.isBrand = YES;
            cell.cataloguePlacesCountLabel.text = [NSString stringWithFormat:@"%d", newPlace.placeCount.intValue];
            cell.cataloguePlacesCountLabel.hidden = NO;
            cell.cataloguePlacesCountImageView.hidden = NO;
        } else {
            cell.isBrand = NO;
            cell.cataloguePlacesCountLabel.hidden = YES;
            cell.cataloguePlacesCountImageView.hidden = YES;
        }

        CLLocation *newPlaceLocation = [[CLLocation alloc] initWithLatitude:newPlace.placeLatitude.floatValue longitude:newPlace.placeLongitude.floatValue];
        [self setDistance:[[Utils getLocation] distanceFromLocation:newPlaceLocation] / 1000 forCell:cell];
    } else if(self.catalogueType == PROMOTIONS) {
        Promotion *promotion = (Promotion *) [self.promotionsArray objectAtIndex:indexPath.item];
        NSString *imageUrl = [promotion.promotionLogos objectForKey:@"image"];
        if(imageUrl != nil && ![imageUrl isEqualToString:@""] && ![imageUrl isEqualToString:@"null"]) {
            cell.catalogueImageView.image = nil;
            cell.catalogueImageView.backgroundColor = [UIColor clearColor];
            [URLRequests loadImage:imageUrl withFolder:nil receive:^(UIImage *image) {
                for(NSIndexPath *indexPathForImage in [collectionView indexPathsForVisibleItems]) {
                    if([[[self.promotionsArray objectAtIndex:indexPathForImage.item].promotionLogos objectForKey:@"image"] isEqualToString:imageUrl]) {
                        ((CatalogueCollectionViewCell *)[collectionView cellForItemAtIndexPath:indexPathForImage]).catalogueImageView.image = image;
                        if(![promotion.promotionBackgroundColor isKindOfClass:[NSNull class]] && ![promotion.promotionBackgroundColor isEqualToString:@""] && ![promotion.promotionBackgroundColor isEqualToString:@"null"]) {
                            cell.catalogueImageView.backgroundColor = [Utils colorFromHexString:[NSString stringWithFormat:@"#%@", promotion.promotionBackgroundColor]];
                        } else {
                            cell.catalogueImageView.backgroundColor = [UIColor clearColor];
                        }
                    }
                }
            }];
        } else {
            //cell.catalogueHeaderLabel.text = promotion.promotionName;
            cell.catalogueHeaderLabel.hidden = YES;
            //cell.catalogueNameHeightConstraint.constant = 0;
            //cell.catalogueNameVerticalConstraint.constant = 0;
            cell.catalogueImageView.image = [UIImage imageNamed:@"no-logo"];
            cell.catalogueImageView.hidden = NO;
        }
        
        cell.locationId = promotion.promotionId;
        if(promotion.promotionFollowed.intValue == 1) {
            [cell.catalogueFollowButton setImage:[UIImage imageNamed:@"remove_from_favs"] forState:UIControlStateNormal];
            cell.followed = 1;
        } else {
            [cell.catalogueFollowButton setImage:[UIImage imageNamed:@"add_to_favs"] forState:UIControlStateNormal];
            cell.followed = 0;
        }
        
        if(promotion.promotionHasPromotion.intValue == 1) {
            cell.cataloguePromoLabel.text = [Utils translate:@"Promo"];
            cell.cataloguePromoView.hidden = NO;
        }
        
        cell.isBrand = NO;
        cell.catalogueNameLabel.text = promotion.promotionName;
        cell.catalogueCategoryLabel.text = [[promotion.promotionCategories objectAtIndex:0] objectForKey:@"name"];
        
        [self setDistance:promotion.promotionDistance.doubleValue forCell:cell];
    } else if(self.catalogueType == SHOPPING_CENTERS) {
        ShoppingCenter *shoppingCenter = (ShoppingCenter *) [self.shoppingCentersArray objectAtIndex:indexPath.item];
        NSString *imageUrl = [shoppingCenter.shoppingCenterLogos objectForKey:@"image"];
        if(imageUrl != nil && ![imageUrl isEqualToString:@""]) {
            cell.catalogueImageView.image = nil;
            cell.catalogueImageView.backgroundColor = [UIColor clearColor];
            [URLRequests loadImage:imageUrl withFolder:nil receive:^(UIImage *image) {
                for(NSIndexPath *indexPathForImage in [collectionView indexPathsForVisibleItems]) {
                    if([[[self.shoppingCentersArray objectAtIndex:indexPathForImage.item].shoppingCenterLogos objectForKey:@"image"] isEqualToString:imageUrl]) {
                        ((CatalogueCollectionViewCell *)[collectionView cellForItemAtIndexPath:indexPathForImage]).catalogueImageView.image = image;
                        if(![shoppingCenter.shoppingCenterBackgroundColor isKindOfClass:[NSNull class]] && ![shoppingCenter.shoppingCenterBackgroundColor isEqualToString:@""] && ![shoppingCenter.shoppingCenterBackgroundColor isEqualToString:@"null"]) {
                            cell.catalogueImageView.backgroundColor = [Utils colorFromHexString:[NSString stringWithFormat:@"#%@", shoppingCenter.shoppingCenterBackgroundColor]];
                        } else {
                            cell.catalogueImageView.backgroundColor = [UIColor clearColor];
                        }
                    }
                }
            }];
        } else {
            //cell.catalogueHeaderLabel.text = shoppingCenter.shoppingCenterName;
            cell.catalogueHeaderLabel.hidden = YES;
            //cell.catalogueNameHeightConstraint.constant = 0;
            //cell.catalogueNameVerticalConstraint.constant = 0;
            cell.catalogueImageView.image = [UIImage imageNamed:@"no-logo"];
            cell.catalogueImageView.hidden = NO;
        }
        
        cell.locationId = shoppingCenter.shoppingCenterId;
        if(shoppingCenter.shoppingCenterFollowed.intValue == 1) {
            [cell.catalogueFollowButton setImage:[UIImage imageNamed:@"remove_from_favs"] forState:UIControlStateNormal];
            cell.followed = 1;
        } else {
            [cell.catalogueFollowButton setImage:[UIImage imageNamed:@"add_to_favs"] forState:UIControlStateNormal];
            cell.followed = 0;
        }
        
        cell.isBrand = NO;
        if(shoppingCenter.shoppingCenterHasPromotion.intValue == 1) {
            cell.cataloguePromoLabel.text = [Utils translate:@"Promo"];
            cell.cataloguePromoView.hidden = NO;
        }
        
        cell.catalogueNameLabel.text = shoppingCenter.shoppingCenterName;
        if(shoppingCenter.shoppingCenterCategories.count > 0) {
            cell.catalogueCategoryLabel.text = [[shoppingCenter.shoppingCenterCategories objectAtIndex:0] objectForKey:@"name"];
        }
        cell.catalogueImageViewHeightConstraint.constant = 124;
        
        cell.cataloguePlacesCountLabel.text = [NSString stringWithFormat:@"%d", shoppingCenter.shoppingCenterPlacesCount.intValue];
        cell.cataloguePlacesCountLabel.hidden = NO;
        cell.cataloguePlacesCountImageView.hidden = NO;

        [self setDistance:shoppingCenter.shoppingCenterDistance.doubleValue forCell:cell];
    }
    
    return cell;
}

- (void)collectionView:(UICollectionView *)collectionView willDisplayCell:(UICollectionViewCell *)cell forItemAtIndexPath:(NSIndexPath *)indexPath {
    CatalogueCollectionViewCell *catalogueCell = (CatalogueCollectionViewCell *)cell;
    if(self.catalogueType == NEW || self.catalogueType == FAVOURITES) {
        NewPlace *newPlace = [self.newsArray objectAtIndex:indexPath.item];
        UIImage *image = [URLRequests getImage:newPlace.placeImage];
        if(image != nil) {
            catalogueCell.catalogueImageView.image = image;
        }
        catalogueCell.catalogueImageView.backgroundColor = [self checkForBackgroundColor:newPlace.placeBackgroundColor];
    } else if(self.catalogueType == PROMOTIONS) {
        Promotion *promotion = [self.promotionsArray objectAtIndex:indexPath.item];
        UIImage *image = [URLRequests getImage:[promotion.promotionLogos objectForKey:@"image"]];
        if(image != nil) {
            catalogueCell.catalogueImageView.image = image;
        }
        catalogueCell.catalogueImageView.backgroundColor = [self checkForBackgroundColor:promotion.promotionBackgroundColor];
    } else if(self.catalogueType == SHOPPING_CENTERS) {
        ShoppingCenter *shoppingCenter = [self.shoppingCentersArray objectAtIndex:indexPath.item];
        UIImage *image = [URLRequests getImage:[shoppingCenter.shoppingCenterLogos objectForKey:@"image"]];
        if(image != nil) {
            catalogueCell.catalogueImageView.image = image;
        }
        catalogueCell.catalogueImageView.backgroundColor = [self checkForBackgroundColor:shoppingCenter.shoppingCenterBackgroundColor];
    }
}

- (void)collectionView:(UICollectionView *)collectionView didSelectItemAtIndexPath:(NSIndexPath *)indexPath {
    if(self.catalogueType == NEW) {
        NewPlace *newPlace = (NewPlace *)[self.newsArray objectAtIndex:indexPath.row];
        self.headerString = newPlace.placeName;
        if(newPlace.placeIsBrand) {
            self.brandId = newPlace.placeId;
            [self performSegueWithIdentifier:@"BrandSegue" sender:self];
        } else {
            self.placeId = newPlace.placeId;
            [self performSegueWithIdentifier:@"PlaceSegue" sender:self];
        }
    } else if(self.catalogueType == PROMOTIONS) {
        Promotion *promotion = [self.promotionsArray objectAtIndex:indexPath.item];
        self.headerString = promotion.promotionName;
        if(promotion.promotionIsMall) {
            self.shoppingCenterId = promotion.promotionId;
            [self performSegueWithIdentifier:@"ShoppingCenterSegue" sender:self];
        } else {
            self.placeId = promotion.promotionId;
            [self performSegueWithIdentifier:@"PlaceSegue" sender:self];
        }
    } else if(self.catalogueType == FAVOURITES) {
        NewPlace *newPlace = (NewPlace *)[self.newsArray objectAtIndex:indexPath.row];
        self.headerString = newPlace.placeName;
        if(newPlace.placeIsBrand) {
            self.brandId = newPlace.placeId;
            [self performSegueWithIdentifier:@"BrandSegue" sender:self];
        } else {
            self.placeId = newPlace.placeId;
            [self performSegueWithIdentifier:@"PlaceSegue" sender:self];
        }
    } else if(self.catalogueType == SHOPPING_CENTERS) {
        ShoppingCenter *shoppingCenter = [self.shoppingCentersArray objectAtIndex:indexPath.row];
        self.headerString = shoppingCenter.shoppingCenterName;
        self.shoppingCenterId = shoppingCenter.shoppingCenterId;
        self.headerString = shoppingCenter.shoppingCenterName;
        [self performSegueWithIdentifier:@"ShoppingCenterSegue" sender:self];
    }
}

-(void)setDistance:(double)distance forCell:(CatalogueCollectionViewCell *)cell {
    if(distance < 1) {
        cell.catalogueDistanceLabel.text = [NSString stringWithFormat:@"%d %@", (int)(distance * 1000), [Utils translate:@"Meter"]];
    } else {
        cell.catalogueDistanceLabel.text = [NSString stringWithFormat:@"%.2f %@", distance, [Utils translate:@"Kilometer"]];
    }
    CGFloat labelWidth = [cell.catalogueDistanceLabel.text boundingRectWithSize:CGSizeMake(CGFLOAT_MAX, 20) options:NSStringDrawingUsesLineFragmentOrigin attributes: @{NSFontAttributeName : cell.catalogueDistanceLabel.font} context:nil].size.width;
    cell.catalogueDistanceLabelWidthConstraint.constant = labelWidth;
}

- (UIColor *)checkForBackgroundColor:(NSString *)colorString {
    if(![colorString isKindOfClass:[NSNull class]] && ![colorString isEqualToString:@""] && ![colorString isEqualToString:@"null"]) {
        return [Utils colorFromHexString:[NSString stringWithFormat:@"#%@", colorString]];
    } else {
        return [Utils colorFromHexString:@"#E6E7E8"];
    }
}

-(id<UIViewControllerAnimatedTransitioning>) navigationController:(UINavigationController *)navigationController animationControllerForOperation:(UINavigationControllerOperation)operation fromViewController:(UIViewController *)fromVC toViewController:(UIViewController *)toVC {
    AnimationTransitioning *animationTransitioning = [[AnimationTransitioning alloc] init];
    if(operation == UINavigationControllerOperationPush) {
        if([toVC isKindOfClass:[SingleBrandViewController class]]) {
            animationTransitioning.animationOption = UIViewAnimationOptionTransitionCrossDissolve;
        } else if([toVC isKindOfClass:[PlaceViewController class]]) {
            animationTransitioning.animationOption = UIViewAnimationOptionTransitionCrossDissolve;
        } else if([toVC isKindOfClass:[ShoppingCenterViewController class]]) {
            animationTransitioning.animationOption = UIViewAnimationOptionTransitionCrossDissolve;
        }
    } else if(operation == UINavigationControllerOperationPop) {
        if([fromVC isKindOfClass:[SingleBrandViewController class]]) {
            animationTransitioning.animationOption = UIViewAnimationOptionTransitionCrossDissolve;
        } else if([fromVC isKindOfClass:[PlaceViewController class]]) {
            animationTransitioning.animationOption = UIViewAnimationOptionTransitionCrossDissolve;
        } else if([fromVC isKindOfClass:[ShoppingCenterViewController class]]) {
            animationTransitioning.animationOption = UIViewAnimationOptionTransitionCrossDissolve;
        }
    }
    return animationTransitioning;
}

#pragma mark - Navigation

- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    if([[segue destinationViewController] isKindOfClass:[PlaceViewController class]]) {
        PlaceViewController *placeViewController = (PlaceViewController *)[segue destinationViewController];
        placeViewController.placeId = self.placeId;
        placeViewController.headerTitle = @"";
        placeViewController.headerFieldType = @"";
        placeViewController.isFromMainDrawer = YES;
        self.catalogueControlViewController.headerLabel.text = self.headerString;
    } else if([[segue destinationViewController] isKindOfClass:[ShoppingCenterViewController class]]) {
        ((ShoppingCenterViewController *)[segue destinationViewController]).shoppingCenterId = self.shoppingCenterId;
        self.catalogueControlViewController.headerLabel.text = self.headerString;
    } else if([[segue destinationViewController] isKindOfClass:[SingleBrandViewController class]]) {
        SingleBrandViewController *singleBrandViewController = (SingleBrandViewController *)[segue destinationViewController];
        singleBrandViewController.headerFieldType = @"search";
        singleBrandViewController.brandId = self.brandId;
        singleBrandViewController.isFromMainDrawer = YES;
        self.catalogueControlViewController.headerLabel.text = self.headerString;
    }
}


@end
