//
//  CategoriesTableViewController.m
//  plus
//
//  Created by Виктор Георгиев on 7/12/16.
//  Copyright © 2016 Outsourcing IT Services Ltd. All rights reserved.
//

#define DURATION 0.3f

#import "AppDelegate.h"

#import "MainMenuViewController.h"
#import "CategoriesViewController.h"
#import "SubcategoriesViewController.h"

#import "CategoriesTableViewCell.h"

#import "URLRequests.h"
#import "AnimationTransitioning.h"
#import "Utils.h"
#import "Filter.h"

@interface CategoriesViewController ()

@property (strong, nonatomic) NSUserDefaults *userDefaults;

@property (strong, nonatomic) NSManagedObjectContext *managedObjectContext;

@property (weak, nonatomic) IBOutlet UIView *selectAllView;

@property (weak, nonatomic) IBOutlet UILabel *selectAllLabel;
@property (weak, nonatomic) IBOutlet UILabel *clearAllLabel;

@property (weak, nonatomic) IBOutlet UIImageView *selectAllImageView;
@property (weak, nonatomic) IBOutlet UIImageView *selectAllCheckImageView;

@property (strong, nonatomic) NSArray<ItemCategory *> *bgCategoriesArray;
@property (strong, nonatomic) NSArray<ItemCategory *> *enCategoriesArray;

@property (strong, nonatomic) IBOutlet UITapGestureRecognizer *selectAllGestureRecognizer;
@property (strong, nonatomic) IBOutlet UITapGestureRecognizer *removeAllGestureRecognizer;


@property (weak, nonatomic) IBOutlet UIButton *showResultsButton;

@property (assign, nonatomic) BOOL hasToLoad;
@property (assign, nonatomic) BOOL isAllSelected;
@property (assign, nonatomic) BOOL changedHeaderLabel;

@end

@implementation CategoriesViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    self.userDefaults = [NSUserDefaults standardUserDefaults];
    self.managedObjectContext = ((AppDelegate *)[UIApplication sharedApplication].delegate).managedObjectContext;
    self.hasToLoad = NO;
    self.isAllSelected = NO;
    self.changedHeaderLabel = NO;
    [self checkSelected];
    [self getCategories];
    
    self.navigationController.delegate = self;
    [self changeLabels];
    
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(changeLanguage) name:@"LanguageNotification" object:nil];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
}

-(void)getCategories {
    NSString *language = [Utils currentLanguage];
    
    [URLRequests getCategories:@"bg" receive:^(NSArray *categories) {
        self.bgCategoriesArray = categories;
        [self saveSubCategories:@"bg"];
        [self didLoad];
        if([language isEqualToString:@"bg"]) {
            self.categoriesArray = self.bgCategoriesArray;
            [Utils setCategories:self.categoriesArray];
            [self.categoriesTableView reloadData];
        }
    }];
    
    [URLRequests getCategories:@"en" receive:^(NSArray *categories) {
        self.enCategoriesArray = categories;
        [self saveSubCategories:@"en"];
        [self didLoad];
        if([language isEqualToString:@"en"]) {
            self.categoriesArray = self.enCategoriesArray;
            [Utils setCategories:self.categoriesArray];
            [self.categoriesTableView reloadData];
        }
    }];
}

- (void)saveSubCategories:(NSString *)language {
    NSError *error;
    NSFetchRequest *request = [NSFetchRequest fetchRequestWithEntityName:@"Filter"];
    NSArray<ItemCategory *> *checkArray;
    if([language isEqualToString:@"bg"]) {
        checkArray = self.bgCategoriesArray;
    } else {
        checkArray = self.enCategoriesArray;
    }
    NSMutableArray<NSString *> *parentArray = [[NSMutableArray alloc] initWithCapacity:0];
    for(ItemCategory *itemCategory in checkArray) {
        [parentArray addObject:[NSString stringWithFormat:@"%d", itemCategory.itemId.intValue]];
        request.predicate = [NSPredicate predicateWithFormat:@"parentId == %@ && filterId == %d", itemCategory.itemId, -1];
        NSArray<Filter *> *filtersArray = (NSArray<Filter *> *)[self.managedObjectContext executeFetchRequest:request error:&error];
        if(filtersArray.count == 0) {
            [URLRequests loadImage:itemCategory.itemMapIcon withFolder:nil receive:nil];
            [URLRequests loadImage:itemCategory.itemCategoryPinIcon withFolder:nil receive:nil];
            Filter *filter = [NSEntityDescription insertNewObjectForEntityForName:@"Filter" inManagedObjectContext:self.managedObjectContext];
            filter.parentId = itemCategory.itemId;
            filter.filterId = [NSNumber numberWithInt:-1];
            filter.filterIsSelected = @(NO);
            if([language isEqualToString:@"bg"]) {
                filter.filterBgName = itemCategory.itemName;
                if(filter.filterEnName == nil) {
                    filter.filterEnName = @"";
                }
            } else {
                filter.filterEnName = itemCategory.itemName;
                if(filter.filterBgName == nil) {
                    filter.filterBgName = @"";
                }
            }
        } else if(filtersArray.count == 1) {
            Filter *filter = [filtersArray objectAtIndex:0];
            if([language isEqualToString:@"bg"]) {
                filter.filterBgName = itemCategory.itemName;
            } else {
                filter.filterEnName = itemCategory.itemName;
            }
        }
        
        for(ItemCategory *subItemCategory in itemCategory.itemSubcategories) {
            [URLRequests loadImage:subItemCategory.itemMapIcon withFolder:nil receive:nil];
            [URLRequests loadImage:subItemCategory.itemCategoryPinIcon withFolder:nil receive:nil];
            
            request.predicate = [NSPredicate predicateWithFormat:@"filterId == %@", subItemCategory.itemId];
            NSArray<Filter *> *filtersArray = (NSArray<Filter *> *)[self.managedObjectContext executeFetchRequest:request error:&error];
            if(filtersArray.count == 0) {
                Filter *filter = [NSEntityDescription insertNewObjectForEntityForName:@"Filter" inManagedObjectContext:self.managedObjectContext];
                filter.parentId = itemCategory.itemId;
                filter.filterId = subItemCategory.itemId;
                if([language isEqualToString:@"bg"]) {
                    filter.filterBgName = subItemCategory.itemName;
                    if(filter.filterEnName == nil) {
                        filter.filterEnName = @"";
                    }
                } else {
                    filter.filterEnName = subItemCategory.itemName;
                    if(filter.filterBgName == nil) {
                        filter.filterBgName = @"";
                    }
                }
                filter.filterIsSelected = @(NO);
            } else if(filtersArray.count == 1) {
                Filter *filter = [filtersArray objectAtIndex:0];
                if([language isEqualToString:@"bg"]) {
                    filter.filterBgName = subItemCategory.itemName;
                } else {
                    filter.filterEnName = subItemCategory.itemName;
                }
            }
            [((AppDelegate *)[UIApplication sharedApplication].delegate) saveContext];
        }
    }
    NSString *filterString = [parentArray componentsJoinedByString:@","];
    [self.userDefaults setValue:filterString forKey:@"parentFilter"];
    [self.userDefaults synchronize];
    [((MainMenuViewController *)self.mainNavigationController.viewControllers.firstObject).brandsViewController.brandsCollectionView reloadData];
}

- (void)didLoad {
    if(!self.hasToLoad) {
        self.hasToLoad = YES;
    } else {
        if(self.bgCategoriesArray.count == self.enCategoriesArray.count) {
            NSUInteger maxLength = 0;
            for(int i = 0; i < self.bgCategoriesArray.count; i++) {
                NSUInteger bgLength = [self.bgCategoriesArray objectAtIndex:i].itemName.length;
                NSUInteger enLength = [self.enCategoriesArray objectAtIndex:i].itemName.length;
                if(bgLength >= maxLength) {
                    maxLength = bgLength;
                }
                if(enLength >= maxLength) {
                    maxLength = enLength;
                }
            }
            self.onLoad(maxLength);
        }
    }
}

- (void)changeLabels {
    self.selectAllLabel.text = [Utils translate:@"Select all"];
    self.clearAllLabel.text = [Utils translate:@"Clear all"];    
    [self.showResultsButton setTitle:[Utils translate:@"Show Results"] forState:UIControlStateNormal];
}

- (void)changeLanguage {
    NSString *language = [Utils currentLanguage];
    if([language isEqualToString:@"bg"]) {
        self.categoriesArray = self.bgCategoriesArray;
    } else if([language isEqualToString:@"en"]) {
        self.categoriesArray = self.enCategoriesArray;
    }
    
    [Utils setCategories:self.categoriesArray];
    [self.categoriesTableView reloadData];
    [self changeLabels];
}

#pragma mark - Table view data source

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return self.categoriesArray.count;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    CategoriesTableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"Cell" forIndexPath:indexPath];
    cell.selectionStyle = UITableViewCellSelectionStyleNone;
    ItemCategory *itemCategory = ((ItemCategory *)[self.categoriesArray objectAtIndex:indexPath.row]);
    cell.categoryLabel.text = itemCategory.itemName;
    NSFetchRequest *request = [NSFetchRequest fetchRequestWithEntityName:@"Filter"];
    request.predicate = [NSPredicate predicateWithFormat:@"parentId == %@ && filterIsSelected == %@", itemCategory.itemId, @(YES)];
    NSError *error;
    CGFloat fromAlpha;
    CGFloat toAlpha;
    NSArray<Filter *> *filtersArray = (NSArray<Filter *> *)[self.managedObjectContext executeFetchRequest:request error:&error];
    if(filtersArray.count > 0) {
        cell.subcategorySelectedCountLabel.text = [NSString stringWithFormat:@"%lu %@ ", (unsigned long)filtersArray.count, [Utils translate:@"From"]];
        cell.subcategoryCountImageView.image = [UIImage imageNamed:@"check_blue"];
        fromAlpha = 0.0f;
        toAlpha = 1.0f;
    } else {
        fromAlpha = 1.0f;
        toAlpha = 0.0f;
    }
    cell.subcategoryCountLabel.text = [NSString stringWithFormat:@"%lu", (unsigned long)itemCategory.itemSubcategories.count];
    
    cell.subcategoryCountImageView.alpha = fromAlpha;
    cell.subcategorySelectedCountLabel.alpha = fromAlpha;
    
    [cell layoutIfNeeded];
    [UIView animateWithDuration:0.5 animations:^{
        cell.subcategoryCountImageView.alpha = toAlpha;
        cell.subcategorySelectedCountLabel.alpha = toAlpha;
        [cell layoutIfNeeded];
    } completion:^(BOOL finished) {
        if(toAlpha == 0) {
            cell.subcategorySelectedCountLabel.text = @"";
            cell.subcategoryCountImageView.image = nil;
        }
    }];
    
    NSString *itemCategoryIcon = itemCategory.itemCategoryIcon;
    itemCategoryIcon = [itemCategoryIcon stringByReplacingOccurrencesOfString:@" " withString:@"%20"];
    UIImage *image = [URLRequests loadImage:itemCategoryIcon withFolder:nil receive:^(UIImage *image) {
        for(NSIndexPath *indexPathForImage in [tableView indexPathsForVisibleRows]) {
            if([[self.categoriesArray objectAtIndex:indexPathForImage.row].itemCategoryIcon isEqualToString:itemCategoryIcon]) {
                ((CategoriesTableViewCell *)[tableView cellForRowAtIndexPath:indexPathForImage]).categoryImageView.image = image;
            }
        }
    }];
    cell.categoryImageView.image = image;

    return cell;
}

- (void)tableView:(UITableView *)tableView willDisplayCell:(UITableViewCell *)cell forRowAtIndexPath:(NSIndexPath *)indexPath {
    ItemCategory *itemCategory = (ItemCategory *) [self.categoriesArray objectAtIndex:indexPath.row];
    UIImage *image = [URLRequests getImage:itemCategory.itemCategoryIcon];
    if(image != nil) {
        ((CategoriesTableViewCell *)cell).categoryImageView.image = image;
    }
}

- (IBAction)onSelectAllTap:(id)sender {
    if(!self.isAllSelected) {
        self.isAllSelected = YES;
        if(!self.changedHeaderLabel) {
            [self selectAllAnimation];
            self.changedHeaderLabel = YES;
        }
        [self categoriesSelect:YES];
    }
}

- (IBAction)onClearAllTap:(id)sender {
    self.isAllSelected = NO;
    if(self.changedHeaderLabel) {
        [self deselectAllAnimation];
        self.changedHeaderLabel = NO;
    }
    [self categoriesSelect:NO];
}

- (void)animationDidStop:(CAAnimation *)theAnimation finished:(BOOL)flag {
    self.selectAllImageView.layer.anchorPoint = CGPointMake(0.5, 0.5);
    self.selectAllImageView.layer.position = CGPointMake(22 + self.selectAllImageView.frame.size.width / 2, self.selectAllImageView.superview.frame.size.height / 2 +  10);
    self.selectAllCheckImageView.layer.anchorPoint = CGPointMake(0.5, 0.5);
    self.selectAllCheckImageView.layer.position = CGPointMake(22 + self.selectAllImageView.frame.size.width / 2, self.selectAllImageView.superview.frame.size.height / 2 + 10);
}

- (void)selectAllAnimation {
    self.selectAllGestureRecognizer.enabled = NO;
    self.removeAllGestureRecognizer.enabled = NO;
    
    self.selectAllImageView.hidden = NO;
    self.selectAllCheckImageView.hidden = NO;
    
    self.selectAllImageView.layer.anchorPoint = CGPointMake(0.0, 0.5);
    self.selectAllImageView.layer.position = CGPointMake(22, self.selectAllView.frame.size.height / 2 + 10);
    
    CABasicAnimation *rotate = [CABasicAnimation animationWithKeyPath:@"transform.rotation"];
    rotate.delegate = self;
    rotate.fromValue = [self degreesToRadians:0];
    rotate.toValue = [self degreesToRadians:-45];
    rotate.duration = DURATION;
    rotate.repeatCount = 1;
    CABasicAnimation *fadeOut = [CABasicAnimation animationWithKeyPath:@"opacity"];
    fadeOut.fromValue = [NSNumber numberWithFloat:1];
    fadeOut.toValue = [NSNumber numberWithFloat:0];
    fadeOut.duration = DURATION;
    fadeOut.repeatCount = 1;
    [self.selectAllImageView.layer addAnimation:rotate forKey:@"rotate"];
    [self.selectAllImageView.layer addAnimation:fadeOut forKey:@"fadeOut"];
    
    self.selectAllCheckImageView.layer.anchorPoint = CGPointMake(0.0, 0.5);
    self.selectAllCheckImageView.layer.position = CGPointMake(22, self.selectAllView.frame.size.height / 2 + 10);
    
    CABasicAnimation *fadeIn = [CABasicAnimation animationWithKeyPath:@"opacity"];
    fadeIn.fromValue = [NSNumber numberWithFloat:0];
    fadeIn.toValue = [NSNumber numberWithFloat:1];
    fadeIn.duration = DURATION;
    fadeIn.repeatCount = 1;
    
    rotate.fromValue = [self degreesToRadians:45];
    rotate.toValue = [self degreesToRadians:0];
    rotate.duration = DURATION;
    rotate.repeatCount = 1;
    [self.selectAllCheckImageView.layer addAnimation:rotate forKey:@"rotate"];
    [self.selectAllCheckImageView.layer addAnimation:fadeIn forKey:@"fadeIn"];
    
    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, DURATION - 0.04 * NSEC_PER_SEC), dispatch_get_main_queue(), ^{
        self.selectAllImageView.hidden = YES;
    });
    
    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, DURATION * NSEC_PER_SEC), dispatch_get_main_queue(), ^{
        self.selectAllGestureRecognizer.enabled = YES;
        self.removeAllGestureRecognizer.enabled = YES;
    });
}

- (void)deselectAllAnimation {
    self.selectAllGestureRecognizer.enabled = NO;
    self.removeAllGestureRecognizer.enabled = NO;
    
    self.selectAllImageView.hidden = NO;
    self.selectAllCheckImageView.hidden = NO;
    
    self.selectAllImageView.layer.anchorPoint = CGPointMake(0.0, 0.5);
    self.selectAllImageView.layer.position = CGPointMake(22, self.selectAllImageView.superview.frame.size.height / 2 + 10);
    
    CABasicAnimation *rotate = [CABasicAnimation animationWithKeyPath:@"transform.rotation"];
    rotate.delegate = self;
    rotate.fromValue = [self degreesToRadians:-45];
    rotate.toValue = [self degreesToRadians:0];
    rotate.duration = DURATION;
    rotate.repeatCount = 1;
    CABasicAnimation *fadeOut = [CABasicAnimation animationWithKeyPath:@"opacity"];
    fadeOut.fromValue = [NSNumber numberWithFloat:0];
    fadeOut.toValue = [NSNumber numberWithFloat:1];
    fadeOut.duration = DURATION;
    fadeOut.repeatCount = 1;
    [self.selectAllImageView.layer addAnimation:rotate forKey:nil];
    [self.selectAllImageView.layer addAnimation:fadeOut forKey:nil];
    
    self.selectAllCheckImageView.layer.anchorPoint = CGPointMake(0.0, 0.5);
    self.selectAllCheckImageView.layer.position = CGPointMake(22, self.selectAllView.frame.size.height / 2 + 10);
    
    CABasicAnimation *fadeIn = [CABasicAnimation animationWithKeyPath:@"opacity"];
    fadeIn.fromValue = [NSNumber numberWithFloat:1];
    fadeIn.toValue = [NSNumber numberWithFloat:0];
    fadeIn.duration = DURATION;
    fadeIn.repeatCount = 1;
    
    rotate.fromValue = [self degreesToRadians:0];
    rotate.toValue = [self degreesToRadians:45];
    rotate.duration = DURATION;
    rotate.repeatCount = 1;
    [self.selectAllCheckImageView.layer addAnimation:rotate forKey:nil];
    [self.selectAllCheckImageView.layer addAnimation:fadeIn forKey:nil];
    
    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, DURATION - 0.04 * NSEC_PER_SEC), dispatch_get_main_queue(), ^{
        self.selectAllCheckImageView.hidden = YES;
    });
    
    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, DURATION * NSEC_PER_SEC), dispatch_get_main_queue(), ^{
        self.selectAllGestureRecognizer.enabled = YES;
        self.removeAllGestureRecognizer.enabled = YES;
    });
}

-(NSNumber *)degreesToRadians:(CGFloat)degrees {
    return [NSNumber numberWithFloat:degrees * M_PI / 180];
}

- (void)categoriesSelect:(BOOL)select {
    NSError *error;
    NSFetchRequest *request = [NSFetchRequest fetchRequestWithEntityName:@"Filter"];
    for(ItemCategory *itemCategory in self.categoriesArray) {
        for(ItemCategory *subItemCategory in itemCategory.itemSubcategories) {
            request.predicate = [NSPredicate predicateWithFormat:@"filterId == %@", subItemCategory.itemId];
            NSArray<Filter *> *filtersArray = (NSArray<Filter *> *)[self.managedObjectContext executeFetchRequest:request error:&error];
            Filter *filter = [filtersArray objectAtIndex:0];
            if(select) {
                filter.filterIsSelected = [NSNumber numberWithInt:1];
            } else {
                filter.filterIsSelected = [NSNumber numberWithInt:0];
            }
        }
    }
    
    [((AppDelegate *)[UIApplication sharedApplication].delegate) saveContext];
    [self.categoriesTableView reloadData];
}

- (void)checkSelected {
    NSError *error;
    NSFetchRequest *request = [NSFetchRequest fetchRequestWithEntityName:@"Filter"];
    NSArray<Filter *> *filtersArray = (NSArray<Filter *> *)[self.managedObjectContext executeFetchRequest:request error:&error];
    self.isAllSelected = NO;
    if(filtersArray.count > 0) {
        self.isAllSelected = YES;
        for(Filter *filter in filtersArray) {
            if(filter.filterIsSelected.intValue == 0 && filter.filterId.intValue >= 0) {
                self.isAllSelected = NO;
                break;
            }
        }
    }
    if(self.isAllSelected) {
        self.changedHeaderLabel = YES;
        self.selectAllImageView.hidden = YES;
        self.selectAllCheckImageView.hidden = NO;
    } else {
        self.changedHeaderLabel = NO;
        self.selectAllImageView.hidden = NO;
        self.selectAllCheckImageView.hidden = YES;
    }
}

- (IBAction)onShowResultsPressed:(id)sender {
    [self currentFilters];
    MainMenuViewController *mainMenuViewController = (MainMenuViewController *)[self.mainNavigationController.viewControllers objectAtIndex:0];
    [mainMenuViewController closeFilterDrawer];
    if(mainMenuViewController.brandsViewController.areBrandsVisible) {
        [mainMenuViewController changeHeader:nil withType:@"search"];
    } else {
        [mainMenuViewController changeHeader:nil withType:@"autocomplete"];
    }
    [mainMenuViewController.mapViewController setFilters];
    [mainMenuViewController.brandsViewController setFilters];    
    [mainMenuViewController.contentNavigationController popToRootViewControllerAnimated:YES];
}

-(void)currentFilters {
    NSMutableArray<NSString *> *filtersIdArray = [[NSMutableArray alloc] initWithCapacity:0];
    NSError *error;
    NSFetchRequest *request = [NSFetchRequest fetchRequestWithEntityName:@"Filter"];
    for(ItemCategory *itemCategory in self.categoriesArray) {
        request.predicate = [NSPredicate predicateWithFormat:@"parentId == %@ && filterId != -1", itemCategory.itemId];
        NSArray<Filter *> *subFilters = (NSArray<Filter *> *)[self.managedObjectContext executeFetchRequest:request error:&error];
        request.predicate = [NSPredicate predicateWithFormat:@"parentId == %@ && filterIsSelected == %@", itemCategory.itemId, @(YES)];
        NSArray<Filter *> *selectedSubfiltersArray = (NSArray<Filter *> *)[self.managedObjectContext executeFetchRequest:request error:&error];
        if(subFilters.count == selectedSubfiltersArray.count) {
            [filtersIdArray addObject:[NSString stringWithFormat:@"%d", itemCategory.itemId.intValue]];
        } else {
            for(Filter *filter in subFilters) {
                if([filter.filterIsSelected isEqual:@(YES)]) {
                    [filtersIdArray addObject:[NSString stringWithFormat:@"%d", filter.filterId.intValue]];
                }
            }
        }
    }
    
    NSString *filters = @"";
    if(filtersIdArray.count > 0) {
        filters = [filtersIdArray componentsJoinedByString:@","];
    }
    
    NSString *parentFilter = (NSString *)[self.userDefaults objectForKey:@"parentFilter"];
    if([filters isEqualToString:parentFilter]) {
        filters = @"";
    }
    
    [[NSUserDefaults standardUserDefaults] setValue:filters forKey:@"currentFilters"];
    [[NSUserDefaults standardUserDefaults] synchronize];
}

-(id<UIViewControllerAnimatedTransitioning>) navigationController:(UINavigationController *)navigationController animationControllerForOperation:(UINavigationControllerOperation)operation fromViewController:(UIViewController *)fromVC toViewController:(UIViewController *)toVC {
    AnimationTransitioning *animatedTransitioning = [[AnimationTransitioning alloc] init];
    if(operation == UINavigationControllerOperationPush) {
        if([toVC isKindOfClass:[SubcategoriesViewController class]]) {
            animatedTransitioning.animationOption = UIViewAnimationOptionTransitionCrossDissolve;
        }
    } else if(operation == UINavigationControllerOperationPop) {
        if([fromVC isKindOfClass:[SubcategoriesViewController class]]) {
            animatedTransitioning.animationOption = UIViewAnimationOptionTransitionCrossDissolve;
        }
    }
    return animatedTransitioning;
}


/*
// Override to support conditional editing of the table view.
- (BOOL)tableView:(UITableView *)tableView canEditRowAtIndexPath:(NSIndexPath *)indexPath {
    // Return NO if you do not want the specified item to be editable.
    return YES;
}
*/

/*
// Override to support editing the table view.
- (void)tableView:(UITableView *)tableView commitEditingStyle:(UITableViewCellEditingStyle)editingStyle forRowAtIndexPath:(NSIndexPath *)indexPath {
    if (editingStyle == UITableViewCellEditingStyleDelete) {
        // Delete the row from the data source
        [tableView deleteRowsAtIndexPaths:@[indexPath] withRowAnimation:UITableViewRowAnimationFade];
    } else if (editingStyle == UITableViewCellEditingStyleInsert) {
        // Create a new instance of the appropriate class, insert it into the array, and add a new row to the table view
    }   
}
*/

/*
// Override to support rearranging the table view.
- (void)tableView:(UITableView *)tableView moveRowAtIndexPath:(NSIndexPath *)fromIndexPath toIndexPath:(NSIndexPath *)toIndexPath {
}
*/

/*
// Override to support conditional rearranging of the table view.
- (BOOL)tableView:(UITableView *)tableView canMoveRowAtIndexPath:(NSIndexPath *)indexPath {
    // Return NO if you do not want the item to be re-orderable.
    return YES;
}
*/


#pragma mark - Navigation

- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    SubcategoriesViewController *subCategories = (SubcategoriesViewController *)[segue destinationViewController];
    NSIndexPath *indexPath = [self.categoriesTableView indexPathForSelectedRow];
    subCategories.subCategory = [self.categoriesArray objectAtIndex:indexPath.row];
}


@end
