//
//  ContentHolderViewController.m
//  plus
//
//  Created by Виктор Георгиев on 8/22/16.
//  Copyright © 2016 Outsourcing IT Services Ltd. All rights reserved.
//

#import "ContentHolderViewController.h"
#import "PlaceViewController.h"
#import "SingleBrandViewController.h"
#import "MainMenuViewController.h"
#import "WebViewController.h"

#import "AnimationTransitioning.h"
#import "PinView.h"

@interface ContentHolderViewController ()


@property (weak, nonatomic) IBOutlet UIView *buttonsBackgroundView;

@property (weak, nonatomic) IBOutlet UIButton *showMapButton;
@property (weak, nonatomic) IBOutlet UIButton *showListButton;

@property (assign, nonatomic) int currentPage;

@property (assign, nonatomic) BOOL initialLaunch;

@end

@implementation ContentHolderViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    
    self.navigationController.delegate = self;
    
    self.currentPage = 0;
    self.initialLaunch = YES;
    
    CGRect rect = self.buttonsBackgroundView.bounds;
    UIBezierPath *maskPath = [UIBezierPath bezierPathWithRoundedRect:rect byRoundingCorners:UIRectCornerTopLeft cornerRadii:CGSizeMake(10, 10)];
    CAShapeLayer *maskLayer = [CAShapeLayer layer];
    maskLayer.frame = rect;
    maskLayer.path = maskPath.CGPath;
    self.buttonsBackgroundView.layer.mask = maskLayer;
    
    self.brandsViewController.mainMenuViewController = (MainMenuViewController *)self.mainNavigationController.viewControllers.firstObject;
    self.brandsViewController.contentHolderViewController = self;
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
}

-(void)scrollViewDidScroll:(UIScrollView *)scrollView {
    self.currentPage = (scrollView.contentOffset.x + scrollView.frame.size.width / 2) / scrollView.frame.size.width;
    if(self.currentPage == 1 && self.initialLaunch) {
        [[NSNotificationCenter defaultCenter] postNotificationName:@"SetLocation" object:nil];
        self.initialLaunch = NO;
    }
    
    if(self.currentPage == 1) {
        self.brandsViewController.areBrandsVisible = NO;
        self.mapViewController.isMapVisible = YES;
        [((MainMenuViewController *)[self.mainNavigationController.viewControllers objectAtIndex:0]) changeHeader:nil withType:@"autocomplete"];
        CLLocationCoordinate2D location = CLLocationCoordinate2DMake(0, 0);
        [self.mapViewController setPins:location];
        [self.mapViewController refreshMap];
    } else if(self.currentPage == 0) {
        self.brandsViewController.areBrandsVisible = YES;
        self.mapViewController.isMapVisible = NO;
        [((MainMenuViewController *)[self.mainNavigationController.viewControllers objectAtIndex:0]) changeHeader:nil withType:@"search"];
        [self.brandsViewController setFilters];
        [self.mapViewController.pinView onClosePressed:self.mapViewController.pinView];
    }
}

- (IBAction)onShowMapPressed:(id)sender {
    if(self.currentPage == 0) {
        self.showMapButton.userInteractionEnabled = NO;
        [UIView animateWithDuration:0.7 animations:^{
            self.showMapButton.alpha = 0;
            self.showListButton.alpha = 1;
        } completion:^(BOOL finished) {
            self.showListButton.userInteractionEnabled = YES;
        }];
        [self.brandsViewController showMap];
    }
}

- (IBAction)onShowListPressed:(id)sender {
    if(self.currentPage == 1) {
        self.showListButton.userInteractionEnabled = NO;
        [UIView animateWithDuration:0.7 animations:^{
            self.showListButton.alpha = 0;
            self.showMapButton.alpha = 1;
        } completion:^(BOOL finished) {
            self.showMapButton.userInteractionEnabled = YES;
        }];
        [self.mapViewController showList];
    }
}

-(id<UIViewControllerAnimatedTransitioning>) navigationController:(UINavigationController *)navigationController animationControllerForOperation:(UINavigationControllerOperation)operation fromViewController:(UIViewController *)fromVC toViewController:(UIViewController *)toVC {
    AnimationTransitioning *animationTransitioning = [[AnimationTransitioning alloc] init];
    if(operation == UINavigationControllerOperationPush) {
        if([toVC isKindOfClass:[PlaceViewController class]]) {
            animationTransitioning.animationOption = UIViewAnimationOptionTransitionCrossDissolve;
        } else if([toVC isKindOfClass:[SingleBrandViewController class]]) {
            animationTransitioning.animationOption = UIViewAnimationOptionTransitionCrossDissolve;
        } else if([toVC isKindOfClass:[WebViewController class]]) {
            animationTransitioning.animationOption = UIViewAnimationOptionTransitionCrossDissolve;
        }
    } else if(operation == UINavigationControllerOperationPop) {
        if([fromVC isKindOfClass:[PlaceViewController class]]) {
            animationTransitioning.animationOption = UIViewAnimationOptionTransitionCrossDissolve;
        } else if([fromVC isKindOfClass:[SingleBrandViewController class]]) {
            animationTransitioning.animationOption = UIViewAnimationOptionTransitionCrossDissolve;
        } else if([fromVC isKindOfClass:[WebViewController class]]) {
            animationTransitioning.animationOption = UIViewAnimationOptionTransitionCrossDissolve;
        }
    }
    return animationTransitioning;
}

#pragma mark - Navigation

- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    if([[segue destinationViewController] isKindOfClass:[BrandsViewController class]]) {
        self.brandsViewController = (BrandsViewController *)[segue destinationViewController];
    } else if([[segue destinationViewController] isKindOfClass:[MapViewController class]]) {
        self.mapViewController = (MapViewController *)[segue destinationViewController];
    }
    ((MainMenuViewController *)[self.mainNavigationController.viewControllers objectAtIndex:0]).mapViewController = self.mapViewController;
    ((MainMenuViewController *)[self.mainNavigationController.viewControllers objectAtIndex:0]).brandsViewController = self.brandsViewController;
}


- (IBAction)showListButton:(id)sender {
}
@end
