//
//  FeedbackViewController.m
//  plus
//
//  Created by Виктор Георгиев on 9/9/16.
//  Copyright © 2016 Outsourcing IT Services Ltd. All rights reserved.
//

#import "FeedbackViewController.h"

#import "Utils.h"
#import "URLRequests.h"

@interface FeedbackViewController ()

@property (weak, nonatomic) IBOutlet UIScrollView *feedbackScrollView;

@property (weak, nonatomic) IBOutlet UILabel *feedbackTitle;
@property (weak, nonatomic) IBOutlet UILabel *feedbackDescription;
@property (weak, nonatomic) IBOutlet UILabel *phoneForFeedback;
@property (weak, nonatomic) IBOutlet UILabel *messageForFeedback;

@property (weak, nonatomic) IBOutlet UITextField *phoneNumberTextField;
@property (weak, nonatomic) IBOutlet UITextField *emailTextField;

@property (weak, nonatomic) IBOutlet UITextView *messageTextView;

@property (weak, nonatomic) IBOutlet UIButton *sendButton;

@property (assign, nonatomic) BOOL hasKeyboardShown;

@end

@implementation FeedbackViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    
    self.feedbackTitle.text = [Utils translate:@"Feedback"];
    self.feedbackDescription.text = [Utils translate:@"Feedback description"];
    self.phoneForFeedback.text = [Utils translate:@"Contact phone"];
    self.messageForFeedback.text = [Utils translate:@"Message"];
    [self.sendButton setTitle:[Utils translate:@"Send"] forState:UIControlStateNormal];
    
    self.phoneNumberTextField.layer.borderColor = [UIColor colorWithRed:204 / 255.0 green:204 / 255.0 blue:204 / 255.0 alpha:1].CGColor;
    self.phoneNumberTextField.layer.borderWidth = 1.0f;
    self.phoneNumberTextField.layer.cornerRadius = 5.0f;
    self.phoneNumberTextField.leftViewMode = UITextFieldViewModeAlways;
    UIImageView *imageView = [[UIImageView alloc] initWithFrame:CGRectMake(0, 0, 50, self.phoneNumberTextField.frame.size.height)];
    imageView.contentMode = UIViewContentModeCenter;
    imageView.image = [UIImage imageNamed:@"feedback_phone"];
    self.phoneNumberTextField.leftView = imageView;
    
    self.emailTextField.layer.borderColor = [UIColor colorWithRed:204 / 255.0 green:204 / 255.0 blue:204 / 255.0 alpha:1].CGColor;
    self.emailTextField.layer.borderWidth = 1.0f;
    self.emailTextField.layer.cornerRadius = 5.0f;
    self.emailTextField.leftViewMode = UITextFieldViewModeAlways;
    imageView = [[UIImageView alloc] initWithFrame:CGRectMake(0, 0, 50, self.emailTextField.frame.size.height)];
    imageView.contentMode = UIViewContentModeCenter;
    imageView.image = [UIImage imageNamed:@"feedback_mail"];
    self.emailTextField.leftView = imageView;
    
    self.messageTextView.layer.borderColor = [UIColor colorWithRed:204 / 255.0 green:204 / 255.0 blue:204 / 255.0 alpha:1].CGColor;
    self.messageTextView.layer.borderWidth = 1.0f;
    self.messageTextView.layer.cornerRadius = 5.0f;
    
    [self.messageTextView setTextContainerInset:UIEdgeInsetsMake(10, 40, 0, 0)];
    
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(keyboardDidShow:) name:UIKeyboardDidShowNotification object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(keyboardDidHide:) name:UIKeyboardDidHideNotification object:nil];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
}

- (void)keyboardDidShow:(NSNotification *)notification {
    if(!self.hasKeyboardShown) {
        NSValue *value = [notification.userInfo objectForKey:UIKeyboardFrameEndUserInfoKey];
        CGRect keyboardRect = value.CGRectValue;
        self.feedbackScrollView.contentSize = CGSizeMake(self.feedbackScrollView.contentSize.width, self.feedbackScrollView.contentSize.height + keyboardRect.size.height);
        
        self.hasKeyboardShown = YES;
    }
}

- (void)keyboardDidHide:(NSNotification *)notification {
    if(self.hasKeyboardShown) {
        NSValue *value = [notification.userInfo objectForKey:UIKeyboardFrameEndUserInfoKey];
        CGRect keyboardRect = value.CGRectValue;
        self.feedbackScrollView.contentSize = CGSizeMake(self.feedbackScrollView.contentSize.width, self.feedbackScrollView.contentSize.height - keyboardRect.size.height);
        
        self.hasKeyboardShown = NO;
    }
}

- (BOOL)textFieldShouldReturn:(UITextField *)textField {
    [textField resignFirstResponder];
    
    return YES;
}

- (IBAction)onPhonePressed:(id)sender {
    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:[NSString stringWithFormat:@"Do you want to dial this number: %@ ?", ((UIButton *)sender).titleLabel.text] message:@"" preferredStyle:UIAlertControllerStyleAlert];
    [alertController addAction:[UIAlertAction actionWithTitle:@"No" style:UIAlertActionStyleDefault handler:^(UIAlertAction *action) {
        [self dismissViewControllerAnimated:YES completion:^{
        }];
    }]];
    
    [alertController addAction:[UIAlertAction actionWithTitle:@"Yes" style:UIAlertActionStyleCancel handler:^(UIAlertAction *action) {
        NSString *number = @"tel:";
        number = [number stringByAppendingString:((UIButton *)sender).titleLabel.text];
        number = [number stringByReplacingOccurrencesOfString:@" " withString:@""];
        NSURL *url = [NSURL URLWithString:number];
        if([[UIApplication sharedApplication] canOpenURL:url]) {
            [[UIApplication sharedApplication] openURL:url];
        }
    }]];
    
    [self presentViewController:alertController animated:YES completion:nil];
}
- (IBAction)onSendPressed:(id)sender {
    [URLRequests sendFeedBack:self.phoneNumberTextField.text email:self.emailTextField.text message:self.messageTextView.text];
}

- (IBAction)onBackPressed:(id)sender {
    [self.navigationController popViewControllerAnimated:YES];
}

/*
#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    // Get the new view controller using [segue destinationViewController].
    // Pass the selected object to the new view controller.
}
*/

@end
