//
//  ViewController.m
//  plus
//
//  Created by Виктор Георгиев on 7/8/16.
//  Copyright © 2016 Outsourcing IT Services Ltd. All rights reserved.
//

#define NEW 0
#define PROMOTIONS 1
#define FAVOURITES 2
#define SHOPPING_CENTERS 3

#import "AppDelegate.h"

#import "MainMenuViewController.h"
#import "CatalogueControlViewController.h"
#import "PlaceViewController.h"
#import "ContentHolderViewController.h"
#import "LoadAnimationViewController.h"
#import "MainWebViewController.h"
#import "TermsAndConditionsViewController.h"
#import "FeedbackViewController.h"
#import "NotificationsHolderViewController.h"
#import "LoginViewController.h"
#import "ProfileViewController.h"
#import "ErrorViewController.h"

#import "SearchResultTableViewCell.h"
#import "MainDrawerTableViewCell.h"

#import "URLRequests.h"
#import "ItemCategory.h"
#import "AnimationTransitioning.h"

#import <FBSDKCoreKit/FBSDKCoreKit.h>

@interface MainMenuViewController ()

@property (strong, nonatomic) NSUserDefaults *userDefaults;

@property (strong, nonatomic) UINavigationController *categoriesNavigationController;

@property (weak, nonatomic) LoadAnimationViewController *animationViewController;

@property (weak, nonatomic) IBOutlet UITableView *searchResultsTableView;

@property (weak, nonatomic) IBOutlet UIView *mainView;
@property (weak, nonatomic) IBOutlet UIView *mainDrawerView;
@property (weak, nonatomic) IBOutlet UIView *filterDrawerView;
@property (weak, nonatomic) IBOutlet UIView *transparentView;
@property (weak, nonatomic) IBOutlet UIView *searchView;

@property (weak, nonatomic) IBOutlet UIButton *searchButton;

@property (weak, nonatomic) IBOutlet UILabel *headerLabel;

@property (weak, nonatomic) IBOutlet NSLayoutConstraint *mainViewLeadingConstraint;
@property (weak, nonatomic) IBOutlet NSLayoutConstraint *mainViewTrailingConstraint;
@property (weak, nonatomic) IBOutlet NSLayoutConstraint *mainDrawerLeadingConstraint;
@property (weak, nonatomic) IBOutlet NSLayoutConstraint *filterDrawerTrailingConstraint;
@property (weak, nonatomic) IBOutlet NSLayoutConstraint *filterDrawerWidthConstraint;
@property (weak, nonatomic) IBOutlet NSLayoutConstraint *searchResultsTableViewHeightConstraint;

@property (strong, nonatomic) IBOutlet UIScreenEdgePanGestureRecognizer *mainDrawerGestureRecognizer;
@property (strong, nonatomic) IBOutlet UIScreenEdgePanGestureRecognizer *filterDrawerGestureRecognizer;

@property (strong, nonatomic) NSArray *mainDrawerArray;
@property (strong, nonatomic) NSArray *mainDrawerImages;
@property (strong, nonatomic) NSArray *searchResultsArray;

@property (strong, nonatomic) NSTimer *timer;

@property (assign, nonatomic) int mainDrawerConstraint;
@property (assign, nonatomic) int filterDrawerConstraint;
@property (assign, nonatomic) int repeatCount;

@property (assign, nonatomic) float initialAlpha;
@property (assign, nonatomic) float finalAlpha;

@property (assign, nonatomic) BOOL isInitialLaunch;
@property (assign, nonatomic) BOOL isMainDrawerOpen;
@property (assign, nonatomic) BOOL isFilterDrawerOpen;
@property (assign, nonatomic) BOOL isItemSelected;
@property (assign, nonatomic) BOOL hasSelectedSearchResult;

@end

@implementation MainMenuViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    
    self.userDefaults = [NSUserDefaults standardUserDefaults];
    
    self.mainDrawerConstraint = 300;
    self.filterDrawerConstraint = 270;
    self.initialAlpha = 0.0;
    self.finalAlpha = 0.6;
    
    self.mainDrawerArray = @[@"New", @"Promotions", @"Favourites", @"Shopping centers", @"Profile", @"Notifications", @"Feedback", @"TermsAndConditions", @"Language" , @"Bulbank Mobile", @"UniCredit Bulbank"];
    self.mainDrawerImages = @[@"menu_new", @"menu_promotions", @"menu_favourites", @"menu_shopping_center", @"menu_profile", @"menu_notifications", @"menu_feedback", @"menu_terms_and_conditions", @"menu_language", @"menu_bulbank_mobile", @"menu_bulbank_online"];
    
    self.isInitialLaunch = YES;
    self.hasSelectedSearchResult = NO;
    
    self.searchTextField.text = [Utils translate:@"Brand search"];
    
    self.autocompleteTextField.text = [Utils translate:@"Map search"];
    self.autocompleteTextField.hidden = YES;
    self.autocompleteTextField.userInteractionEnabled = NO;
    
    [URLRequests getNotifications:^(long newNotifications) {
        self.newNotificationsCount = newNotifications;
        if(self.newNotificationsCount > 0) {
            self.notificationsDotView.layer.borderWidth = 2.0f;
            self.notificationsDotView.layer.borderColor = [UIColor whiteColor].CGColor;
            self.notificationsDotView.hidden = NO;
        }
        [self.mainDrawerTableView reloadData];
    }];
    
    self.navigationController.delegate = self;
    self.filterDrawerGestureRecognizer.delegate = self;
    
    if([Utils deviceHasInternetConneciton]) {
        if([CLLocationManager locationServicesEnabled]) {
            if ([CLLocationManager authorizationStatus]==kCLAuthorizationStatusDenied) {
                UIAlertController *alertController = [UIAlertController alertControllerWithTitle:@"Plus" message:[Utils translate:@"Location error"] preferredStyle:UIAlertControllerStyleAlert];
                [alertController addAction:[UIAlertAction actionWithTitle:@"Close" style:UIAlertActionStyleCancel handler:^(UIAlertAction * _Nonnull action) {
                    exit(0);
                }]];
                [self presentViewController:alertController animated:YES completion:nil];
                [alertController addAction:[UIAlertAction actionWithTitle:@"Settings" style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
                    [[UIApplication sharedApplication] openURL:[NSURL URLWithString:UIApplicationOpenSettingsURLString]];
                }]];
            } else {
                [Utils displayAnimation];
            }
        }
    } else {
        [self performSegueWithIdentifier:@"ErrorSegue" sender:self];
    }
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
}

-(void)startNotificationAnimation {
    self.repeatCount = 0;
    self.timer = [NSTimer scheduledTimerWithTimeInterval:0.12 target:self selector:@selector(animation) userInfo:nil repeats:YES];
    [self.timer fire];
}

-(void)animation {
    self.repeatCount++;
    if(self.repeatCount > 5) {
        [self.timer invalidate];
        [self.view layoutIfNeeded];
        [UIView animateWithDuration:0.03 delay:1.06 options:UIViewAnimationOptionCurveLinear animations:^{
            self.notificationsDotViewLeadingConstraint.constant = 16;
            [self.view layoutIfNeeded];
        } completion:^(BOOL finished) {
            
        }];
    } else {
        [self.view layoutIfNeeded];
        [UIView animateWithDuration:0.06 delay:1.0 options:UIViewAnimationOptionCurveLinear animations:^{
            self.notificationsDotViewLeadingConstraint.constant = 14;
            [self.view layoutIfNeeded];
        } completion:^(BOOL finished) {
            
        }];
        
        [UIView animateWithDuration:0.06 delay:1.06 options:UIViewAnimationOptionCurveLinear animations:^{
            self.notificationsDotViewLeadingConstraint.constant = 18;
            [self.view layoutIfNeeded];
        } completion:^(BOOL finished) {
            
        }];
    }
}

-(void)changeHeader:(NSString *)header withType:(NSString *)type {
    if([type isEqualToString:@"title"]) {
        self.headerLabel.text = header;
        self.headerLabel.alpha = 0.0;
        self.headerLabel.hidden = NO;
        [UIView animateWithDuration:0.7 animations:^{
            self.headerLabel.alpha = 1;
            self.autocompleteTextField.alpha = 0;
            self.searchView.alpha = 0;
        } completion:^(BOOL finished) {
            self.autocompleteTextField.hidden = YES;
            self.autocompleteTextField.userInteractionEnabled = NO;
            self.autocompleteTextField.text = [Utils translate:@"Map search"];
            
            self.searchView.hidden = YES;
            self.searchView.userInteractionEnabled = NO;
            self.searchTextField.text = [Utils translate:@"Brand search"];
        }];
    } else if([type isEqualToString:@"autocomplete"]) {
        self.autocompleteTextField.alpha = 0.0;
        self.autocompleteTextField.hidden = NO;
        [UIView animateWithDuration:0.7 animations:^{
            self.autocompleteTextField.alpha = 1;
            self.headerLabel.alpha = 0;
            self.searchView.alpha = 0;
        } completion:^(BOOL finished) {
            self.autocompleteTextField.userInteractionEnabled = YES;
            
            self.headerLabel.hidden = YES;
            
            self.searchView.hidden = YES;
            self.searchView.userInteractionEnabled = NO;
            self.searchTextField.text = [Utils translate:@"Brand search"];
        }];
    } else if([type isEqualToString:@"search"] || self.brandsViewController.areBrandsVisible) {
        self.searchView.alpha = 0.0;
        self.searchView.hidden = NO;
        [UIView animateWithDuration:0.7 animations:^{
            self.searchView.alpha = 1;
            self.headerLabel.alpha = 0;
            self.autocompleteTextField.alpha = 0;
        } completion:^(BOOL finished) {
            self.searchView.userInteractionEnabled = YES;
            
            self.headerLabel.hidden = YES;
            
            self.autocompleteTextField.hidden = YES;
            self.autocompleteTextField.userInteractionEnabled = NO;
            self.autocompleteTextField.text = [Utils translate:@"Map search"];
        }];
    }
}

- (void)textFieldDidBeginEditing:(UITextField *)textField {
    self.hasSelectedSearchResult = NO;
    textField.text = @"";
}

- (IBAction)textFieldDidChangeText:(id)sender {
    UITextField *textField = (UITextField *)sender;
    if(textField == self.autocompleteTextField) {
        if(textField.text.length > 0) {
            self.searchResultsTableView.hidden = NO;
            if([textField.text containsString:[Utils translate:@"Map search"]]) {
                textField.text = [textField.text substringFromIndex:textField.text.length - 1];
            } else {
                NSString *mapSearch = [Utils translate:@"Map search"];
                NSString *text = [mapSearch substringToIndex:mapSearch.length - 1];
                if([textField.text isEqualToString:text]) {
                    textField.text = @"";
                }
            }
        } else {
            textField.text = [Utils translate:@"Map search"];
            self.searchResultsTableView.hidden = YES;
        }
        
        NSString *urlString = [NSString stringWithFormat:@"https://maps.googleapis.com/maps/api/place/autocomplete/json?input=%@&components=country%%3ABG&key=AIzaSyAHGuBF9bYmj6er1R1dNm62O-bJqruy-hA", [textField.text stringByAddingPercentEscapesUsingEncoding:NSUTF8StringEncoding]];
        NSURL *url = [NSURL URLWithString:urlString];
        NSURLRequest *request = [NSURLRequest requestWithURL:url];
        NSURLSessionConfiguration* configuration = [NSURLSessionConfiguration defaultSessionConfiguration];
        NSURLSession* urlSession = [NSURLSession sessionWithConfiguration:configuration delegate:nil delegateQueue:[NSOperationQueue mainQueue]];
        NSURLSessionDataTask *urlConnection = [urlSession dataTaskWithRequest:request completionHandler:^(NSData * _Nullable data, NSURLResponse * _Nullable response, NSError * _Nullable error) {
            if(data != nil) {
                NSError *error;
                NSDictionary *predictionsDictionary = [NSJSONSerialization JSONObjectWithData:data options:0 error:&error];
                if(predictionsDictionary.count > 0) {
                    self.searchResultsArray = (NSArray *)[predictionsDictionary objectForKey:@"predictions"];
                    [self.searchResultsTableView reloadData];
                }
            }
        }];
        [urlConnection resume];
    } else if(textField == self.searchTextField) {
        if(textField.text.length > 0) {
            if([textField.text containsString:[Utils translate:@"Brand search"]]) {
                textField.text = [textField.text substringFromIndex:textField.text.length - 1];
            } else {
                NSString *brandSearch = [Utils translate:@"Brand search"];
                NSString *text = [brandSearch substringToIndex:brandSearch.length - 1];
                if([textField.text isEqualToString:text]) {
                    textField.text = @"";
                }
            }
        } else {
            textField.text = [Utils translate:@"Brand search"];
            self.searchResultsTableView.hidden = YES;
        }

        if(![textField.text isEqualToString:@""] && ![textField.text isEqualToString:[Utils translate:@"Brand search"]] && ![textField.text isEqualToString:[Utils translate:@"Map search"]]) {
            
            NSMutableArray *resultsArray = [[NSMutableArray alloc] initWithCapacity:0];
            for(BrandName *brand in self.brandNamesArray) {
                long length = textField.text.length;
                NSString *name = brand.brandName;
                if(length < name.length) {
                    if([textField.text caseInsensitiveCompare:[name substringWithRange:NSMakeRange(0, length)]] == NSOrderedSame) {
                        [resultsArray addObject:@{@"description" : name}];
                    }
                }
            }
            
            if(resultsArray.count > 0) {
                self.searchResultsTableView.hidden = NO;
                self.searchResultsArray = resultsArray;
                if(resultsArray.count <= 5) {
                    self.searchResultsTableViewHeightConstraint.constant = resultsArray.count * 41 - 1;
                } else {
                   self.searchResultsTableViewHeightConstraint.constant = 5 * 41 - 1;
                }
                [self.searchResultsTableView reloadData];
            } else {
                self.searchResultsTableView.hidden = YES;
                self.searchResultsArray = [[NSArray alloc] init];
                self.searchResultsTableViewHeightConstraint.constant = 0;
                [self.searchResultsTableView reloadData];
            }
        }
    }
}

- (void)textFieldDidEndEditing:(UITextField *)textField {
    if(textField == self.autocompleteTextField) {
        if(!self.hasSelectedSearchResult) {
            self.autocompleteTextField.text = [Utils translate:@"Map search"];
            self.searchResultsTableView.hidden = YES;
        }
    } else if(textField == self.searchTextField) {
        self.searchTextField.text = [Utils translate:@"Brand search"];
        self.searchResultsTableView.hidden = YES;
    }
}

- (BOOL)textFieldShouldReturn:(UITextField *)textField {
    if(textField == self.autocompleteTextField || textField == self.searchTextField) {
        if(textField == self.autocompleteTextField) {
            if([textField.text isEqualToString:@""]) {
                textField.text = [Utils translate:@"Map search"];
            }
        } else if(textField == self.searchTextField) {
            if([textField.text isEqualToString:@""]) {
                textField.text = [Utils translate:@"Brand search"];
            } else {
                [self onSearchPressed:self];
            }
        }
        [self textFieldDidEndEditing:textField];
    }

    if(textField == self.searchTextField) {
        [self.searchTextField resignFirstResponder];
    } else if(textField == self.autocompleteTextField) {
        [self.autocompleteTextField resignFirstResponder];
    }
    return YES;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    if(tableView == self.mainDrawerTableView) {
        return self.mainDrawerArray.count;
    } else {
        return self.searchResultsArray.count;
    }
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    long row = indexPath.row;
    if(tableView == self.mainDrawerTableView) {
        MainDrawerTableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"Cell"];
        cell.selectionStyle = UITableViewCellSelectionStyleNone;
        UIColor *separatorColor = nil;
        UIColor *backgroundColor = nil;
        if(row <= 2) {
            separatorColor = [UIColor colorWithRed:76 / 255.0 green:100 / 255.0 blue:110 / 255.0 alpha:1];
            backgroundColor = [UIColor colorWithRed:70 / 255.0 green:90 / 255.0 blue:101 / 255.0 alpha:1];
        } else if(row == 3) {
            separatorColor = [UIColor colorWithRed:139 / 255.0 green:159 / 255.0 blue:168 / 255.0 alpha:1];
            backgroundColor = [UIColor colorWithRed:70 / 255.0 green:90 / 255.0 blue:101 / 255.0 alpha:1];
        } else if(row > 3 && indexPath.row <= 7) {
            separatorColor = [UIColor colorWithRed:63 / 255.0 green:82 / 255.0 blue:89 / 255.0 alpha:1];
            backgroundColor = [UIColor colorWithRed:54 / 255.0 green:71 / 255.0 blue:79 / 255.0 alpha:1];
        } else if(row == 8) {
            separatorColor = [UIColor colorWithRed:101 / 255.0 green:114 / 255.0 blue:120 / 255.0 alpha:1];
            backgroundColor = [UIColor colorWithRed:54 / 255.0 green:71 / 255.0 blue:79 / 255.0 alpha:1];
        } else if(row > 8) {
            separatorColor = [UIColor colorWithRed:50 / 255.0 green:63 / 255.0 blue:71 / 255.0 alpha:1];
            backgroundColor = [UIColor colorWithRed:39 / 255.0 green:50 / 255.0 blue:56 / 255.0 alpha:1];
        }
        cell.drawerCellSeparator.backgroundColor = separatorColor;
        cell.backgroundColor = backgroundColor;
        
        cell.drawerCellLabel.text = [Utils translate:[self.mainDrawerArray objectAtIndex:row]];
        cell.drawerCellImage.image = [UIImage imageNamed:[self.mainDrawerImages objectAtIndex:row]];
        
        if(row == 5) {
            if(self.newNotificationsCount > 0) {
                cell.drawerCellLabel.textColor = [UIColor colorWithRed:235.0 / 255 green:194.0 / 255 blue:55 / 255.0 alpha:1];
                cell.drawerCellImage.image = [UIImage imageNamed:@"menu_notifications_unread"];
                cell.drawerCellNotificationsCountLabel.text = [NSString stringWithFormat:@"%lu", self.newNotificationsCount];
                cell.drawerCellNotificationsView.layer.borderColor = [UIColor colorWithRed:235.0 / 255 green:194.0 / 255 blue:55 / 255.0 alpha:1].CGColor;
                cell.drawerCellNotificationsView.layer.borderWidth = 2.0f;
                cell.drawerCellNotificationsView.hidden = NO;
            } else {
                cell.drawerCellLabel.textColor = [UIColor whiteColor];
                cell.drawerCellImage.image = [UIImage imageNamed:@"menu_notifications"];
                cell.drawerCellNotificationsView.hidden = YES;
            }
        }
        return cell;
    } else {
        SearchResultTableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"Cell"];
        NSString *searchString = [[self.searchResultsArray objectAtIndex:row] objectForKey:@"description"];
        if([searchString containsString:@"&#39;"]) {
            searchString = [searchString stringByReplacingOccurrencesOfString:@"&#39;" withString:@"'"];
        }
        cell.searchResultLabel.text = searchString;
        return cell;
    }
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    if(tableView == self.mainDrawerTableView) {
        if(indexPath.row >= 0 && indexPath.row <= 3) {
            [self mainDrawerAnimation: -self.mainDrawerConstraint andMainViewConstraint:0 andAlpha:0];
            [self turnOnGestureRecognizers];
            self.isMainDrawerOpen = NO;
            [self performSegueWithIdentifier:@"CatalogueSegue" sender:self];
                self.isItemSelected = YES;
        } else if(indexPath.row == 4) {
            [self performSegueWithIdentifier:@"ProfileSegue" sender:self];
            [self mainDrawerAnimation: -self.mainDrawerConstraint andMainViewConstraint:0 andAlpha:0];
            self.isMainDrawerOpen = NO;
            [self turnOnGestureRecognizers];
        } else if(indexPath.row == 5) {
            [self performSegueWithIdentifier:@"NotificationSegue" sender:self];
            [self mainDrawerAnimation: -self.mainDrawerConstraint andMainViewConstraint:0 andAlpha:0];
            self.isMainDrawerOpen = NO;
        } else if(indexPath.row == 6) {
            [self mainDrawerAnimation: -self.mainDrawerConstraint andMainViewConstraint:0 andAlpha:0];
            [self turnOnGestureRecognizers];
            self.isMainDrawerOpen = NO;
            [self performSegueWithIdentifier:@"FeedbackSegue" sender:self];
        } else if(indexPath.row == 7) {
            [self mainDrawerAnimation: -self.mainDrawerConstraint andMainViewConstraint:0 andAlpha:0];
            [self turnOnGestureRecognizers];
            self.isMainDrawerOpen = NO;
            [self performSegueWithIdentifier:@"TermsAndConditionsSegue" sender:self];
        } else if(indexPath.row == 8) {
            [self changeLanguage];
        } else if(indexPath.row == 9) {
            [[UIApplication sharedApplication] openURL: [NSURL URLWithString:@"itms-apps://itunes.apple.com/app/id475724779"] options:@{UIApplicationOpenURLOptionsSourceApplicationKey :[[NSBundle mainBundle] bundleIdentifier]} completionHandler:nil];
        } else if(indexPath.row == 10) {
            [self mainDrawerAnimation: -self.mainDrawerConstraint andMainViewConstraint:0 andAlpha:0];
            [self turnOnGestureRecognizers];
            self.isMainDrawerOpen = NO;
            [self performSegueWithIdentifier:@"MainWebViewSegue" sender:self];
        }
    } else {
        if(!self.brandsViewController.areBrandsVisible) {
            self.hasSelectedSearchResult = YES;
            NSString *placeId = [[self.searchResultsArray objectAtIndex:indexPath.row] objectForKey:@"place_id"];
            [self.autocompleteTextField resignFirstResponder];
            self.autocompleteTextField.text = [[self.searchResultsArray objectAtIndex:indexPath.row] objectForKey:@"description"];
            NSString *urlString = [NSString stringWithFormat:@"https://maps.googleapis.com/maps/api/place/details/json?placeid=%@&key=AIzaSyAHGuBF9bYmj6er1R1dNm62O-bJqruy-hA", placeId];
            NSURL *url = [NSURL URLWithString:urlString];
            NSURLRequest *request = [NSURLRequest requestWithURL:url];
            NSURLSessionConfiguration* configuration = [NSURLSessionConfiguration defaultSessionConfiguration];
            NSURLSession* urlSession = [NSURLSession sessionWithConfiguration:configuration delegate:nil delegateQueue:[NSOperationQueue mainQueue]];
            NSURLSessionDataTask *urlConnection = [urlSession dataTaskWithRequest:request completionHandler:^(NSData * _Nullable data, NSURLResponse * _Nullable response, NSError * _Nullable error) {
                if(data != nil) {
                    NSError *error;
                    NSDictionary *predictionsDictionary = [NSJSONSerialization JSONObjectWithData:data options:0 error:&error];
                    if(predictionsDictionary.count > 0) {
                        NSDictionary *locationDictionary = [[[predictionsDictionary objectForKey:@"result"] objectForKey:@"geometry"] objectForKey:@"location"];
                        double latitude = [[locationDictionary objectForKey:@"lat"] doubleValue];
                        double longitude = [[locationDictionary objectForKey:@"lng"] doubleValue];
                        CLLocation *location = [[CLLocation alloc] initWithLatitude:latitude longitude:longitude];
                        self.searchResultsTableView.hidden = YES;
                        [self.mapViewController setLocation:location];
                    }
                }
            }];
            [urlConnection resume];
        } else {
            self.searchResultsTableView.hidden = YES;
            self.searchResultsTableViewHeightConstraint.constant = 0;
            for(BrandName *brandName in self.brandNamesArray) {
                SearchResultTableViewCell *cell = [tableView cellForRowAtIndexPath:indexPath];
                if([brandName.brandName.lowercaseString containsString:cell.searchResultLabel.text.lowercaseString]) {
                    self.brandsViewController.loadFromSearchResults = YES;
                    self.brandsViewController.brandId = [NSNumber numberWithInt:brandName.brandId.intValue];
                    [self.brandsViewController performSegueWithIdentifier:@"BrandSegue" sender:self.brandsViewController];
                    if([self.searchTextField isFirstResponder]) {
                        [self.searchTextField resignFirstResponder];
                    }
                }
            }
        }
    }
}

- (IBAction)onSearchPressed:(id)sender {
    self.brandsViewController.keyword = self.searchTextField.text;
    self.brandsViewController.shouldDisplayAnimation = YES;
    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, 0.3 * NSEC_PER_SEC), dispatch_get_main_queue(), ^{
        [Utils displayAnimation];
    });
    [self.categoriesViewController categoriesSelect:NO];
    [self.userDefaults setValue:@"" forKey:@"currentFilters"];
    [self.userDefaults setValue:@"" forKey:@"mapFilters"];
    [self.userDefaults setValue:@"" forKey:@"brandFilters"];
    [self.userDefaults synchronize];
    [self.view endEditing:YES];
    [self.searchTextField resignFirstResponder];
    [self.brandsViewController showSearchResults];
}

- (IBAction)closeDrawersControl:(id)sender {
    if([sender isKindOfClass:[UIPanGestureRecognizer class]]) {
        if(self.isMainDrawerOpen) {
            [self closeMainDrawerSwipeRecognizer:sender];
        } else if(self.isFilterDrawerOpen) {
            [self closeFilterDrawerSwipeRecognizer:sender];
        }
    } else {
        if(self.isMainDrawerOpen) {
            [self mainDrawerControl:sender];
        } else if(self.isFilterDrawerOpen) {
            [self filterDrawerControl:sender];
        }
    }
}

//main drawer animations
- (IBAction)mainDrawerControl:(id)sender {
    int drawerConstraint;
    int mainViewConstraint;
    float alpha;
    if(!self.isMainDrawerOpen && !self.isFilterDrawerOpen) {
        drawerConstraint = 0;
        mainViewConstraint = self.mainDrawerConstraint;
        alpha = self.finalAlpha;
        self.isMainDrawerOpen = YES;
        if([self.searchTextField isFirstResponder]) {
            [self.searchTextField resignFirstResponder];
        }
        if([self.autocompleteTextField isFirstResponder]) {
            [self.autocompleteTextField resignFirstResponder];
        }
    } else {
        drawerConstraint = -self.mainDrawerConstraint;
        mainViewConstraint = 0;
        alpha = self.initialAlpha;
        self.isMainDrawerOpen = NO;
    }
    
    if(!self.isFilterDrawerOpen) {
        [self.transparentView setHidden:NO];
        [self mainDrawerAnimation:drawerConstraint andMainViewConstraint:mainViewConstraint andAlpha:alpha];
    }
}

- (IBAction)openMainDrawerSwipeRecognizer:(id)sender {
    UIScreenEdgePanGestureRecognizer *drag = (UIScreenEdgePanGestureRecognizer *) sender;
    if(drag.state == UIGestureRecognizerStateBegan) {
        self.filterDrawerGestureRecognizer.enabled = NO;
        if([self.searchTextField isFirstResponder]) {
            [self.searchTextField resignFirstResponder];
        }
        if([self.autocompleteTextField isFirstResponder]) {
            [self.autocompleteTextField resignFirstResponder];
        }
        [self.transparentView setHidden:NO];
    }
    
    if([drag velocityInView:self.view].x > 500) {
        [self mainDrawerAnimation:0 andMainViewConstraint:self.mainDrawerConstraint andAlpha:self.finalAlpha];
        self.isMainDrawerOpen = YES;
    } else {
        CGPoint point = [drag translationInView:self.view];
        if(point.x < self.mainDrawerConstraint) {
            [self setMainDrawerConstraint:-self.mainDrawerConstraint + point.x andMainViewLeadingConstraint:point.x andAlpha:point.x / self.mainDrawerConstraint * self.finalAlpha];
            if((point.x >= self.mainDrawerConstraint / 2 && drag.state == UIGestureRecognizerStateEnded)) {
                [self mainDrawerAnimation:0 andMainViewConstraint:self.mainDrawerConstraint andAlpha:self.finalAlpha];
                self.isMainDrawerOpen = YES;
            } else if(point.x < self.mainDrawerConstraint / 2 && drag.state == UIGestureRecognizerStateEnded) {
                [self mainDrawerAnimation:-self.mainDrawerConstraint andMainViewConstraint:0 andAlpha:self.initialAlpha];
                self.isMainDrawerOpen = NO;
            }
        } else {
            self.isMainDrawerOpen = YES;
        }
    }
}

- (IBAction)closeMainDrawerSwipeRecognizer:(id)sender {
    UIPanGestureRecognizer *swipe = (UIPanGestureRecognizer *) sender;
    CGPoint point = [swipe translationInView:self.mainDrawerView];
    
    if([swipe velocityInView:self.view].x < -500) {
        [self mainDrawerAnimation: - self.mainDrawerConstraint andMainViewConstraint:0 andAlpha:self.initialAlpha];
        self.isMainDrawerOpen = NO;
    } else {
        if(point.x < 0 && point.x >= - self.mainDrawerConstraint) {
            [self setMainDrawerConstraint:point.x andMainViewLeadingConstraint:self.mainDrawerConstraint + point.x andAlpha:(self.mainDrawerConstraint + point.x) / self.mainDrawerConstraint * self.finalAlpha];
        }
        if(swipe.state == UIGestureRecognizerStateEnded) {
            if(-point.x <= self.mainDrawerConstraint / 2 && swipe.state == UIGestureRecognizerStateEnded) {
                [self mainDrawerAnimation:0 andMainViewConstraint:self.mainDrawerConstraint andAlpha:self.finalAlpha];
                self.isMainDrawerOpen = YES;
            } else if(-point.x > self.mainDrawerConstraint / 2 && swipe.state == UIGestureRecognizerStateEnded) {
                [self mainDrawerAnimation: - self.mainDrawerConstraint andMainViewConstraint:0 andAlpha:self.initialAlpha];
                self.isMainDrawerOpen = NO;
            }
        }
    }
}

-(void)setMainDrawerConstraint: (float)mainDrawerConstraint andMainViewLeadingConstraint:(float)mainViewLeadingConstraint andAlpha:(float)alpha {
    self.mainDrawerLeadingConstraint.constant = mainDrawerConstraint;
    self.mainViewLeadingConstraint.constant = mainViewLeadingConstraint;
    self.mainViewTrailingConstraint.constant = - mainViewLeadingConstraint;
    self.transparentView.alpha = alpha;
    [self.view layoutIfNeeded];
}

-(void)mainDrawerAnimation: (int)drawerConstraint andMainViewConstraint:(int)mainViewConstraint andAlpha:(float)alpha {
    [self turnOffGestureRecognizers];
    [UIView animateWithDuration:0.5 animations:^{
        [self setMainDrawerConstraint:drawerConstraint andMainViewLeadingConstraint:mainViewConstraint andAlpha:alpha];
    } completion:^(BOOL finished){
        if(finished && alpha == 0) {
            [self.transparentView setHidden:YES];
            [self turnOnGestureRecognizers];
        }
        if(self.isItemSelected) {
            self.isItemSelected = NO;
            
        }
    }];
}

//filterDrawerAnimations
- (IBAction)filterDrawerControl:(id)sender {
    int drawerConstraint;
    int mainViewConstraint;
    float alpha;
    if(!self.isFilterDrawerOpen && !self.isMainDrawerOpen) {
        drawerConstraint = 0;
        mainViewConstraint = self.filterDrawerConstraint;
        alpha = self.finalAlpha;
        self.isFilterDrawerOpen = YES;
        if([self.searchTextField isFirstResponder]) {
            [self.searchTextField resignFirstResponder];
        }
        if([self.autocompleteTextField isFirstResponder]) {
            [self.autocompleteTextField resignFirstResponder];
        }
    } else {
        drawerConstraint = - self.filterDrawerConstraint;
        mainViewConstraint = 0;
        alpha = self.initialAlpha;
        self.isFilterDrawerOpen = NO;
        if(self.isMainDrawerOpen) {
            [self mainDrawerControl:sender];
        }
    }
    
    [self.transparentView setHidden:NO];
    [self filterDrawerAnimation:drawerConstraint andMainViewConstraint:mainViewConstraint andAlpha:alpha];
}

- (IBAction)openFilterDrawerSwipeRecognizer:(id)sender {
    UIScreenEdgePanGestureRecognizer *drag = (UIScreenEdgePanGestureRecognizer *) sender;
    if(drag.state == UIGestureRecognizerStateBegan) {
        self.mainDrawerGestureRecognizer.enabled = NO;
        [self.transparentView setHidden:NO];
        if([self.searchTextField isFirstResponder]) {
            [self.searchTextField resignFirstResponder];
        }
        if([self.autocompleteTextField isFirstResponder]) {
            [self.autocompleteTextField resignFirstResponder];
        }
    }
    
    if([drag velocityInView:self.view].x < -500) {
        [self filterDrawerAnimation:0 andMainViewConstraint:self.filterDrawerConstraint andAlpha:self.finalAlpha];
        self.isFilterDrawerOpen = YES;
        [self turnOffGestureRecognizers];
    } else {
        CGPoint point = [drag translationInView:self.view];
        if(-point.x < self.filterDrawerConstraint) {
            [self setFilterDrawerConstraint:- self.filterDrawerConstraint - point.x andMainViewLeadingConstraint: - point.x andAlpha: - point.x / self.filterDrawerConstraint * self.finalAlpha];
            if(-point.x >= self.filterDrawerConstraint / 2 && drag.state == UIGestureRecognizerStateEnded) {
                [self filterDrawerAnimation:0 andMainViewConstraint:self.filterDrawerConstraint andAlpha:self.finalAlpha];
                self.isFilterDrawerOpen = YES;
                [self turnOffGestureRecognizers];
            } else if(-point.x < self.filterDrawerConstraint / 2 && drag.state == UIGestureRecognizerStateEnded) {
                [self filterDrawerAnimation:-self.filterDrawerConstraint andMainViewConstraint:0 andAlpha:self.initialAlpha];
                self.isFilterDrawerOpen = NO;
                [self turnOffGestureRecognizers];
            }
        } else {
            self.isFilterDrawerOpen = YES;
        }
    }
}

- (IBAction)closeFilterDrawerSwipeRecognizer:(id)sender {
    UIPanGestureRecognizer *swipe = (UIPanGestureRecognizer *) sender;
    if([swipe velocityInView:self.view].x > 500) {
        [self filterDrawerAnimation:- self.filterDrawerConstraint andMainViewConstraint:0 andAlpha:self.initialAlpha];
        self.isFilterDrawerOpen = NO;
        [self turnOffGestureRecognizers];
    } else {
        CGPoint point = [swipe translationInView:self.filterDrawerView];
        if(point.x > 0 && point.x <= self.filterDrawerConstraint) {
            [self setFilterDrawerConstraint: - point.x andMainViewLeadingConstraint:self.filterDrawerConstraint - point.x andAlpha:(self.filterDrawerConstraint - point.x) / self.filterDrawerConstraint * self.finalAlpha];
        }
        if(swipe.state == UIGestureRecognizerStateEnded) {
            if(point.x <= self.filterDrawerConstraint / 2 && swipe.state == UIGestureRecognizerStateEnded) {
                [self filterDrawerAnimation:0 andMainViewConstraint:self.filterDrawerConstraint andAlpha:self.finalAlpha];
                self.isFilterDrawerOpen = YES;
            } else if(point.x > self.filterDrawerConstraint / 2 && swipe.state == UIGestureRecognizerStateEnded) {
                [self filterDrawerAnimation:- self.filterDrawerConstraint andMainViewConstraint:0 andAlpha:self.initialAlpha];
                self.isFilterDrawerOpen = NO;
            }
        }
    }
}

-(void)setFilterDrawerConstraint: (float)filterDrawerConstraint andMainViewLeadingConstraint:(float)mainViewLeadingConstraint andAlpha:(float)alpha {
    self.filterDrawerTrailingConstraint.constant = filterDrawerConstraint;
    self.mainViewLeadingConstraint.constant = - mainViewLeadingConstraint;
    self.mainViewTrailingConstraint.constant =  mainViewLeadingConstraint;
    self.transparentView.alpha = alpha;
    [self.view layoutIfNeeded];
}

-(void)filterDrawerAnimation: (int)drawerConstraint andMainViewConstraint:(int)mainViewConstraint andAlpha:(float)alpha {
    [self turnOffGestureRecognizers];
    [UIView animateWithDuration:0.5 animations:^{
        [self setFilterDrawerConstraint:drawerConstraint andMainViewLeadingConstraint:mainViewConstraint andAlpha:alpha];
    } completion:^(BOOL finished){
        if(finished && alpha == 0) {
            [self.transparentView setHidden:YES];
            [self turnOnGestureRecognizers];
            [self.categoriesNavigationController popToRootViewControllerAnimated:YES];
            [self.categoriesViewController currentFilters];
            [self.mapViewController setPins:CLLocationCoordinate2DMake(0, 0)];
            [self.brandsViewController setFilters];
        }
   }];
}

-(void)closeFilterDrawer {
    [self filterDrawerAnimation: -self.filterDrawerConstraint andMainViewConstraint:0 andAlpha:0];
    self.isFilterDrawerOpen = NO;
}

-(void)setFilterDrawerWidth:(CGFloat) width {
    self.filterDrawerConstraint = width;
    self.filterDrawerWidthConstraint.constant = width;
    self.filterDrawerTrailingConstraint.constant = -width;
    [self.view layoutIfNeeded];
}

-(void)turnOnGestureRecognizers {
    self.mainDrawerGestureRecognizer.enabled = YES;
    self.filterDrawerGestureRecognizer.enabled = YES;
}

-(void)turnOffGestureRecognizers {
    self.mainDrawerGestureRecognizer.enabled = NO;
    self.filterDrawerGestureRecognizer.enabled = NO;
}

- (void)changeLanguage {
    NSString *language = [Utils currentLanguage];
    if([language isEqualToString:@"bg"]) {
        [self.userDefaults setObject:[NSArray arrayWithObjects:@"en", @"bg", nil] forKey:@"AppleLanguages"];
        [[NSNotificationCenter defaultCenter] postNotificationName:@"LanguageNotification" object:self userInfo:@{@"language" : @"en"}];
    } else if([language isEqualToString:@"en"]) {
        [self.userDefaults setObject:[NSArray arrayWithObjects:@"bg", @"en", nil] forKey:@"AppleLanguages"];
        [[NSNotificationCenter defaultCenter] postNotificationName:@"LanguageNotification" object:self userInfo:@{@"language" : @"bg"}];
    } else {
        [self.userDefaults setObject:[NSArray arrayWithObjects:@"en", @"bg", nil] forKey:@"AppleLanguages"];
        [[NSNotificationCenter defaultCenter] postNotificationName:@"LanguageNotification" object:self userInfo:@{@"language" : @"bg"}];
    }
    
    self.searchTextField.text = [Utils translate:@"Brand search"];
    self.autocompleteTextField.text = [Utils translate:@"Map search"];
    
    [self.userDefaults synchronize];
    [self.mainDrawerTableView reloadData];
}

-(void)downloadContent {
    [self.categoriesViewController getCategories];
    self.brandsViewController.shouldAddBrands = YES;
    [self.brandsViewController getBrandsList];
    [self.brandsViewController.locationManager requestLocation];
    self.mapViewController.initialLaunch = YES;
    [self.mapViewController.locationManager requestLocation];
}

-(void)presentViewController:(UIViewController *)viewControllerToPresent animated:(BOOL)flag completion:(void (^)(void))completion {
    if([viewControllerToPresent isKindOfClass:[LoadAnimationViewController class]]) {
        [super presentViewController:viewControllerToPresent animated:flag completion:^ {
            if(completion != nil) {
                completion();
            }
            
            UIViewController *viewController = self.navigationController.viewControllers.lastObject;
            if([viewController isKindOfClass:[MainWebViewController class]]) {
                [((MainWebViewController *)viewController).webView loadRequest:((MainWebViewController *)viewController).request];
            }
        }];
    } else {
        [super presentViewController:viewControllerToPresent animated:flag completion:completion];
    }
}

-(id<UIViewControllerAnimatedTransitioning>) navigationController:(UINavigationController *)navigationController animationControllerForOperation:(UINavigationControllerOperation)operation fromViewController:(UIViewController *)fromVC toViewController:(UIViewController *)toVC {
    AnimationTransitioning *animationTransitioning = [[AnimationTransitioning alloc] init];
    if(operation == UINavigationControllerOperationPush) {
        if([toVC isKindOfClass:[CatalogueControlViewController class]]) {
            animationTransitioning.animationOption = UIViewAnimationOptionTransitionCrossDissolve;
        } else if([toVC isKindOfClass:[TermsAndConditionsViewController class]]) {
            animationTransitioning.animationOption = UIViewAnimationOptionTransitionCrossDissolve;
        } else if([toVC isKindOfClass:[FeedbackViewController class]]) {
            animationTransitioning.animationOption = UIViewAnimationOptionTransitionCrossDissolve;
        } else if([toVC isKindOfClass:[MainWebViewController class]]) {
            animationTransitioning.animationOption = UIViewAnimationOptionTransitionCrossDissolve;
        } else if([toVC isKindOfClass:[NotificationsHolderViewController class]]) {
            animationTransitioning.animationOption = UIViewAnimationOptionTransitionCrossDissolve;
        } else if([toVC isKindOfClass:[ProfileViewController class]]) {
            animationTransitioning.animationOption = UIViewAnimationOptionTransitionCrossDissolve;
        } else if([toVC isKindOfClass:[LoginViewController class]]) {
            animationTransitioning.animationOption = UIViewAnimationOptionTransitionCrossDissolve;
        }
    } else if(operation == UINavigationControllerOperationPop) {
        if([fromVC isKindOfClass:[CatalogueControlViewController class]]) {
            animationTransitioning.animationOption = UIViewAnimationOptionTransitionCrossDissolve;
        } else if([fromVC isKindOfClass:[TermsAndConditionsViewController class]]) {
            animationTransitioning.animationOption = UIViewAnimationOptionTransitionCrossDissolve;
        } else if([fromVC isKindOfClass:[FeedbackViewController class]]) {
            animationTransitioning.animationOption = UIViewAnimationOptionTransitionCrossDissolve;
        } else if([fromVC isKindOfClass:[MainWebViewController class]]) {
            animationTransitioning.animationOption = UIViewAnimationOptionTransitionCrossDissolve;
        } else if([fromVC isKindOfClass:[NotificationsHolderViewController class]]) {
            animationTransitioning.animationOption = UIViewAnimationOptionTransitionCrossDissolve;
        } else if([fromVC isKindOfClass:[ProfileViewController class]]) {
            animationTransitioning.animationOption = UIViewAnimationOptionTransitionCrossDissolve;
        } else if([fromVC isKindOfClass:[LoginViewController class]]) {
            animationTransitioning.animationOption = UIViewAnimationOptionTransitionCrossDissolve;
        }
    }
    return animationTransitioning;
}

-(void) prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    if([[segue destinationViewController] isKindOfClass:[UINavigationController class]]) {
        UIViewController *viewController = [((UINavigationController *)[segue destinationViewController]).viewControllers objectAtIndex:0];
        if([viewController isKindOfClass:[CategoriesViewController class]]) {
            self.categoriesNavigationController = (UINavigationController *)[segue destinationViewController];
            self.categoriesViewController = (CategoriesViewController *)viewController;
            ((CategoriesViewController *)viewController).mainNavigationController = self.navigationController;
            ((CategoriesViewController *)viewController).onLoad = ^(NSUInteger maxLength) {
                CGSize screenSize = [UIScreen mainScreen].bounds.size;
                CGFloat width = maxLength * 8 + 253;
                if(width >= screenSize.width) {
                    [self setFilterDrawerWidth:screenSize.width - 50];
                } else {
                    [self setFilterDrawerWidth:width];
                }
            };
        } else if([viewController isKindOfClass:[ContentHolderViewController class]]) {
            ((ContentHolderViewController *)viewController).mainNavigationController = self.navigationController;
            self.contentNavigationController = [segue destinationViewController];
        }
    } else if([[segue destinationViewController] isKindOfClass:[LoadAnimationViewController class]]) {
        self.animationViewController = (LoadAnimationViewController *)[segue destinationViewController];
        self.animationViewController.isInitialLaunch = self.isInitialLaunch;
        self.isInitialLaunch = NO;
    } else if([[segue destinationViewController] isKindOfClass:[MainWebViewController class]]) {
        NSString *urlString;
        if([[Utils currentLanguage] isEqualToString:@"bg"]) {
            urlString = @"http://www.unicreditbulbank.bg/bg/index.htm";
        } else {
            urlString = @"http://www.unicreditbulbank.bg/en/index.htm";
        }
        NSURL *url = [NSURL URLWithString:urlString];
        NSURLRequest *request = [NSURLRequest requestWithURL:url];
        ((MainWebViewController *)[segue destinationViewController]).request = request;

    } else if([[segue destinationViewController] isKindOfClass:[ErrorViewController class]]) {
        ErrorViewController *errorViewController = (ErrorViewController *)[segue destinationViewController];
        errorViewController.onDismiss = ^() {
            dispatch_after(dispatch_time(DISPATCH_TIME_NOW, 0.3 * NSEC_PER_SEC), dispatch_get_main_queue(), ^{
                [Utils displayAnimation];
            });
            [self downloadContent];
        };
    }
    
    if([[segue identifier] isEqualToString:@"CatalogueSegue"]) {
        CatalogueControlViewController *catalogueControlViewController = (CatalogueControlViewController *)[segue destinationViewController];
        if([sender isKindOfClass:[MainMenuViewController class]]) {
            long row = [self.mainDrawerTableView indexPathForSelectedRow].row;
            if(row == 0) {
                catalogueControlViewController.catalogueType = NEW;
            } else if(row == 1) {
                catalogueControlViewController.catalogueType = PROMOTIONS;
            } else if(row == 2) {
                catalogueControlViewController.catalogueType = FAVOURITES;
            } else if(row == 3) {
                catalogueControlViewController.catalogueType = SHOPPING_CENTERS;
            }
        } else {
            catalogueControlViewController.catalogueType = FAVOURITES;
        }
    }
}

@end
