    //
//  MapViewController.m
//  plus
//
//  Created by Виктор Георгиев on 8/3/16.
//  Copyright © 2016 Outsourcing IT Services Ltd. All rights reserved.
//

#import "AppDelegate.h"

#import "MainMenuViewController.h"
#import "MapViewController.h"
#import "BrandsViewController.h"
#import "PlaceViewController.h"
#import "CategoriesViewController.h"
#import "ContentHolderViewController.h"

#import "InfoAnnotationView.h"

#import "Filter.h"
#import "MapGestureRecognizer.h"
#import "PinAnnotation.h"
#import "PinView.h"
#import "URLRequests.h"

@interface MapViewController ()

@property (strong, nonatomic) MainMenuViewController *mainMenuViewController;

@property (weak, nonatomic) IBOutlet MKMapView *mapView;

@property (strong, nonatomic) CLLocation *lastKnownLocation;
@property (assign, nonatomic) CLLocationDistance radius;
@property (assign, nonatomic) CLLocationCoordinate2D currentMapLocation;
@property (assign, nonatomic) CLLocationCoordinate2D oldMapLocation;

@property (assign, nonatomic) BOOL getPins;
@property (assign, nonatomic) BOOL shouldSetPins;
@property (assign, nonatomic) BOOL hasUpdatedUserLocation;
@property (assign, nonatomic) BOOL fullyRendered;
@property (assign, nonatomic) BOOL isPinViewOpened;

@property (weak, nonatomic) IBOutlet UIButton *myLocationButton;

@property (strong, nonatomic) NSMutableArray<NSNumber *> *minLatitudeArray;
@property (strong, nonatomic) NSMutableArray<NSNumber *> *maxLatitudeArray;
@property (strong, nonatomic) NSMutableArray<NSNumber *> *minLongitudeArray;
@property (strong, nonatomic) NSMutableArray<NSNumber *> *maxLongitudeArray;

@property (assign, nonatomic) int placeId;
@property (assign, nonatomic) int categoryId;

@property (assign, nonatomic) float osVersion;

@end

@implementation MapViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    self.mainMenuViewController = (MainMenuViewController *)[((ContentHolderViewController *)[self.navigationController.viewControllers objectAtIndex:0]).mainNavigationController.viewControllers objectAtIndex:0];
    
    self.radius = 0;
    self.initialLaunch = YES;
    self.getPins = YES;
    self.shouldSetPins = NO;
    self.fullyRendered = NO;
    self.hasUpdatedUserLocation = NO;
    self.isMapVisible = NO;
    
    self.locationManager = [[CLLocationManager alloc] init];
    self.locationManager.delegate = self;
    
    self.osVersion = [[UIDevice currentDevice].systemVersion floatValue];
    
    self.minLatitudeArray = [[NSMutableArray alloc] initWithCapacity:0];
    self.maxLatitudeArray = [[NSMutableArray alloc] initWithCapacity:0];
    self.minLongitudeArray = [[NSMutableArray alloc] initWithCapacity:0];
    self.maxLongitudeArray = [[NSMutableArray alloc] initWithCapacity:0];
    
    [self.myLocationButton setTitle:[Utils translate:@"My location"] forState:UIControlStateNormal];

    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(changeLanguage) name:@"LanguageNotification" object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(didFinishLoading) name:@"FinishedLoading" object:nil];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
}

-(void)refreshMap {
    NSArray<id<MKAnnotation>> *annotationsArray = self.mapView.annotations;
    [self.mapView removeAnnotations:annotationsArray];
    [self.mapView addAnnotations:annotationsArray];
}

-(void)viewDidAppear:(BOOL)animated {
    [super viewDidAppear:animated];
    self.isMapVisible = YES;
    if(!self.initialLaunch) {
        [self setPins:self.currentMapLocation];
    }
}

-(void)viewDidDisappear:(BOOL)animated {
    [super viewDidDisappear:animated];
    self.isMapVisible = NO;
}

-(void)setFilters {
    if(self.isMapVisible) {
        if(self.navigationController.viewControllers.count == 2) {
            [self.navigationController popToViewController:[self.navigationController.viewControllers objectAtIndex:0] animated:YES];
        }
        self.getPins = YES;
        [self setPins:self.currentMapLocation];
    }
}

-(void)setLocation:(CLLocation *)location {
    [self didTapMap:self];
    if(location == nil) {
        location = [[CLLocation alloc] initWithLatitude:self.currentMapLocation.latitude longitude:self.currentMapLocation.longitude];
    }
    MKMapCamera *camera = [MKMapCamera cameraLookingAtCenterCoordinate:CLLocationCoordinate2DMake(location.coordinate.latitude, location.coordinate.longitude) fromEyeCoordinate:CLLocationCoordinate2DMake(location.coordinate.latitude, location.coordinate.longitude) eyeAltitude:600];
    [self.mapView setCamera:camera animated:YES];
}

-(void)didFinishLoading {
    self.shouldSetPins = YES;
    [self setPins:self.currentMapLocation];
}

- (void)mapView:(MKMapView *)mapView regionDidChangeAnimated:(BOOL)animated {
    if(self.hasUpdatedUserLocation) {
        MainMenuViewController *mainMenuViewController = (MainMenuViewController *)[((ContentHolderViewController *)[self.navigationController.viewControllers objectAtIndex:0]).mainNavigationController.viewControllers objectAtIndex:0];
        [mainMenuViewController textFieldShouldReturn:mainMenuViewController.autocompleteTextField];
        self.currentMapLocation = mapView.region.center;
        CLLocationCoordinate2D location = CLLocationCoordinate2DMake(self.currentMapLocation.latitude, self.currentMapLocation.longitude);
        BOOL newTask = NO;
        if(!self.initialLaunch) {
            for(int i = 0; i < self.minLatitudeArray.count; i++) {
                if((float)location.longitude > ((NSNumber *)[self.minLongitudeArray objectAtIndex:i]).floatValue * 0.995 && (float)location.longitude < ((NSNumber *)[self.maxLongitudeArray objectAtIndex:i]).floatValue * 0.995 && (float)location.latitude < ((NSNumber *)[self.maxLatitudeArray objectAtIndex:i]).floatValue * 0.995 && (float)location.latitude > ((NSNumber *)[self.minLatitudeArray objectAtIndex:i]).floatValue * 0.995) {
                    newTask = NO;
                    break;
                } else {
                    newTask = YES;
                }
            }
        } else {
            newTask = YES;
        }
        if(newTask) {
            self.shouldSetPins = YES;
            [self setPins:self.currentMapLocation];
        }
        
        if(self.pinView != nil && self.isPinViewOpened) {
            [self.pinView onClosePressed:self];
            self.pinView = nil;
            self.isPinViewOpened = NO;
        }
    }
}

-(void)locationManager:(CLLocationManager *)manager didUpdateLocations:(NSArray<CLLocation *> *)locations {
    self.lastKnownLocation = locations.lastObject;
    [Utils saveLocation:self.lastKnownLocation];
    MKMapCamera *camera = [MKMapCamera cameraLookingAtCenterCoordinate:CLLocationCoordinate2DMake(self.lastKnownLocation.coordinate.latitude, self.lastKnownLocation.coordinate.longitude) fromEyeCoordinate:CLLocationCoordinate2DMake(self.lastKnownLocation.coordinate.latitude, self.lastKnownLocation.coordinate.longitude) eyeAltitude:15000];

    double latitude = camera.centerCoordinate.latitude;
    double longitude = camera.centerCoordinate.longitude;
    if(latitude > -90 && latitude < 90 && longitude > - 180 && longitude < 180) {
        [self.mapView setCamera:camera animated:YES];
    }
    
    if(self.initialLaunch) {
        self.initialLaunch = NO;
        self.shouldSetPins = YES;
        [self setPins:self.lastKnownLocation.coordinate];
    }
}

-(void)locationManager:(CLLocationManager *)manager didFailWithError:(NSError *)error {
    
}

-(void)setPins:(CLLocationCoordinate2D)location {
    if(self.isMapVisible) {
        if(location.latitude == 0 && location.longitude == 0) {
            location = self.currentMapLocation;
        }
        
        NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
        NSString *currentFilters = (NSString *)[userDefaults valueForKey:@"currentFilters"];
        NSString *savedFilters = (NSString *)[userDefaults valueForKey:@"mapFilters"];
        if(![savedFilters isEqualToString:currentFilters] || self.initialLaunch || self.shouldSetPins) {
            self.minLatitudeArray = [[NSMutableArray alloc] initWithCapacity:0];
            self.maxLatitudeArray = [[NSMutableArray alloc] initWithCapacity:0];
            self.minLongitudeArray = [[NSMutableArray alloc] initWithCapacity:0];
            self.maxLongitudeArray = [[NSMutableArray alloc] initWithCapacity:0];
            [userDefaults setValue:currentFilters forKey:@"mapFilters"];
            [[NSUserDefaults standardUserDefaults] synchronize];
            if(!self.initialLaunch && !self.shouldSetPins) {
                if(![currentFilters isEqualToString:@""]) {
                    NSArray<NSString *> *filters = [currentFilters componentsSeparatedByString:@","];
                    for(id<MKAnnotation> annotation in self.mapView.annotations) {
                        if(annotation != self.mapView.userLocation) {
                            NSNumber *categoryId = ((PinAnnotation *)annotation).categoryId;
                            if(categoryId != nil) {
                                if(![filters containsObject:[NSString stringWithFormat:@"%d", categoryId.intValue]]) {
                                    [self.mapView removeAnnotation:annotation];
                                }
                            }
                        }
                    }
                }
            }
            [URLRequests getPins:location withFilters:currentFilters receive:^(NSData *data) {
                if(data != nil) {
                    float minLatitude = 180;
                    float maxLatitude = 0;
                    float minLongitude = 180;
                    float maxLongitude = 0;
                    NSError *error;
                    NSDictionary *objectDict = [NSJSONSerialization JSONObjectWithData:data options:0 error:&error];
                    NSArray *pinsArray = [objectDict objectForKey:@"data"];
                    for(NSDictionary *pin in pinsArray) {
                        CLLocationCoordinate2D coordinate = CLLocationCoordinate2DMake([[pin objectForKey:@"lat"] doubleValue], [[pin objectForKey:@"lng"] doubleValue]);
                        if(coordinate.latitude == 0 && coordinate.longitude == 0) {
                            coordinate = CLLocationCoordinate2DMake(0.00001, 0.00001);
                        }
                        if(coordinate.latitude < minLatitude) {
                            minLatitude = coordinate.latitude;
                        }
                        if(coordinate.latitude > maxLatitude) {
                            maxLatitude = coordinate.latitude;
                        }
                        if(coordinate.longitude < minLongitude) {
                            minLongitude = coordinate.longitude;
                        }
                        if(coordinate.longitude > maxLongitude) {
                            maxLongitude = coordinate.longitude;
                        }
                        
                        PinAnnotation *annotation = [[PinAnnotation alloc] init];
                        [annotation setCoordinate:coordinate];
                        NSNumber *categoryId = [pin objectForKey:@"category_id"];
                        NSNumber *locationId = [pin objectForKey:@"id"];
                        if(categoryId != nil) {
                            annotation.categoryId = categoryId;
                        } else {
                            annotation.categoryId = [NSNumber numberWithInt:-1];
                        }
                        
                        if(locationId != nil) {
                            annotation.locationId = locationId ;
                        } else {
                            annotation.locationId = [NSNumber numberWithInt:-1];
                        }
                        
                        if(![[self.mapView annotations] containsObject:annotation]) {
                            [self.mapView addAnnotation:annotation];
                        }
                    }
                    
                    [self.minLatitudeArray addObject:[NSNumber numberWithFloat:minLatitude]];
                    [self.maxLatitudeArray addObject:[NSNumber numberWithFloat:maxLatitude]];
                    [self.minLongitudeArray addObject:[NSNumber numberWithFloat:minLongitude]];
                    [self.maxLongitudeArray addObject:[NSNumber numberWithFloat:maxLongitude]];
                    
                    self.radius = 0;
                    self.shouldSetPins = NO;
                }
            }];
        }
    }
}

- (void)mapView:(MKMapView *)mapView didUpdateUserLocation:(MKUserLocation *)userLocation {
    if(!self.hasUpdatedUserLocation) {
        self.currentMapLocation = CLLocationCoordinate2DMake(userLocation.coordinate.latitude, userLocation.coordinate.longitude);
        self.hasUpdatedUserLocation = YES;
    }
}

-(void)mapViewDidFinishRenderingMap:(MKMapView *)mapView fullyRendered:(BOOL)fullyRendered {
    self.fullyRendered = fullyRendered; 
}

-(MKAnnotationView *) mapView:(MKMapView *)mapView viewForAnnotation:(id <MKAnnotation>) annotation {
    if(annotation == self.mapView.userLocation) {
        return nil;
    }
    
    static NSString *const pinIdentifier = @"mapPin";
    InfoAnnotationView *annotationView = nil;
    annotationView = (InfoAnnotationView *)[mapView dequeueReusableAnnotationViewWithIdentifier:pinIdentifier];
    if(annotationView == nil) {
        annotationView = [[InfoAnnotationView alloc] initWithAnnotation:annotation reuseIdentifier:pinIdentifier];
    }
    annotationView.annotation = annotation;
   
    int categoryId = ((PinAnnotation *)annotationView.annotation).categoryId.intValue;
    if(categoryId != -1) {
        NSArray<ItemCategory *> *categoriesArray = [Utils getCategories];
        NSString *urlString;
        for(ItemCategory *itemCategory in categoriesArray) {
            if(itemCategory.itemId.intValue == categoryId) {
                urlString = itemCategory.itemMapIcon;
                break;
            } else {
                BOOL found = NO;
                for(ItemCategory *subcategoryItem in itemCategory.itemSubcategories) {
                    if(subcategoryItem.itemId.intValue == categoryId) {

                        urlString = subcategoryItem.itemMapIcon;
                        found = YES;
                        break;
                    }
                }
                if(found) {
                    break;
                }
            }
        }
        UIImage *image = [URLRequests loadImage:urlString withFolder:nil receive:^(UIImage *image) {
        }];
        if(image != nil) {
            image = [Utils imageWithImage:image scaledToSize:CGSizeMake(40, 80)];
            annotationView.image = image;
        }
    }
    
    return annotationView;
}

- (void)mapView:(MKMapView *)mapView didSelectAnnotationView:(MKAnnotationView *)view {
    if(view.annotation != mapView.userLocation) {
        MainMenuViewController *mainMenuViewController = (MainMenuViewController *)[((ContentHolderViewController *)[self.navigationController.viewControllers objectAtIndex:0]).mainNavigationController.viewControllers objectAtIndex:0];
        [mainMenuViewController textFieldShouldReturn:mainMenuViewController.autocompleteTextField];
        int locationId = -1;
        self.categoryId = -1;
        NSArray<id <MKAnnotation>> *annotations = [mapView annotations];
        for(PinAnnotation *annotation in annotations) {
            MKAnnotationView* annotationView = [mapView viewForAnnotation: annotation];
            if([annotationView isEqual:view]) {
                locationId = annotation.locationId.intValue;
                self.categoryId = annotation.categoryId.intValue;
            }
            for(UIView *subView in annotationView.subviews) {
                if([subView isKindOfClass:[PinView class]]) {
                    [subView removeFromSuperview];
                }
            }
        }
        
        if(locationId != - 1) {
            self.isPinViewOpened = NO;
            [URLRequests getPin:locationId withLanguage:[Utils currentLanguage] receive:^(NSDictionary *dictionary) {
                CGFloat height = 210;
                PinView *pinView = [[[NSBundle mainBundle] loadNibNamed:@"PinView" owner:self options:nil] lastObject];
                [pinView loadAttributes];
                pinView.onOpenBrand = ^(BOOL performSegue) {
                    if(performSegue) {
                        [self performSegueWithIdentifier:@"SinglePlaceSegue" sender:self];
                    }
                };
                pinView.navigationController = self.navigationController;
                pinView.categoriesArray = [Utils getCategories];
                pinView.mapView = self.mapView;
                pinView.pinDictionary = dictionary;
                pinView.annotation = view.annotation;
                pinView.followed = [dictionary objectForKey:@"followed"];
                pinView.locationId = [NSNumber numberWithInt:locationId];
                self.placeId = locationId;
                pinView.categoryId = [NSNumber numberWithInt:self.categoryId];
                
//pin followed
                int followed = [pinView.followed intValue];
                if(followed == 1) {
                    [pinView.pinFollowButton setImage:[UIImage imageNamed:@"remove_from_favs"] forState:UIControlStateNormal];
                } else {
                    [pinView.pinFollowButton setImage:[UIImage imageNamed:@"add_to_favs"] forState:UIControlStateNormal];
                }
                
//pin category
                NSFetchRequest *request = [NSFetchRequest fetchRequestWithEntityName:@"Filter"];
                request.predicate = [NSPredicate predicateWithFormat:@"filterId == %d", self.categoryId];
                NSManagedObjectContext *managedObjectContext = ((AppDelegate *)[UIApplication sharedApplication].delegate).managedObjectContext;
                NSError *error;
                NSArray<Filter *> *filtersArray = (NSArray<Filter *> *)[managedObjectContext executeFetchRequest:request error:&error];
                if(filtersArray.count == 1) {
                    Filter *filter = [filtersArray objectAtIndex:0];
                    pinView.filter = filter;
                }
                
//pin distance
                CLLocation *pinLocation = [[CLLocation alloc] initWithLatitude:[pinView.annotation coordinate].latitude longitude:[pinView.annotation coordinate].longitude];
                pinView.pinDistance = [self.lastKnownLocation distanceFromLocation:pinLocation];
                
//pin labels
                [pinView setLabels];
                
                if(pinView.pinWebsiteButtonHeightConstraint.constant != 0) {
                    height += pinView.pinWebsiteButtonHeightConstraint.constant + 7;
                } else {
                    pinView.pinWebsiteButtonVerticalConstraint.constant = 0;
                }
                
//pin image
                NSDictionary *logoDictionary = [dictionary objectForKey:@"logo"];
                if(logoDictionary != nil) {
                    UIImage *image = [URLRequests loadImage:[logoDictionary objectForKey:@"image"] withFolder:nil receive:^(UIImage *image) {
                        if(image != nil) {
                            pinView.pinImageView.image = image;
                        }
                    }];
                    
                    if(image != nil) {
                        pinView.pinImageView.image = image;
                    }
                } else {
//                    pinView.pinImageViewLabel.text = [dictionary objectForKey:@"name"];
//                    pinView.pinPlaceLabel.text = @"";
//                    pinView.pinImageViewLabel.hidden = NO;
//                    height += 7;
                    pinView.pinImageView.image = [UIImage imageNamed:@"no-logo"];
                    pinView.pinImageView.backgroundColor = [Utils colorFromHexString:@"E6E7E8"];
                }
            
//pinView frame height
                for(UILabel *label in pinView.labelsArray) {
                    if(label.text != nil && ![label.text isEqualToString:@""]) {
                        CGFloat labelHeight = [label.text boundingRectWithSize:CGSizeMake(170, CGFLOAT_MAX) options:NSStringDrawingUsesLineFragmentOrigin attributes: @{NSFontAttributeName : label.font} context:nil].size.height;
                        height += labelHeight + 7;
                    }
                }
                pinView.frame = CGRectMake(-(213) / 2 + 20, -height - 5, 213, height);

//opening pinView
                pinView.alpha = 0;
                [view addSubview:pinView];
                [self.view layoutIfNeeded];
                [UIView animateWithDuration:1 animations:^{
                    pinView.alpha = 1;
                    [self.view layoutIfNeeded];
                }];
                self.pinView = pinView;
                
                MKCoordinateRegion region = [mapView convertRect:mapView.frame toRegionFromView:self.view];
                [mapView setCamera:[MKMapCamera cameraLookingAtCenterCoordinate:CLLocationCoordinate2DMake([view.annotation coordinate].latitude + region.span.latitudeDelta / 4, [view.annotation coordinate].longitude) fromEyeCoordinate:CLLocationCoordinate2DMake([view.annotation coordinate].latitude, [view.annotation coordinate].longitude) eyeAltitude:mapView.camera.altitude] animated:YES];
                dispatch_after(dispatch_time(DISPATCH_TIME_NOW, 3 * NSEC_PER_SEC), dispatch_get_main_queue(), ^{
                    self.isPinViewOpened = YES;
                });
            }];
        }
    }
}

- (IBAction)didTapMap:(id)sender {
    MainMenuViewController *mainMenuViewController = (MainMenuViewController *)[((ContentHolderViewController *)[self.navigationController.viewControllers objectAtIndex:0]).mainNavigationController.viewControllers objectAtIndex:0];
    [mainMenuViewController textFieldShouldReturn:mainMenuViewController.autocompleteTextField];
    NSArray<id <MKAnnotation>> *annotations = [self.mapView annotations];
    for(PinAnnotation *annotation in annotations) {
        MKAnnotationView* annotationView = [self.mapView viewForAnnotation: annotation];
        for(UIView *subView in annotationView.subviews) {
            if([subView isKindOfClass:[PinView class]]) {
                [subView removeFromSuperview];
            }
        }
    }
}

-(void)changeLanguage {
    [self.myLocationButton setTitle:[Utils translate:@"My location"] forState:UIControlStateNormal];
}

- (IBAction)onMyLocationPressed:(id)sender {
    [self.mapView setCenterCoordinate:self.mapView.userLocation.location.coordinate animated:YES];
    self.getPins = NO;
}

- (void)showList {
    if(self.fullyRendered) {
        UIScrollView *contentScrollView = ((ContentHolderViewController *)[self.navigationController.viewControllers objectAtIndex:0]).contentScrollView;
        [UIView animateWithDuration:0.3 animations:^{
            contentScrollView.contentOffset = CGPointMake(0, contentScrollView.contentOffset.y);
        }];
        
    }
}

#pragma mark - Navigation

- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    if([[segue destinationViewController] isKindOfClass:[PlaceViewController class]]) {
        PlaceViewController *placeViewController = (PlaceViewController *)[segue destinationViewController];
        placeViewController.placeId = [NSNumber numberWithInt:self.placeId];
        placeViewController.headerFieldType = @"autocomplete";
    }
}


@end
