//
//  NotificationsViewController.m
//  plus
//
//  Created by Виктор Георгиев on 9/12/16.
//  Copyright © 2016 Outsourcing IT Services Ltd. All rights reserved.
//

#import "AppDelegate.h"

#import "NotificationsViewController.h"
#import "MainMenuViewController.h"
#import "SingleBrandViewController.h"
#import "PlaceViewController.h"
#import "NotificationsHolderViewController.h"

#import "NotificationTableViewCell.h"

#import "URLRequests.h"
#import "Utils.h"
#import "Notification.h"
#import "NotificationTapGestureRecognizer.h"

@interface NotificationsViewController ()

@property (weak, nonatomic) IBOutlet UITableView *notificationsTableView;

@property (strong, nonatomic) NSMutableArray<NSNumber *>* rowsIndexArray;

@end

@implementation NotificationsViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    
    [[UIApplication sharedApplication] setApplicationIconBadgeNumber: 0];
    [[UIApplication sharedApplication] cancelAllLocalNotifications];
}

-(void)viewDidAppear:(BOOL)animated {
    [super viewDidAppear:animated];
    

    
    [URLRequests getNotifications:^(long newNotifications) {
        NSManagedObjectContext *managedObjectContext = ((AppDelegate *)[UIApplication sharedApplication].delegate).managedObjectContext;
        NSError *error;
        NSFetchRequest *request = [NSFetchRequest fetchRequestWithEntityName:@"Notification"];
        request.sortDescriptors = @[[NSSortDescriptor sortDescriptorWithKey:@"notificationId" ascending:NO]];
        self.notificationsArray = (NSArray<Notification *> *)[managedObjectContext executeFetchRequest:request error:&error];
        
        [self.notificationsTableView reloadData];
        
        self.rowsIndexArray = [[NSMutableArray alloc] initWithCapacity:self.notificationsArray.count];
        for(int i = 0; i < self.notificationsArray.count; i++) {
            [self.rowsIndexArray setObject:[NSNumber numberWithInt:0] atIndexedSubscript:i];
        }
        
        [Utils dismissAnimation];
    }];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
}

- (CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
    Notification *notification = [self.notificationsArray objectAtIndex:indexPath.row];
    CGFloat height = [notification.notificationName boundingRectWithSize:CGSizeMake([UIScreen mainScreen].bounds.size.width - 125, CGFLOAT_MAX) options:NSStringDrawingUsesLineFragmentOrigin attributes:@{NSFontAttributeName : [UIFont fontWithName:@"Unicredit CY" size:18]} context:nil].size.height;
    height += 20;
    
    if(![notification.notificationImage isEqualToString:@""]) {
        UIImage *image = [URLRequests loadImage:notification.notificationImage withFolder:nil receive:^(UIImage *image) {
            for(NSIndexPath *indexPathForImage in [self.notificationsTableView indexPathsForVisibleRows]) {
                if([[self.notificationsArray objectAtIndex:indexPathForImage.row].notificationImage isEqualToString:notification.notificationImage]) {
                    [tableView reloadData];
                }
            }
        }];
        if(image != nil) {
            float scale = ([UIScreen mainScreen].bounds.size.width - 40) / image.size.width;
            height += image.size.height * scale + 10;
        }
    }
    
    if(![notification.notificationContent isEqualToString:@""]) {
        if([self.rowsIndexArray objectAtIndex:indexPath.row].intValue == 0) {
            height += 62.4;
        } else {
            height += [notification.notificationContent boundingRectWithSize:CGSizeMake([UIScreen mainScreen].bounds.size.width - 80, CGFLOAT_MAX) options:NSStringDrawingUsesLineFragmentOrigin attributes:@{NSFontAttributeName : [UIFont fontWithName:@"PTSans-Regular" size:16]} context:nil].size.height + 25;
        }
    } else if(![notification.notificationDescription isEqualToString:@""]) {
        UILabel *label = [[UILabel alloc] init];
        label.lineBreakMode = NSLineBreakByWordWrapping;
        if([self.rowsIndexArray objectAtIndex:indexPath.row].intValue == 0) {
            label.numberOfLines = 3;
        } else {
            label.numberOfLines = 0;
            height += 50;
        }
        NSAttributedString * attributedString = [[NSAttributedString alloc] initWithData:[notification.notificationDescription dataUsingEncoding:NSUnicodeStringEncoding] options:@{ NSDocumentTypeDocumentAttribute: NSHTMLTextDocumentType } documentAttributes:nil error:nil];
        label.attributedText = attributedString;
        height += [label textRectForBounds:CGRectMake(0, 0, ([UIScreen mainScreen].bounds.size.width - 80), CGFLOAT_MAX) limitedToNumberOfLines:label.numberOfLines].size.height;
        height += 20;
    }
    
    return height + 20;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return self.notificationsArray.count;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    NotificationTableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"Cell"];
    cell.notificationsViewController = self;
    Notification *notification = [self.notificationsArray objectAtIndex:indexPath.row];
    if([notification.notificationObject isEqualToString:@"promotion"]) {
        cell.notificationTitleLabel.text = [NSString stringWithFormat:@"%@ %@", [Utils translate:@"New promotion"], notification.notificationName];
    } else {
        cell.notificationTitleLabel.text = [NSString stringWithFormat:@"%@", notification.notificationName];
    }
    
    CGFloat widthToTakeOut = 0;
    if(notification.notificationIsRead.intValue == 0) {
        cell.notificationHeaderImageViewLeadingConstraint.constant = 15;
        cell.notificationHeaderImageViewWidthConstraint.constant = 20;
        cell.notificationHeaderView.backgroundColor = [UIColor colorWithRed:70.0 / 255 green:173.0 / 255 blue:198 / 255.0 alpha:1];
        widthToTakeOut = 125;
    } else {
        cell.notificationHeaderImageViewLeadingConstraint.constant = 0;
        cell.notificationHeaderImageViewWidthConstraint.constant = 0;
        cell.notificationHeaderView.backgroundColor = [UIColor colorWithRed:135.0 / 255 green:146.0 / 255 blue:152.0 / 255 alpha:1];
        widthToTakeOut = 35;
    }
    
    if([self.rowsIndexArray objectAtIndex:indexPath.row].intValue == 1) {
        cell.notificationHeaderImageView.image = [UIImage imageNamed:@"notifications_fold"];
        cell.notificationDescriptionLabelBottomConstraint.constant = 50;
    } else {
        cell.notificationHeaderImageView.image = [UIImage imageNamed:@"notifications_collapse"];
        cell.notificationDescriptionLabelBottomConstraint.constant = 10;
    }
    
    CGFloat height = [cell.notificationTitleLabel.text boundingRectWithSize:CGSizeMake([UIScreen mainScreen].bounds.size.width - widthToTakeOut, CGFLOAT_MAX) options:NSStringDrawingUsesLineFragmentOrigin attributes:@{NSFontAttributeName : cell.notificationTitleLabel.font} context:nil].size.height;
    
    cell.notificationTitleViewHeightConstraint.constant = height + 20;
    
    if(![notification.notificationImage isEqualToString:@""]) {
        UIImage *image = [URLRequests loadImage:notification.notificationImage withFolder:nil receive:^(UIImage *image) {
            for(NSIndexPath *indexPathForImage in [self.notificationsTableView indexPathsForVisibleRows]) {
                if([[self.notificationsArray objectAtIndex:indexPathForImage.row].notificationImage isEqualToString:notification.notificationImage]) {
                    float scale = ([UIScreen mainScreen].bounds.size.width - 40) / image.size.width;
                    cell.notificationImageViewHeightConstraint.constant = image.size.height * scale;
                    ((NotificationTableViewCell *)[self.notificationsTableView cellForRowAtIndexPath:indexPathForImage]).notificationImageView.image = image;
                }
            }
        }];
        if(image != nil) {
            float scale = ([UIScreen mainScreen].bounds.size.width - 40) / image.size.width;
            cell.contentView.frame = CGRectMake(0, 0, cell.bounds.size.width, cell.notificationTitleViewHeightConstraint.constant + image.size.height * scale + 40);
            cell.notificationImageViewHeightConstraint.constant = image.size.height * scale;
        } else {
            cell.notificationImageViewHeightConstraint.constant = 0;
        }
        cell.notificationImageView.image = image;
        
    } else {
        cell.notificationImageView.image = nil;
        cell.notificationImageViewHeightConstraint.constant = 0;
    }
    
    if(![notification.notificationContent isEqualToString:@""]) {
        cell.notificationDescriptionLabel.text = notification.notificationContent;
    } else if(![notification.notificationDescription isEqualToString:@""]) {
        NSAttributedString * attributedString = [[NSAttributedString alloc] initWithData:[notification.notificationDescription dataUsingEncoding:NSUnicodeStringEncoding] options:@{NSDocumentTypeDocumentAttribute: NSHTMLTextDocumentType} documentAttributes:nil error:nil];
        
        cell.notificationDescriptionLabel.attributedText = attributedString;
    }
    
    cell.notificationButton.tag = -1;
    cell.notificationButton.tag = notification.notificationObjectId.integerValue;
    if([notification.notificationObject isEqualToString:@"brand"]) {
        [cell.notificationButton setTitle:[NSString stringWithFormat:@"%@ %@", [Utils translate:@"See brand"], notification.notificationName] forState:UIControlStateNormal];
        cell.isBrand = YES;
    } else if([notification.notificationObject isEqualToString:@"location"]) {
        [cell.notificationButton setTitle:[NSString stringWithFormat:@"%@ %@", [Utils translate:@"See location"], notification.notificationName] forState:UIControlStateNormal];
        cell.isBrand = NO;
    } else if([notification.notificationObject isEqualToString:@"promotion"]) {
        if(notification.notificationBrand.intValue == 1) {
            [cell.notificationButton setTitle:[NSString stringWithFormat:@"%@ %@", [Utils translate:@"See brand"], notification.notificationName] forState:UIControlStateNormal];
            cell.isBrand = YES;
        } else {
            [cell.notificationButton setTitle:[NSString stringWithFormat:@"%@ %@", [Utils translate:@"See location"], notification.notificationName] forState:UIControlStateNormal];
            cell.isBrand = NO;
        }
    }
    
    NotificationTapGestureRecognizer *tap = [[NotificationTapGestureRecognizer alloc] initWithTarget:self action:@selector(changeRowHeight:)];
    tap.index = indexPath.row;
    [cell.notificationHeaderView addGestureRecognizer:tap];
    
    return cell;
}

-(void)changeRowHeight:(NotificationTapGestureRecognizer *)tap {
    NotificationTableViewCell *cell = [self.notificationsTableView cellForRowAtIndexPath:[NSIndexPath indexPathForRow:[NSNumber numberWithLong:tap.index].integerValue inSection:0]];
    NSNumber *rowIndex = [self.rowsIndexArray objectAtIndex:tap.index];
    CATransition *fade = [CATransition animation];
    [cell.notificationHeaderImageView.layer addAnimation:fade forKey:@"fade"];
    if(rowIndex.intValue == 1) {
        [self.rowsIndexArray setObject:[NSNumber numberWithInt:0] atIndexedSubscript:tap.index];
    } else if(rowIndex.intValue == 0) {
        [self.rowsIndexArray setObject:[NSNumber numberWithInt:1] atIndexedSubscript:tap.index];
    }
    
    CGFloat height = cell.frame.size.height;
    CGFloat newHeight = [self tableView:self.notificationsTableView heightForRowAtIndexPath:[NSIndexPath indexPathForRow:tap.index inSection:0]];
    [cell changeRowHeight:newHeight - height];

    height = newHeight - height;
    for(NSIndexPath *visibleRow in self.notificationsTableView.indexPathsForVisibleRows) {
        if(visibleRow.row > tap.index) {
            NotificationTableViewCell *cell = [self.notificationsTableView cellForRowAtIndexPath:[NSIndexPath indexPathForRow:visibleRow.row inSection:0]];
            [self.view layoutIfNeeded];
             [UIView animateWithDuration:0.25 animations:^{
                CGRect rect = cell.frame;
                rect.origin.y += height;
                cell.frame = rect;
                [self.view layoutIfNeeded];
            }];
        }
    }
    [self.notificationsTableView beginUpdates];
    [self.notificationsTableView endUpdates];
}

- (IBAction)onBackPressed:(id)sender {
    [self.navigationController popViewControllerAnimated:YES];
}


#pragma mark - Navigation

- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    if([[segue destinationViewController] isKindOfClass:[SingleBrandViewController class]]) {
        SingleBrandViewController *singleBrandViewController = (SingleBrandViewController *)[segue destinationViewController];
        singleBrandViewController.brandId = [NSNumber numberWithLong:self.brandId];
        singleBrandViewController.isFromMainDrawer = YES;
    } else if([[segue destinationViewController] isKindOfClass:[PlaceViewController class]]) {
        PlaceViewController *placeViewController = (PlaceViewController *)[segue destinationViewController];
        placeViewController.placeId = [NSNumber numberWithLong:self.placeId];
        placeViewController.headerTitle = @"";
        placeViewController.headerFieldType = @"";
        //placeViewController.oldTitle = [self.shoppingCenterDictionary objectForKey:@"name"];
        placeViewController.isFromMainDrawer = YES;
    }
}


@end
