//
//  PinView.m
//  plus
//
//  Created by Виктор Георгиев on 8/3/16.
//  Copyright © 2016 Outsourcing IT Services Ltd. All rights reserved.
//

#define DURATION 0.5f

#import "AppDelegate.h"

#import "WebViewController.h"
#import "ContentHolderViewController.h"
#import "BrandsViewController.h"

#import "PinView.h"

#import "URLRequests.h"
#import "Utils.h"
#import "Brand.h"

@interface PinView()

@end

@implementation PinView

- (void)loadAttributes {
    [self.pinImageView setUserInteractionEnabled:YES];
    UITapGestureRecognizer *imageTap = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(openBrand)];
    [imageTap setNumberOfTapsRequired:1];
    [imageTap setNumberOfTouchesRequired:1];
    [self.pinImageView addGestureRecognizer:imageTap];
    
    [self.pinPlaceLabel setUserInteractionEnabled:YES];
    UITapGestureRecognizer *labelTap = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(openBrand)];
    [labelTap setNumberOfTapsRequired:1];
    [labelTap setNumberOfTouchesRequired:1];
    [self.pinPlaceLabel addGestureRecognizer:labelTap];
    
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(changeLanguage) name:@"LanguageNotification" object:nil];
}

- (void)drawRect:(CGRect)rect {
    self.pinImageView.layer.borderWidth = 2.0;
    self.pinImageView.layer.borderColor = [UIColor colorWithRed:235 / 255.0 green:235 / 255.0 blue:235 / 255.0 alpha:1].CGColor;    
}

- (IBAction)onClosePressed:(id)sender {
    [self layoutIfNeeded];
    [UIView animateWithDuration:0.5 animations:^{
        self.alpha = 0;
    } completion:^(BOOL finished) {
        [self.mapView deselectAnnotation:self.annotation animated:NO];
        [self removeFromSuperview];
    }];
}

- (IBAction)onFollowButtonPressed:(id)sender {
    [URLRequests followPlace:self.followed.intValue withLocationId:self.locationId.intValue receive:^(BOOL success) {
        if(success) {
            BrandsViewController *brandsViewController = ((ContentHolderViewController *)[self.navigationController.viewControllers objectAtIndex:0]).brandsViewController;
            if(self.followed.intValue == 1) {
                [self unfollowAnimation:self.pinFollowButton];
                self.followed = 0;
                for(Brand *brand in brandsViewController.brandsArray) {
                    if(brand.brandId.intValue == self.locationId.intValue) {
                        brand.isFollowed = NO;
                        break;
                    }
                }
            } else {
                [self followAnimation:self.pinFollowButton];
                self.followed = [NSNumber numberWithInt:1];
                for(Brand *brand in brandsViewController.brandsArray) {
                    if(brand.brandId.intValue == self.locationId.intValue) {
                        brand.isFollowed = YES;
                        break;
                    }
                }
            }
            [brandsViewController.brandsCollectionView reloadData];
        }
    }];
}

- (void)setLabels {
    self.labelsArray = [[NSMutableArray alloc] initWithCapacity:0];
    
//pin category
    if([[Utils currentLanguage] isEqualToString:@"bg"]) {
        self.pinCategoryLabel.text = self.filter.filterBgName;
    } else {
        self.pinCategoryLabel.text = self.filter.filterEnName;
    }
    
    NSString *urlString;
    for(ItemCategory *itemCategory in self.categoriesArray) {
        if(itemCategory.itemId.intValue == self.categoryId.intValue) {
            self.pinCategoryLabel.textColor = [Utils colorFromHexString:itemCategory.itemCategoryPinColor];
            urlString = itemCategory.itemCategoryPinIcon;
        } else {
            for(ItemCategory *subItemCategory in itemCategory.itemSubcategories) {
                if(subItemCategory.itemId.intValue == self.categoryId.intValue) {
                    urlString = subItemCategory.itemCategoryPinIcon;
                    self.pinCategoryLabel.textColor = [Utils colorFromHexString:subItemCategory.itemCategoryPinColor];
                }
            }
        }
    }

    if(self.pinCategoryLabel.text == nil) {
        self.pinCategoryImageViewHeightConstraint.constant = 0;
        self.pinCategoryImageViewVerticalConstraint.constant = 0;
    } else {
        self.pinCategoryImageView.image = [URLRequests loadImage:urlString withFolder:nil receive:nil];
    }
  
//pin place
    self.pinPlaceLabel.text = [self.pinDictionary objectForKey:@"name"];

//pin promotion
    self.pinPromotionLabel.attributedText = [self promotionString];
    
    if([[self.pinDictionary objectForKey:@"website"] isKindOfClass:[NSString class]]) {
        [self.pinWebsiteButton setTitle:[self.pinDictionary objectForKey:@"website"] forState:UIControlStateNormal];
        CGFloat height = [[self.pinDictionary objectForKey:@"website"] boundingRectWithSize:CGSizeMake(150, CGFLOAT_MAX) options:NSStringDrawingUsesLineFragmentOrigin attributes: @{NSFontAttributeName : self.pinWebsiteButton.titleLabel.font} context:nil].size.height;
        self.pinWebsiteButtonHeightConstraint.constant = height;
    } else {
        self.pinWebsiteButtonHeightConstraint.constant = 0;
    }
//pin address
    self.pinAddressLabel.text = [[self.pinDictionary objectForKey:@"address"] objectForKey:@"address1"];
  
//pin distance
    if(self.pinDistance < 1000) {
        self.pinDistanceLabel.text = [NSString stringWithFormat:@"%d %@", (int)self.pinDistance, [Utils translate:@"Meter"]];
    } else {
        self.pinDistanceLabel.text = [NSString stringWithFormat:@"%.2f %@", self.pinDistance / 1000, [Utils translate:@"Kilometer"]];
    }
    
    [self.labelsArray addObject:self.pinCategoryLabel];
    [self.labelsArray addObject:self.pinPlaceLabel];
    [self.labelsArray addObject:self.pinAddressLabel];
    [self.labelsArray addObject:self.pinDistanceLabel];
}

- (void)changeLanguage {
    NSString *language = [Utils currentLanguage];
    if([language isEqualToString:@"bg"]) {
        self.pinCategoryLabel.text = self.filter.filterBgName;
    } else {
        self.pinCategoryLabel.text = self.filter.filterEnName;
    }
    
    self.pinPromotionLabel.attributedText = [self promotionString];
    
    [URLRequests getPin:self.locationId.intValue withLanguage:language receive:^(NSDictionary *dictionary) {
        self.pinAddressLabel.text = [[dictionary objectForKey:@"address"] objectForKey:@"address1"];
    }];
    
    if(self.pinDistance < 1000) {
        self.pinDistanceLabel.text = [NSString stringWithFormat:@"%d %@", (int)self.pinDistance, [Utils translate:@"Meter"]];
    } else {
        self.pinDistanceLabel.text = [NSString stringWithFormat:@"%.2f %@", self.pinDistance / 1000, [Utils translate:@"Kilometer"]];
    }
}

- (NSMutableAttributedString *)promotionString {
    if([[self.pinDictionary objectForKey:@"unicredit_promotion"] isKindOfClass:[NSString class]]) {
        NSString *promotion = [self.pinDictionary objectForKey:@"unicredit_promotion"];
        NSString *forString = [Utils translate:@"For"];
        NSString *currency = [Utils translate:@"Currency"];
        
        NSMutableAttributedString *promotionString = [[NSMutableAttributedString alloc] initWithString:[NSString stringWithFormat:@"%@ %@ 1 %@", promotion, forString, currency]];
        [promotionString addAttribute:NSFontAttributeName value:[UIFont fontWithName:@"PTSans-Bold" size:14] range:NSMakeRange(0, promotion.length)];
        [promotionString addAttribute:NSFontAttributeName value:[UIFont fontWithName:@"PTSans-Regular" size:14] range:NSMakeRange(promotion.length + 1, forString.length)];
        [promotionString addAttribute:NSFontAttributeName value:[UIFont fontWithName:@"PTSans-Bold" size:14] range:NSMakeRange(promotion.length + forString.length + 2, 1)];
        [promotionString addAttribute:NSFontAttributeName value:[UIFont fontWithName:@"PTSans-Regular" size:14] range:NSMakeRange(promotionString.length - currency.length, 3)];
        
        return promotionString;
    } else {
        return [[NSMutableAttributedString alloc] initWithString:@""];
    }
}

- (void)openBrand {
    self.onOpenBrand(YES);
    [self onClosePressed:self];
}

- (IBAction)onWebsitePressed:(id)sender {
    WebViewController *webViewController = [[UIStoryboard storyboardWithName:@"Main" bundle:nil] instantiateViewControllerWithIdentifier:@"WebViewController"];
    NSURL *url = [NSURL URLWithString:self.pinWebsiteButton.titleLabel.text];
    NSURLRequest *request = [NSURLRequest requestWithURL:url];
    webViewController.request = request;
    webViewController.isFromMainDrawer = NO;
    [self.navigationController pushViewController:webViewController animated:YES];
}

- (void)animationDidStop:(CAAnimation *)theAnimation finished:(BOOL)flag {
    if(self.followed) {
        [self.pinFollowButton setImage:[UIImage imageNamed:@"remove_from_favs"] forState:UIControlStateNormal];
    } else {
        [self.pinFollowButton setImage:[UIImage imageNamed:@"add_to_favs"] forState:UIControlStateNormal];
    }
}

-(void)followAnimation:(UIButton *)button {
    CABasicAnimation *rotate = [CABasicAnimation animationWithKeyPath:@"transform.rotation"];
    rotate.fromValue = [NSNumber numberWithFloat:0];
    rotate.toValue = [NSNumber numberWithFloat:-6.5];
    rotate.duration = DURATION;
    rotate.repeatCount = 1;
    CABasicAnimation *zoomOut = [CABasicAnimation animationWithKeyPath:@"transform.scale"];
    zoomOut.fromValue = [NSNumber numberWithFloat:1];
    zoomOut.toValue = [NSNumber numberWithFloat:0.2];
    zoomOut.duration = DURATION / 2;
    zoomOut.repeatCount = 1;
    CABasicAnimation *zoomIn = [CABasicAnimation animationWithKeyPath:@"transform.scale"];
    zoomIn.fromValue = [NSNumber numberWithFloat:0.2];
    zoomIn.toValue = [NSNumber numberWithFloat:1];
    zoomIn.duration = DURATION / 4;
    zoomIn.beginTime = CACurrentMediaTime() + DURATION * 3 / 4;
    zoomIn.repeatCount = 1;
    CABasicAnimation *fadeOut = [CABasicAnimation animationWithKeyPath:@"opacity"];
    fadeOut.delegate = self;
    fadeOut.fromValue = [NSNumber numberWithFloat:1.0];
    fadeOut.toValue = [NSNumber numberWithFloat:0.0];
    fadeOut.duration = DURATION / 4;
    fadeOut.repeatCount = 1;
    CABasicAnimation *continueFade = [CABasicAnimation animationWithKeyPath:@"opacity"];
    continueFade.fromValue = [NSNumber numberWithFloat:0];
    continueFade.toValue = [NSNumber numberWithFloat:0.0];
    continueFade.beginTime = CACurrentMediaTime() + DURATION / 4;
    continueFade.duration = DURATION / 2;
    continueFade.repeatCount = 1;
    
    [button.layer addAnimation:rotate forKey:nil];
    [button.layer addAnimation:zoomOut forKey:nil];
    [button.layer addAnimation:zoomIn forKey:nil];
    [button.layer addAnimation:fadeOut forKey:nil];
    [button.layer addAnimation:continueFade forKey:nil];
}

-(void)unfollowAnimation:(UIButton *)button {
    CABasicAnimation *zoomOut = [CABasicAnimation animationWithKeyPath:@"transform.scale"];
    zoomOut.delegate = self;
    zoomOut.fromValue = [NSNumber numberWithFloat:1];
    zoomOut.toValue = [NSNumber numberWithFloat:0.05];
    zoomOut.duration = DURATION / 2;
    zoomOut.repeatCount = 1;
    CABasicAnimation *zoomIn = [CABasicAnimation animationWithKeyPath:@"transform.scale"];
    zoomIn.fromValue = [NSNumber numberWithFloat:0.05];
    zoomIn.toValue = [NSNumber numberWithFloat:1];
    zoomIn.duration = DURATION / 2 + DURATION / 10;
    zoomIn.beginTime = CACurrentMediaTime() + DURATION / 2 - DURATION / 10;
    zoomIn.repeatCount = 1;
    
    [button.layer addAnimation:zoomOut forKey:nil];
    [button.layer addAnimation:zoomIn forKey:nil];
}



@end
