//
//  SinglePlaceViewController.m
//  plus
//
//  Created by Виктор Георгиев on 7/20/16.
//  Copyright © 2016 Outsourcing IT Services Ltd. All rights reserved.
//

#define FAVOURITES 2

#import "AppDelegate.h"
#import <FBSDKCoreKit/FBSDKCoreKit.h>
#import <FBSDKShareKit/FBSDKShareKit.h>

#import "PlaceViewController.h"
#import "MainMenuViewController.h"
#import "ContentHolderViewController.h"
#import "WebViewController.h"
#import "GalleryViewController.h"
#import "CatalogueViewController.h"
#import "NotificationsViewController.h"
#import "NotificationsHolderViewController.h"
#import "ShoppingCenterViewController.h"

#import "PromotionTableViewCell.h"

#import "InfoAnnotationView.h"
#import "NewPlace.h"
#import "Promotion.h"
#import "ShoppingCenter.h"
#import "ItemCategory.h"
#import "PinAnnotation.h"
#import "URLRequests.h"
#import "Utils.h"

@interface PlaceViewController ()

@property (assign, nonatomic) CLLocationCoordinate2D placeCoordinate;

@property (strong, nonatomic) NSDictionary *placeDictionary;

@property (strong, nonatomic) ItemCategory *category;

@property (strong, nonatomic) Filter *filter;

@property (weak, nonatomic) IBOutlet UIScrollView *placeScrollView;
@property (weak, nonatomic) IBOutlet UIScrollView *imagesScrollView;

@property (weak, nonatomic) IBOutlet MKMapView *mapView;

@property (strong, nonatomic) NSArray<NSDictionary *> *promotionsArray;
@property (strong, nonatomic) NSMutableArray<NSNumber *> *promotionsTableViewRows;

@property (weak, nonatomic) IBOutlet UIView *promotionView;

@property (weak, nonatomic) IBOutlet UIButton *leftArrowButton;
@property (weak, nonatomic) IBOutlet UIButton *rightArrowButton;
@property (weak, nonatomic) IBOutlet UIButton *backButton;
@property (weak, nonatomic) IBOutlet UIButton *websiteButton;
@property (weak, nonatomic) IBOutlet UIButton *facebookButton;
@property (weak, nonatomic) IBOutlet UIButton *mapControlButton;
@property (weak, nonatomic) IBOutlet UIButton *phoneButton;
@property (weak, nonatomic) IBOutlet UIButton *mobilePhoneButton;
@property (weak, nonatomic) IBOutlet UIButton *addToFavouritesButton;
@property (weak, nonatomic) IBOutlet UIButton *shareButton;

@property (weak, nonatomic) IBOutlet UIImageView *shadowImageView;
@property (weak, nonatomic) IBOutlet UIImageView *categoryImageView;
@property (weak, nonatomic) IBOutlet UIImageView *addressImageView;
@property (weak, nonatomic) IBOutlet UIImageView *distanceImageView;

@property (weak, nonatomic) IBOutlet UILabel *placeNameLabel;
@property (weak, nonatomic) IBOutlet UILabel *promotionLabel;
@property (weak, nonatomic) IBOutlet UILabel *categoryLabel;
@property (weak, nonatomic) IBOutlet UILabel *addressLabel;
@property (weak, nonatomic) IBOutlet UILabel *distanceLabel;
@property (weak, nonatomic) IBOutlet UILabel *descriptionLabel;

@property (weak, nonatomic) IBOutlet NSLayoutConstraint *categoryImageViewVerticalConstraint;
@property (weak, nonatomic) IBOutlet NSLayoutConstraint *categoryLabelVerticalConstraint;
@property (weak, nonatomic) IBOutlet NSLayoutConstraint *promotionViewWidthConstraint;
@property (weak, nonatomic) IBOutlet NSLayoutConstraint *distanceLabelWidthConstraint;
@property (weak, nonatomic) IBOutlet NSLayoutConstraint *websiteButtonHeightConstraint;
@property (weak, nonatomic) IBOutlet NSLayoutConstraint *websiteButtonVerticalConstraint;
@property (weak, nonatomic) IBOutlet NSLayoutConstraint *facebookButtonHeightConstraint;
@property (weak, nonatomic) IBOutlet NSLayoutConstraint *facebookButtonVerticalConstraint;
@property (weak, nonatomic) IBOutlet NSLayoutConstraint *phoneButtonVerticalConstraint;
@property (weak, nonatomic) IBOutlet NSLayoutConstraint *phoneButtonHeightConstraint;
@property (weak, nonatomic) IBOutlet NSLayoutConstraint *mobilePhoneButtonVerticalConstraint;
@property (weak, nonatomic) IBOutlet NSLayoutConstraint *mobilePhoneButtonHeightConstraint;
@property (weak, nonatomic) IBOutlet NSLayoutConstraint *descriptionLabelHeightConstraint;
@property (weak, nonatomic) IBOutlet NSLayoutConstraint *mapViewHeightConstraint;
@property (weak, nonatomic) IBOutlet NSLayoutConstraint *mapViewLeadingConstraint;
@property (weak, nonatomic) IBOutlet NSLayoutConstraint *mapViewTrailingConstraint;
@property (weak, nonatomic) IBOutlet NSLayoutConstraint *mapViewBottomConstraint;

@property (assign, nonatomic) int imagesCount;
@property (assign, nonatomic) int currentPage;
@property (assign, nonatomic) int followed;

@property (assign, nonatomic) float placeScrollViewOffset;

@property (assign, nonatomic) BOOL initialLaunch;
@property (assign, nonatomic) BOOL isMapOpened;

@end

@implementation PlaceViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    if(self.isFromMainDrawer) {
        self.backButton.userInteractionEnabled = YES;
    }

    self.initialLaunch = YES;
    
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(changeLanguage:) name:@"LanguageNotification" object:nil];
}

-(void)viewDidAppear:(BOOL)animated {
    [super viewDidAppear:animated];
    
    if(self.initialLaunch) {
        if(!self.isFromMainDrawer) {
            CGRect rect = self.backButton.bounds;
            UIBezierPath *maskPath = [UIBezierPath bezierPathWithRoundedRect:rect byRoundingCorners:UIRectCornerTopRight cornerRadii:CGSizeMake(10, 10)];
            CAShapeLayer *maskLayer = [CAShapeLayer layer];
            maskLayer.fillColor = [Utils colorFromHexString:@"#46ADC6"].CGColor;
            maskLayer.frame = rect;
            maskLayer.path = maskPath.CGPath;
            [self.backButton.layer insertSublayer:maskLayer atIndex:0];
            self.backButton.hidden = NO;
        }
        
        [URLRequests getItem:self.placeId.intValue receive:^(NSData *data) {
            NSError *error;
            NSArray *dictArray = [NSJSONSerialization JSONObjectWithData:data options:0 error:&error];
            if(dictArray.count > 0) {
                self.placeDictionary = [[NSJSONSerialization JSONObjectWithData:data options:0 error:&error] objectAtIndex:0];
                
                if(!self.isFromMainDrawer) {
                    [((MainMenuViewController *)[((ContentHolderViewController *)[self.navigationController.viewControllers objectAtIndex:0]).mainNavigationController.viewControllers objectAtIndex:0]) changeHeader:[self.placeDictionary objectForKey:@"name"] withType:@"title"];
                } else {
                    if([self.navigationController.viewControllers.firstObject isKindOfClass:[CatalogueViewController class]]) {
                        ((CatalogueViewController *)[self.navigationController.viewControllers objectAtIndex:0]).catalogueControlViewController.headerLabel.text = [self.placeDictionary objectForKey:@"name"];
                    } else if([self.navigationController.viewControllers.firstObject isKindOfClass:[NotificationsViewController class]]) {
                        NotificationsHolderViewController *notificationsHolderViewController = (NotificationsHolderViewController *)((NotificationsViewController *)[self.navigationController.viewControllers objectAtIndex:0]).notificationsHolderViewController;
                        notificationsHolderViewController.headerLabel.text = [self.placeDictionary objectForKey:@"name"];
                        notificationsHolderViewController.headerLabel.textColor = [UIColor whiteColor];
                        notificationsHolderViewController.headerImageView.hidden = YES;
                        notificationsHolderViewController.headerNotificationCountView.hidden = YES;
                        notificationsHolderViewController.headerLabelCenterConstraint.constant = 0;
                        
                    }
                }
                
                int categoryId = [[[[[self.placeDictionary objectForKey:@"categories"] objectAtIndex:0] objectForKey:@"pivot"] objectForKey:@"category_id"] intValue];
                for(ItemCategory *itemCategory in [Utils getCategories]) {
                    if(itemCategory.itemId.intValue == categoryId) {
                        self.category = itemCategory;
                        break;
                    } else {
                        BOOL found = NO;
                        for(ItemCategory *itemSubcategory in itemCategory.itemSubcategories) {
                            if(itemSubcategory.itemId.intValue == categoryId) {
                                self.category = itemSubcategory;
                                found = YES;
                                break;
                            }
                        }
                        
                        if(found) {
                            break;
                        }
                    }
                }
                
                self.promotionsArray = (NSArray<NSDictionary *> *)[self.placeDictionary objectForKey:@"promotions"];
                int promotionTableViewHeight = 0;
                for(NSDictionary *promotion in self.promotionsArray ) {
                    CGFloat labelHeight = [[promotion objectForKey:@"title"] boundingRectWithSize:CGSizeMake([UIScreen mainScreen].bounds.size.width - 50, CGFLOAT_MAX) options:NSStringDrawingUsesLineFragmentOrigin attributes: @{NSFontAttributeName : [UIFont fontWithName:@"PTSans-Regular" size:16]} context:nil].size.height;
                    promotionTableViewHeight += labelHeight + 77;
                }
                self.promotionsTableViewHeightConstraint.constant = promotionTableViewHeight;
                
                [self loadContent];
            } else {
                [self.navigationController popViewControllerAnimated:YES];
            }
        }];
        
        self.initialLaunch = NO;
    }
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
}

-(void)viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:animated];
    if(self.oldTitle != nil && ![self.oldTitle isEqualToString:@""]) {
        if([self.navigationController.viewControllers.firstObject isKindOfClass:[NotificationsViewController class]]) {
            NotificationsHolderViewController *notificationsHolderViewController = (NotificationsHolderViewController *)((NotificationsViewController *)[self.navigationController.viewControllers objectAtIndex:0]).notificationsHolderViewController;
            notificationsHolderViewController.headerLabel.text = self.oldTitle;
            notificationsHolderViewController.headerLabel.textColor = [UIColor whiteColor];
            notificationsHolderViewController.headerImageView.hidden = YES;
            notificationsHolderViewController.headerNotificationCountView.hidden = YES;
            notificationsHolderViewController.headerLabelCenterConstraint.constant = 0;
        }
    }
}

-(void)loadContent {
    self.imagesCount = 0;
    self.currentPage = 0;
    self.isMapOpened = NO;
    
//place gallery
    NSString *imageUrl = [[self.placeDictionary objectForKey:@"logo"] objectForKey:@"image"];
    NSString *folder = [self.placeDictionary objectForKey:@"name"];
    folder = [folder stringByReplacingOccurrencesOfString:@" " withString:@"_"];
    if(imageUrl != nil) {
        UIImage *image = [URLRequests loadImage:imageUrl withFolder:nil receive:^(UIImage *image) {
            if(image != nil) {
                [self setImage:image];
            }
            
            if(self.imagesCount > 0) {
                self.placeNameLabel.hidden = YES;
            }
        }];
        if(image != nil) {
            [self setImage:image];
        }
    }
    
    BOOL shouldSetDefaultImage = YES;
    NSObject *object = [self.placeDictionary objectForKey:@"images"];
    if([object isKindOfClass:[NSDictionary class]]) {
        NSDictionary *imagesDictionary = (NSDictionary *)object;
        for(NSString *key in imagesDictionary.allKeys) {
            shouldSetDefaultImage = NO;
            UIImage *image = [URLRequests loadImage:[[imagesDictionary objectForKey:key] objectForKey:@"image"] withFolder:folder receive:^(UIImage *image) {
                if(image != nil) {
                    [self setImage:image];
                }
                if(self.imagesCount > 0) {
                    self.placeNameLabel.hidden = YES;
                }
            }];
            if(image != nil) {
                [self setImage:image];
            }
        }
    } else if([object isKindOfClass:[NSArray class]]) {
        NSArray<NSDictionary *> *imagesArray = (NSArray<NSDictionary *> *)object;
        int imageIndex = 1;
        for(NSDictionary *dictionary in imagesArray) {
            shouldSetDefaultImage = NO;
            UIImage *image = [URLRequests loadImage:[dictionary objectForKey:@"image"] withFolder:folder receive:^(UIImage *image) {
                [self setImage:image];
            }];
            imageIndex++;
            if(image != nil) {
                [self setImage:image];
            }
        }
    }
    
    if(self.imagesCount == 0 && imageUrl == nil && shouldSetDefaultImage) {
        //self.placeNameLabel.text = [self.placeDictionary objectForKey:@"name"];
        self.placeNameLabel.hidden = YES;
        [self setImage:[UIImage imageNamed:@"no-logo"]];
        self.imagesScrollView.backgroundColor = [Utils colorFromHexString:@"#E6E7E8"];
    } else {
        self.placeNameLabel.hidden = YES;
    }
    self.imagesScrollView.hidden = NO;
    
//place promotion
    self.promotionLabel.attributedText = [self promotionString:self.placeDictionary];
    self.promotionViewWidthConstraint.constant = self.promotionLabel.attributedText.length * 11.5;
    
//place promotionView
    self.promotionView.hidden = NO;

//place shadow
    self.shadowImageView.hidden = NO;
   
//place category
    UIImage *image = [URLRequests loadImage:self.category.itemCategoryPinIcon withFolder:nil receive:^(UIImage *image) {
        self.categoryImageView.image = image;
    }];
    self.categoryImageView.image = image;
    
    NSFetchRequest *request = [NSFetchRequest fetchRequestWithEntityName:@"Filter"];
    request.predicate = [NSPredicate predicateWithFormat:@"filterId == %d", self.category.itemId.intValue];
    NSManagedObjectContext *managedObjectContext = ((AppDelegate *)[UIApplication sharedApplication].delegate).managedObjectContext;
    NSError *error;
    NSArray<Filter *> *filtersArray = (NSArray<Filter *> *)[managedObjectContext executeFetchRequest:request error:&error];
    if(filtersArray.count == 1) {
        self.filter = [filtersArray objectAtIndex:0];
        if([[Utils currentLanguage] isEqualToString:@"bg"]) {
            self.categoryLabel.text = self.filter.filterBgName;
        } else {
            self.categoryLabel.text = self.filter.filterEnName;
        }
        self.categoryLabel.textColor = [Utils colorFromHexString:self.category.itemCategoryPinColor];
    }
        
//place address
    NSString *addressString = [[NSString alloc] init];
    NSDictionary *addressDictionary = [self.placeDictionary objectForKey:@"address"];
    NSString *city = [addressDictionary objectForKey:@"city"];
    if(city != nil) {
        addressString = [addressString stringByAppendingString:[addressDictionary objectForKey:@"city"]];
    }
    NSString *address = [addressDictionary objectForKey:@"address1"];
    if(address != nil) {
        if(city != nil) {
            addressString = [addressString stringByAppendingString:@", "];
        }
        addressString = [addressString stringByAppendingString:[addressDictionary objectForKey:@"address1"]];
    }
    if(addressString != nil && ![addressString isEqualToString:@""]) {
        self.addressLabel.text = addressString;
        self.addressImageView.hidden = NO;
    }

//place distance
    CLLocation *pinLocation = [[CLLocation alloc] initWithLatitude:[[self.placeDictionary objectForKey:@"lat"] floatValue] longitude:[[self.placeDictionary objectForKey:@"lng"] floatValue]];
    CLLocationDistance distance = [[Utils getLocation] distanceFromLocation:pinLocation];
    if(distance < 1000) {
        self.distanceLabel.text = [NSString stringWithFormat:@"%d %@", (int)distance, [Utils translate:@"Meter"]];
    } else {
        self.distanceLabel.text = [NSString stringWithFormat:@"%.2f %@", distance / 1000, [Utils translate:@"Kilometer"]];
    }
    CGFloat labelWidth = [self.distanceLabel.text boundingRectWithSize:CGSizeMake(CGFLOAT_MAX, 20) options:NSStringDrawingUsesLineFragmentOrigin attributes: @{NSFontAttributeName : self.distanceLabel.font} context:nil].size.width;
    self.distanceLabelWidthConstraint.constant = labelWidth;
    self.distanceLabel.text = self.distanceLabel.text;
    self.distanceImageView.hidden = NO;
    
//place website button
    labelWidth = [[UIScreen mainScreen] bounds].size.width;
    NSString *website = [NSString stringWithFormat:@"%@", [self.placeDictionary objectForKey:@"website"]];
    CGFloat labelHeight = [website boundingRectWithSize:CGSizeMake(labelWidth - 50, CGFLOAT_MAX) options:NSStringDrawingUsesLineFragmentOrigin attributes: @{NSFontAttributeName : self.websiteButton.titleLabel.font} context:nil].size.height;
    self.websiteButtonHeightConstraint.constant = labelHeight;
    if(website != nil && ![website isEqualToString:@""] && ![website isEqualToString:@"0"]) {
        [self.websiteButton setTitle:website forState:UIControlStateNormal];
        self.websiteButton.hidden = NO;
    } else {
        self.websiteButtonVerticalConstraint.constant = 0;
        self.websiteButtonHeightConstraint.constant = 0;
    }

//place facebook button
    NSString *facebook = [NSString stringWithFormat:@"%@", [self.placeDictionary objectForKey:@"facebook"]];
    labelHeight = [facebook boundingRectWithSize:CGSizeMake(labelWidth - 50, CGFLOAT_MAX) options:NSStringDrawingUsesLineFragmentOrigin attributes: @{NSFontAttributeName : self.facebookButton.titleLabel.font} context:nil].size.height;
    self.facebookButtonHeightConstraint.constant = labelHeight;
    if(facebook != nil && ![facebook isEqualToString:@""] && ![facebook isEqualToString:@"0"]) {
        [self.facebookButton setTitle:facebook forState:UIControlStateNormal];
        self.facebookButton.hidden = NO;
    } else {
        self.facebookButtonVerticalConstraint.constant = 0;
        self.facebookButtonHeightConstraint.constant = 0;
    }
    
//place phone button
    if([self.placeDictionary objectForKey:@"phone"] != nil) {
        NSString *phone = [NSString stringWithFormat:@"%@", [self.placeDictionary objectForKey:@"phone"]];
        if(![phone isEqualToString:@""] && ![phone isEqualToString:@"0"]) {
            [self.phoneButton setTitle:phone forState:UIControlStateNormal];
            self.phoneButton.hidden = NO;
        } else {
            self.phoneButtonVerticalConstraint.constant = 0;
            self.phoneButtonHeightConstraint.constant = 0;
        }
    } else {
        self.phoneButtonVerticalConstraint.constant = 0;
        self.phoneButtonHeightConstraint.constant = 0;
    }
    
//place mobile phone button
    if([self.placeDictionary objectForKey:@"mobile_phone"] != nil) {
        NSString *phone = [NSString stringWithFormat:@"%@", [self.placeDictionary objectForKey:@"mobile_phone"]];
        if(![phone isEqualToString:@""] && ![phone isEqualToString:@"0"]) {
            [self.mobilePhoneButton setTitle:phone forState:UIControlStateNormal];
            self.mobilePhoneButton.hidden = NO;
        } else {
            self.mobilePhoneButtonVerticalConstraint.constant = 0;
            self.mobilePhoneButtonHeightConstraint.constant = 0;
        }
    } else {
        self.mobilePhoneButtonVerticalConstraint.constant = 0;
        self.mobilePhoneButtonHeightConstraint.constant = 0;
    }
    
//place promotions
    self.promotionsTableViewRows = [[NSMutableArray alloc] initWithCapacity:self.promotionsArray.count];
    for(int i = 0; i < self.promotionsArray.count; i++) {
        [self.promotionsTableViewRows addObject:[NSNumber numberWithInt:0]];
    }
    [self.promotionsTableView reloadData];
    
//place description
    NSString *description = [NSString stringWithFormat:@"%@", [self.placeDictionary objectForKey:@"description"]];
    labelHeight = [description boundingRectWithSize:CGSizeMake(labelWidth - 50, CGFLOAT_MAX) options:NSStringDrawingUsesLineFragmentOrigin attributes: @{NSFontAttributeName : self.descriptionLabel.font} context:nil].size.height;
    if(description != nil && ![description isEqualToString:@""]) {
        self.descriptionLabelHeightConstraint.constant = labelHeight;
        self.descriptionLabel.text = description;
    } else {
        self.descriptionLabelHeightConstraint.constant = 0;
    }
    
    
//place add to favourites button
    if(((NSNumber *)[self.placeDictionary objectForKey:@"followed"]).intValue == 0) {
        self.followed = 0;
        [self.addToFavouritesButton setTitle:[Utils translate:@"Follow"] forState:UIControlStateNormal];
    } else {
        self.followed = 1;
        [self.addToFavouritesButton setTitle:[Utils translate:@"Unfollow"] forState:UIControlStateNormal];
    }
    
//place share button
    [self.shareButton setTitle:[Utils translate:@"Share"] forState:UIControlStateNormal];
    
//place map annotation view
    PinAnnotation *annotation = [[PinAnnotation alloc] init];
    self.placeCoordinate = CLLocationCoordinate2DMake([[self.placeDictionary objectForKey:@"lat"] doubleValue], [[self.placeDictionary objectForKey:@"lng"] doubleValue]);
    [annotation setCoordinate:self.placeCoordinate];
    [self.mapView addAnnotation:annotation];
    MKMapCamera *camera = [MKMapCamera cameraLookingAtCenterCoordinate:self.placeCoordinate fromEyeCoordinate:self.placeCoordinate eyeAltitude:15000];
    double latitude = camera.centerCoordinate.latitude;
    double longitude = camera.centerCoordinate.longitude;
    if(latitude > -90 && latitude < 90 && longitude > - 180 && longitude < 180) {
        [self.mapView setCamera:camera animated:NO];
    }

    
//place animation
    self.categoryImageViewVerticalConstraint.constant = 55;
    self.categoryLabelVerticalConstraint.constant = 55;
    [self.view layoutIfNeeded];
    [UIView animateWithDuration:0.5 delay:0 options:UIViewAnimationOptionCurveLinear animations:^{
        self.categoryImageViewVerticalConstraint.constant = 45;
        self.categoryLabelVerticalConstraint.constant = 45;
        self.categoryImageView.alpha = 1;
        self.categoryLabel.alpha = 1;
        self.addressImageView.alpha = 1;
        self.addressLabel.alpha = 1;
        self.distanceImageView.alpha = 1;
        self.distanceLabel.alpha = 1;
        self.websiteButton.alpha = 1;
        self.facebookButton.alpha = 1;
        self.phoneButton.alpha = 1;
        self.mobilePhoneButton.alpha = 1;
        self.descriptionLabel.alpha = 1;
        self.addToFavouritesButton.alpha = 1;
        self.shareButton.alpha = 1;
        self.mapView.alpha = 1;
        self.mapControlButton.alpha = 1;
        [self.view layoutIfNeeded];
    } completion:nil];
}

-(CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
    if(tableView == self.promotionsTableView) {
        if([self.promotionsTableViewRows objectAtIndex:indexPath.row].intValue == 0) {
            CGFloat labelHeight = [[[self.promotionsArray objectAtIndex:indexPath.row] objectForKey:@"title"] boundingRectWithSize:CGSizeMake([UIScreen mainScreen].bounds.size.width - 50, CGFLOAT_MAX) options:NSStringDrawingUsesLineFragmentOrigin attributes: @{NSFontAttributeName : [UIFont fontWithName:@"PTSans-Regular" size:16]} context:nil].size.height;
            labelHeight = labelHeight + 77;
            return labelHeight;
        } else {
            int height = 0;
            CGFloat labelHeight = [[[self.promotionsArray objectAtIndex:indexPath.row] objectForKey:@"title"] boundingRectWithSize:CGSizeMake([UIScreen mainScreen].bounds.size.width - 50, CGFLOAT_MAX) options:NSStringDrawingUsesLineFragmentOrigin attributes: @{NSFontAttributeName : [UIFont fontWithName:@"PTSans-Regular" size:16]} context:nil].size.height;
            height += labelHeight + 87;
            if([[self.promotionsArray objectAtIndex:indexPath.row] objectForKey:@"content"]) {
                CGFloat labelHeight = [[[self.promotionsArray objectAtIndex:indexPath.row] objectForKey:@"content"] boundingRectWithSize:CGSizeMake([UIScreen mainScreen].bounds.size.width - 50, CGFLOAT_MAX) options:NSStringDrawingUsesLineFragmentOrigin attributes: @{NSFontAttributeName : [UIFont fontWithName:@"PTSans-Regular" size:14]} context:nil].size.height;
                height += labelHeight;
            }
            if(((NSArray *)[[self.promotionsArray objectAtIndex:indexPath.row] objectForKey:@"images"]).count > 0) {
                UIImage *image = [URLRequests loadImage:[[[self.promotionsArray objectAtIndex:indexPath.row] objectForKey:@"images"] objectAtIndex:0] withFolder:nil receive:nil];
                CGFloat width = [UIScreen mainScreen].bounds.size.width - 30;
                CGFloat imageWidth = image.size.width;
                CGFloat scale = width / imageWidth;
                height += image.size.height * scale;
            }
            return height;
        }
    }
    return 0;
}


- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return self.promotionsArray.count;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    PromotionTableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"Cell"];
    cell.placeViewController = self;
    cell.index = indexPath.row;
    cell.promotionNameLabel.text = [[self.promotionsArray objectAtIndex:indexPath.row] objectForKey:@"title"];
    cell.promoLabel.text = [Utils translate:@"Promo"];
    NSDateFormatter *dateFormatter = [[NSDateFormatter alloc] init];
    dateFormatter.dateFormat = @"yyyy-MM-dd HH:mm:ss";
    NSDate *startingDate = [dateFormatter dateFromString:[[self.promotionsArray objectAtIndex:indexPath.row] objectForKey:@"starts"]];
    NSDate *endingDate = [dateFormatter dateFromString:[[self.promotionsArray objectAtIndex:indexPath.row] objectForKey:@"ends"]];
    dateFormatter.dateFormat = @"dd.MM.yyyy";
    NSString *dateString = [dateFormatter stringFromDate:startingDate];
    NSString *promotionString = [NSString stringWithFormat:@"%@ %@", [Utils translate:@"From"], dateString];
    dateString = [dateFormatter stringFromDate:endingDate];
    promotionString = [promotionString stringByAppendingFormat:@" %@ %@", [Utils translate:@"To"], dateString];
    if(![promotionString containsString:@"null"]) {
        cell.promotionDuration.hidden = NO;
    } else {
        cell.promotionDuration.hidden = YES;
    }
    cell.promotionDuration.text = promotionString;
    
    if(![[[self.promotionsArray objectAtIndex:indexPath.row] objectForKey:@"content"] isEqualToString:@""]) {
        cell.promotionDescriptionLabel.text = [[self.promotionsArray objectAtIndex:indexPath.row] objectForKey:@"content"];
    }
    
    if(((NSArray *)[[self.promotionsArray objectAtIndex:indexPath.row] objectForKey:@"images"]).count == 0) {
        cell.promotionImageViewHeightConstraint.constant = 0;
        cell.promotionImageViewVerticalConstraint.constant = 0;
    } else {
        UIImage *image = [URLRequests loadImage:[[[self.promotionsArray objectAtIndex:indexPath.row] objectForKey:@"images"] objectAtIndex:0] withFolder:nil receive:^(UIImage *image) {
            if(image != nil) {
                CGFloat width = [UIScreen mainScreen].bounds.size.width - 30;
                CGFloat imageWidth = image.size.width;
                CGFloat scale = width / imageWidth;
                cell.promotionImageViewHeightConstraint.constant = image.size.height * scale;
                cell.promotionImageView.image = image;
            }
        }];
        if(image != nil) {
            CGFloat width = [UIScreen mainScreen].bounds.size.width - 30;
            CGFloat imageWidth = image.size.width;
            CGFloat scale = width / imageWidth;
            cell.promotionImageViewHeightConstraint.constant = image.size.height * scale;
            cell.promotionImageView.image = image;
        }
    }
    return cell;
}

-(int)changeRowHeight:(long)index withCurrentHeight:(int)currentHeight {
    int height = 0;
    if([self.promotionsTableViewRows objectAtIndex:index].intValue == 0) {
        [self.promotionsTableViewRows setObject:[NSNumber numberWithInt:1] atIndexedSubscript:index];
        if([[self.promotionsArray objectAtIndex:index] objectForKey:@"content"]) {
            CGFloat labelHeight = [[[self.promotionsArray objectAtIndex:index] objectForKey:@"content"] boundingRectWithSize:CGSizeMake([UIScreen mainScreen].bounds.size.width - 50, CGFLOAT_MAX) options:NSStringDrawingUsesLineFragmentOrigin attributes: @{NSFontAttributeName : [UIFont fontWithName:@"PTSans-Regular" size:14]} context:nil].size.height;
            height += labelHeight + 10;
        }
        if(((NSArray *)[[self.promotionsArray objectAtIndex:index] objectForKey:@"images"]).count > 0) {
            UIImage *image = [URLRequests loadImage:[[[self.promotionsArray objectAtIndex:index] objectForKey:@"images"] objectAtIndex:0] withFolder:nil receive:nil];
            CGFloat width = [UIScreen mainScreen].bounds.size.width - 30;
            CGFloat imageWidth = image.size.width;
            CGFloat scale = width / imageWidth;
            height += image.size.height * scale;
        }
        height += currentHeight;
    } else {
        [self.promotionsTableViewRows setObject:[NSNumber numberWithInt:0] atIndexedSubscript:index];
        CGFloat labelHeight = [[[self.promotionsArray objectAtIndex:index] objectForKey:@"title"] boundingRectWithSize:CGSizeMake([UIScreen mainScreen].bounds.size.width - 50, CGFLOAT_MAX) options:NSStringDrawingUsesLineFragmentOrigin attributes: @{NSFontAttributeName : [UIFont fontWithName:@"PTSans-Regular" size:16]} context:nil].size.height;
        height = labelHeight + 77;
    }
    
    return height;
}

-(MKAnnotationView *) mapView:(MKMapView *)mapView viewForAnnotation:(id <MKAnnotation>) annotation {
    if(annotation == self.mapView.userLocation) {
        return nil;
    }
    
    static NSString *const pinIdentifier = @"mapPin";
    InfoAnnotationView *annotationView = nil;
    annotationView = (InfoAnnotationView *)[mapView dequeueReusableAnnotationViewWithIdentifier:pinIdentifier];
    if(annotationView == nil) {
        annotationView = [[InfoAnnotationView alloc] initWithAnnotation:annotation reuseIdentifier:pinIdentifier];
    }
    annotationView.annotation = annotation;
    
    if(self.category.itemId.intValue != -1) {
        UIImage *image = [URLRequests loadImage:self.category.itemMapIcon withFolder:nil receive:^(UIImage *image) {
        }];
        if(image != nil) {
            image = [Utils imageWithImage:image scaledToSize:CGSizeMake(40, 80)];
            annotationView.image = image;
        }
    }
    
    return annotationView;
}

- (IBAction)openWebView:(id)sender {
    WebViewController *webViewController = [[UIStoryboard storyboardWithName:@"Main" bundle:nil] instantiateViewControllerWithIdentifier:@"WebViewController"];
    NSURL *url = [NSURL URLWithString:((UIButton *)sender).titleLabel.text];
    NSURLRequest *request = [NSURLRequest requestWithURL:url];
    webViewController.request = request;
    webViewController.isFromMainDrawer = self.isFromMainDrawer;
    [self.navigationController pushViewController:webViewController animated:YES];
}

- (IBAction)openDialer:(id)sender {
    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:[NSString stringWithFormat:@"Do you want to dial this number: %@ ?", ((UIButton *)sender).titleLabel.text] message:@"" preferredStyle:UIAlertControllerStyleAlert];
    [alertController addAction:[UIAlertAction actionWithTitle:@"No" style:UIAlertActionStyleDefault handler:^(UIAlertAction *action) {
        [self dismissViewControllerAnimated:YES completion:^{
        }];
    }]];
    
    [alertController addAction:[UIAlertAction actionWithTitle:@"Yes" style:UIAlertActionStyleCancel handler:^(UIAlertAction *action) {
        NSString *number = @"tel:";
        number = [number stringByAppendingString:((UIButton *)sender).titleLabel.text];
        number = [number stringByReplacingOccurrencesOfString:@" " withString:@""];
        NSURL *url = [NSURL URLWithString:number];
        if([[UIApplication sharedApplication] canOpenURL:url]) {
            [[UIApplication sharedApplication] openURL:url];
        }
    }]];
    
    [self presentViewController:alertController animated:YES completion:nil];
}

- (IBAction)addToFavourites:(id)sender {
    [URLRequests followPlace:self.followed withLocationId:self.placeId.intValue receive:^(BOOL success) {
        if(success) {
            if(self.followed == 1) {
                [self.addToFavouritesButton setTitle:[Utils translate:@"Follow"] forState:UIControlStateNormal];
                self.followed = 0;
                UIViewController *viewController = self.navigationController.viewControllers.firstObject;
                if([viewController isKindOfClass:[ContentHolderViewController class]]) {
                    BrandsViewController *brandsViewController = ((ContentHolderViewController *)viewController).brandsViewController;
                    for(Brand *brand in brandsViewController.brandsArray) {
                        if(brand.brandId.intValue == self.placeId.intValue) {
                            brand.isFollowed = NO;
                            break;
                        }
                    }
                    [brandsViewController.brandsCollectionView reloadData];
                } else if([viewController isKindOfClass:[CatalogueViewController class]]) {
                    if([[self.navigationController.viewControllers objectAtIndex:self.navigationController.viewControllers.count - 2] isKindOfClass:[CatalogueViewController class]]) {
                        CatalogueViewController *catalogueViewController = (CatalogueViewController *)[self.navigationController.viewControllers objectAtIndex:self.navigationController.viewControllers.count - 2];
                        if(catalogueViewController.newsArray != nil) {
                            for(NewPlace *newPlace in catalogueViewController.newsArray) {
                                if(newPlace.placeId.intValue == self.placeId.intValue && catalogueViewController.catalogueType != FAVOURITES) {
                                    newPlace.placeIsFollowed = NO;
                                } else if(newPlace.placeId.intValue == self.placeId.intValue && catalogueViewController.catalogueType == FAVOURITES) {
                                    NSMutableArray *mutableArray = [catalogueViewController.newsArray mutableCopy];
                                    [mutableArray removeObjectAtIndex:[mutableArray indexOfObject:newPlace]];
                                    catalogueViewController.newsArray = [mutableArray copy];
                                    
                                    BrandsViewController *brandsViewController = ((MainMenuViewController *)[catalogueViewController.catalogueControlViewController.navigationController.viewControllers objectAtIndex:0]).brandsViewController;
                                    for(Brand *brand in brandsViewController.brandsArray) {
                                        if(brand.brandId.intValue == self.placeId.intValue) {
                                            brand.isFollowed = NO;
                                            break;
                                        }
                                    }
                                    [brandsViewController.brandsCollectionView reloadData];
                                }
                            }
                        } else if(catalogueViewController.shoppingCentersArray != nil) {
                            for(ShoppingCenter *shoppingCenter in catalogueViewController.shoppingCentersArray) {
                                if(shoppingCenter.shoppingCenterId.intValue == self.placeId.intValue) {
                                    shoppingCenter.shoppingCenterFollowed = [NSNumber numberWithInt:self.followed];
                                }
                            }
                        } else if(catalogueViewController.promotionsArray != nil) {
                            for(Promotion *promotion in catalogueViewController.promotionsArray) {
                                if(promotion.promotionId.intValue == self.placeId.intValue) {
                                    promotion.promotionFollowed = [NSNumber numberWithInt:self.followed];
                                }
                            }
                        }
                        [catalogueViewController.catalogueCollectionView reloadData];
                    } else if([[self.navigationController.viewControllers objectAtIndex:self.navigationController.viewControllers.count - 2] isKindOfClass:[ShoppingCenterViewController class]]) {
                        ShoppingCenterViewController *shoppingCenterViewController = (ShoppingCenterViewController *)[self.navigationController.viewControllers objectAtIndex:self.navigationController.viewControllers.count - 2];
                        NSMutableArray<NSDictionary *> *tempArray = [[NSMutableArray alloc] initWithCapacity:shoppingCenterViewController.shoppingCenterPlacesArray.count];
                        for(NSDictionary *shoppingCenter in shoppingCenterViewController.shoppingCenterPlacesArray) {
                            NSMutableDictionary *tempDict = [shoppingCenter mutableCopy];
                            if([[shoppingCenter objectForKey:@"id"] intValue] == self.placeId.intValue) {
                                [tempDict setObject:[NSNumber numberWithInt:self.followed] forKey:@"followed"];
                            }
                            [tempArray addObject:tempDict];
                        }
                        shoppingCenterViewController.shoppingCenterPlacesArray = tempArray;
                        [shoppingCenterViewController.shoppingCenterPlacesCollectionView reloadData];
                    }
                }
            } else {
                [self.addToFavouritesButton setTitle:[Utils translate:@"Unfollow"] forState:UIControlStateNormal];
                self.followed = 1;
                UIViewController *viewController = self.navigationController.viewControllers.firstObject;
                if([viewController isKindOfClass:[ContentHolderViewController class]]) {
                    BrandsViewController *brandsViewController = ((ContentHolderViewController *)viewController).brandsViewController;
                    for(Brand *brand in brandsViewController.brandsArray) {
                        if(brand.brandId.intValue == self.placeId.intValue) {
                            brand.isFollowed = YES;
                            break;
                        }
                    }
                    [brandsViewController.brandsCollectionView reloadData];
                } else if([viewController isKindOfClass:[CatalogueViewController class]]) {
                    if([[self.navigationController.viewControllers objectAtIndex:self.navigationController.viewControllers.count - 2] isKindOfClass:[CatalogueViewController class]]) {
                        CatalogueViewController *catalogueViewController = (CatalogueViewController *)[self.navigationController.viewControllers objectAtIndex:self.navigationController.viewControllers.count - 2];
                        if(catalogueViewController.newsArray != nil) {
                            for(NewPlace *newPlace in catalogueViewController.newsArray) {
                                if(newPlace.placeId.intValue == self.placeId.intValue) {
                                    newPlace.placeIsFollowed = NO;
                                }
                            }
                        } else if(catalogueViewController.shoppingCentersArray != nil) {
                            for(ShoppingCenter *shoppingCenter in catalogueViewController.shoppingCentersArray) {
                                if(shoppingCenter.shoppingCenterId.intValue == self.placeId.intValue) {
                                    shoppingCenter.shoppingCenterFollowed = [NSNumber numberWithInt:self.followed];
                                }
                            }
                        } else if(catalogueViewController.promotionsArray != nil) {
                            for(Promotion *promotion in catalogueViewController.promotionsArray) {
                                if(promotion.promotionId.intValue == self.placeId.intValue) {
                                    promotion.promotionFollowed = [NSNumber numberWithInt:self.followed];
                                }
                            }
                        }
                        [catalogueViewController.catalogueCollectionView reloadData];
                    } else if([[self.navigationController.viewControllers objectAtIndex:self.navigationController.viewControllers.count - 2] isKindOfClass:[ShoppingCenterViewController class]]) {
                        ShoppingCenterViewController *shoppingCenterViewController = (ShoppingCenterViewController *)[self.navigationController.viewControllers objectAtIndex:self.navigationController.viewControllers.count - 2];
                        NSMutableArray<NSDictionary *> *tempArray = [[NSMutableArray alloc] initWithCapacity:shoppingCenterViewController.shoppingCenterPlacesArray.count];
                        for(NSDictionary *shoppingCenter in shoppingCenterViewController.shoppingCenterPlacesArray) {
                            NSMutableDictionary *tempDict = [shoppingCenter mutableCopy];
                            if([[shoppingCenter objectForKey:@"id"] intValue] == self.placeId.intValue) {
                                [tempDict setObject:[NSNumber numberWithInt:self.followed] forKey:@"followed"];
                            }
                            [tempArray addObject:tempDict];
                        }
                        shoppingCenterViewController.shoppingCenterPlacesArray = tempArray;
                        [shoppingCenterViewController.shoppingCenterPlacesCollectionView reloadData];

                    }
                }
            }
        }
    }];
}

- (IBAction)onSharePressed:(id)sender {
    if([FBSDKAccessToken currentAccessToken] != nil) {
        FBSDKShareLinkContent *content = [[FBSDKShareLinkContent alloc] init];
        content.contentURL = [NSURL URLWithString:@"http://www.unicreditbulbank.bg/bg/Individual_Clients/Plus_programme/About/index.htm"];
        content.contentTitle = [self.placeDictionary objectForKey:@"name"];
        content.contentDescription = [NSString stringWithFormat:[Utils translate:@"Share description"], [self.placeDictionary objectForKey:@"name"]];
        FBSDKShareDialog *shareDialog = [[FBSDKShareDialog alloc] init];
        shareDialog.fromViewController = self;
        shareDialog.shareContent = content;
        [shareDialog show];
    } else {
        NSArray *itemsToShare = [NSArray arrayWithObjects:[self.placeDictionary objectForKey:@"name"], [NSString stringWithFormat:[Utils translate:@"Share description"], [self.placeDictionary objectForKey:@"name"]], [NSURL URLWithString:@"http://www.unicreditbulbank.bg/bg/Individual_Clients/Plus_programme/About/index.htm"], nil];
        
        UIActivityViewController *activityVC = [[UIActivityViewController alloc] initWithActivityItems:itemsToShare applicationActivities:nil];
        activityVC.excludedActivityTypes = @[UIActivityTypePrint, UIActivityTypeCopyToPasteboard, UIActivityTypeAssignToContact, UIActivityTypeSaveToCameraRoll, UIActivityTypePostToWeibo];
        
        [self presentViewController:activityVC animated:YES completion:nil];
    }
}

-(void)setImage:(UIImage *)image {
    CGRect frame;
    frame.origin.x = self.imagesScrollView.frame.size.width * self.imagesCount;
    frame.origin.y = 0;
    frame.size = self.imagesScrollView.frame.size;
    
    UIImageView *imageView = [[UIImageView alloc] initWithFrame:frame];
    imageView.image = image;
    imageView.contentMode = UIViewContentModeScaleAspectFit;
    if([self.placeDictionary objectForKey:@"color"]) {
        imageView.backgroundColor = [Utils colorFromHexString:[NSString stringWithFormat:@"#%@", [self.placeDictionary objectForKey:@"color"]]];
    }

    UITapGestureRecognizer *imageTap = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(openGallery)];
    imageTap.numberOfTapsRequired = 1;
    imageTap.numberOfTouchesRequired = 1;
    
    imageView.userInteractionEnabled = YES;
    [imageView addGestureRecognizer:imageTap];
    
    [self.imagesScrollView addSubview:imageView];

    self.imagesCount++;
    self.imagesScrollView.contentSize = CGSizeMake(self.imagesScrollView.frame.size.width * self.imagesCount, self.imagesScrollView.frame.size.height);
    
    if(self.imagesCount <= 1) {
        self.leftArrowButton.hidden = YES;
        self.leftArrowButton.userInteractionEnabled = NO;
        self.rightArrowButton.hidden = YES;
        self.rightArrowButton.userInteractionEnabled = NO;
    } else {
        self.leftArrowButton.hidden = NO;
        self.leftArrowButton.userInteractionEnabled = YES;
        self.rightArrowButton.hidden = NO;
        self.rightArrowButton.userInteractionEnabled = YES;
    }

}

-(void)openGallery {
    [self performSegueWithIdentifier:@"GallerySegue" sender:self];
}

- (void)scrollViewDidScroll:(UIScrollView *)scrollView {
    if(scrollView == self.imagesScrollView) {
        self.currentPage = (scrollView.contentOffset.x + scrollView.frame.size.width / 2) / scrollView.frame.size.width;
    } else {
        self.placeScrollViewOffset = scrollView.contentOffset.y;
    }
}

- (IBAction)onLeftArrowPressed:(id)sender {
    if(self.currentPage > 0) {
        self.currentPage--;
        [UIView animateWithDuration:0.3 animations:^{
            [self.imagesScrollView setContentOffset:CGPointMake(self.currentPage * self.imagesScrollView.frame.size.width, 0)];
        }];
    }
}

- (IBAction)onRightArrowPressed:(id)sender {
    if(self.currentPage < self.imagesCount - 1) {
        self.currentPage++;
        [UIView animateWithDuration:0.3 animations:^{
            [self.imagesScrollView setContentOffset:CGPointMake(self.currentPage * self.imagesScrollView.frame.size.width, 0)];
        }];
    }
}

- (IBAction)mapControl:(id)sender {
    CGFloat screenHeight = [[UIScreen mainScreen] bounds].size.height;
    if(!self.isMapOpened) {
        self.placeScrollView.scrollEnabled = NO;
        [self.view layoutIfNeeded];
        [UIView animateWithDuration:0.3 delay:0.0 options:UIViewAnimationOptionCurveLinear animations:^{
            self.placeScrollView.contentOffset = CGPointMake(self.placeScrollView.contentOffset.x, self.placeScrollView.contentSize.height);
            self.mapViewHeightConstraint.constant = screenHeight - 50;
            self.mapViewLeadingConstraint.constant = 0;
            self.mapViewTrailingConstraint.constant = 0;
            self.mapViewBottomConstraint.constant = 0;
            CATransition *fade = [CATransition animation];
            [self.mapControlButton.layer addAnimation:fade forKey:@"fade"];
            [self.mapControlButton setImage:[UIImage imageNamed:@"shrink_map"] forState:UIControlStateNormal];
            self.backButton.alpha = 0;
            [self.view layoutIfNeeded];
        } completion:^(BOOL finished) {
            self.mapView.scrollEnabled = YES;
            self.mapView.zoomEnabled = YES;
            self.backButton.userInteractionEnabled = NO;
            self.mapView.showsUserLocation = YES;
        }];
        self.isMapOpened = YES;
    } else {
        MKMapCamera *camera = [MKMapCamera cameraLookingAtCenterCoordinate:self.placeCoordinate fromEyeCoordinate:self.placeCoordinate eyeAltitude:self.mapView.camera.altitude];
        double latitude = camera.centerCoordinate.latitude;
        double longitude = camera.centerCoordinate.longitude;
        if(latitude > -90 && latitude < 90 && longitude > - 180 && longitude < 180) {
            [self.mapView setCamera:camera animated:YES];
        }
        [self.view layoutIfNeeded];
        [UIView animateWithDuration:0.3 delay:0.0 options:UIViewAnimationOptionCurveLinear animations:^{
            self.placeScrollView.contentOffset = CGPointMake(self.placeScrollView.contentOffset.x, self.placeScrollViewOffset);
            self.mapViewHeightConstraint.constant = 135;
            self.mapViewLeadingConstraint.constant = 15;
            self.mapViewTrailingConstraint.constant = 15;
            self.mapViewBottomConstraint.constant = 25;
            CATransition *fade = [CATransition animation];
            [self.mapControlButton.layer addAnimation:fade forKey:@"fade"];
            [self.mapControlButton setImage:[UIImage imageNamed:@"expand_map"] forState:UIControlStateNormal];
            self.backButton.alpha = 1;
            [self.view layoutIfNeeded];
        } completion:^(BOOL finished) {
            self.placeScrollView.scrollEnabled = YES;
            self.mapView.scrollEnabled = NO;
            self.mapView.zoomEnabled = NO;
            self.backButton.userInteractionEnabled = YES;
            self.mapView.showsUserLocation = NO;
        }];
        self.isMapOpened = NO;
    }
}

-(void)changeLanguage:(NSNotification *)notification {
    NSString *language = [notification.userInfo objectForKey:@"language"];
    [URLRequests getItem:self.placeId.intValue receive:^(NSData *data) {
        NSError *error;
        NSDictionary *dictionary = [[NSJSONSerialization JSONObjectWithData:data options:0 error:&error] objectAtIndex:0];
        
        if([language isEqualToString:@"bg"]) {
            self.categoryLabel.text = self.filter.filterBgName;
        } else {
            self.categoryLabel.text = self.filter.filterEnName;
        }
        self.categoryLabel.textColor = [Utils colorFromHexString:self.category.itemCategoryPinColor];
        
        NSString *addressString = [[NSString alloc] init];
        NSDictionary *addressDictionary = [dictionary objectForKey:@"address"];
        NSString *city = [addressDictionary objectForKey:@"city"];
        if(city != nil) {
            addressString = [addressString stringByAppendingString:[addressDictionary objectForKey:@"city"]];
        }
        NSString *address = [addressDictionary objectForKey:@"address1"];
        if(address != nil) {
            if(city != nil) {
                addressString = [addressString stringByAppendingString:@", "];
            }
            addressString = [addressString stringByAppendingString:[addressDictionary objectForKey:@"address1"]];
        }
        self.addressLabel.text = addressString;
        
        NSArray<NSString *> *distanceArray = [self.distanceLabel.text componentsSeparatedByString:@" "];
        float distance = [[distanceArray objectAtIndex:0] floatValue];
        if(distance >= 1000) {
            self.distanceLabel.text = [NSString stringWithFormat:@"%.2f %@", distance, [Utils translate:@"Kilometer"]];
        } else {
            self.distanceLabel.text = [NSString stringWithFormat:@"%d %@", (int)distance , [Utils translate:@"Meter"]];
        }
        CGFloat labelWidth = [self.distanceLabel.text boundingRectWithSize:CGSizeMake(CGFLOAT_MAX, 20) options:NSStringDrawingUsesLineFragmentOrigin attributes: @{NSFontAttributeName : self.distanceLabel.font} context:nil].size.width;
        self.distanceLabelWidthConstraint.constant = labelWidth;
        
        NSString *description = [dictionary objectForKey:@"description"];
        CGFloat labelHeight = [description boundingRectWithSize:CGSizeMake([UIScreen mainScreen].bounds.size.width - 50, CGFLOAT_MAX) options:NSStringDrawingUsesLineFragmentOrigin attributes: @{NSFontAttributeName : self.descriptionLabel.font} context:nil].size.height;
        self.descriptionLabelHeightConstraint.constant = labelHeight;
        self.descriptionLabel.text = description;
        
        if(((NSNumber *)[dictionary objectForKey:@"followed"]).intValue == 0) {
            self.followed = 0;
            [self.addToFavouritesButton setTitle:[Utils translate:@"Follow"] forState:UIControlStateNormal];
        } else {
            self.followed = 1;
            [self.addToFavouritesButton setTitle:[Utils translate:@"Unfollow"] forState:UIControlStateNormal];
        }
        
        [self.shareButton setTitle:[Utils translate:@"Share"] forState:UIControlStateNormal];
    }];
}

- (IBAction)onBackPressed:(id)sender {
    if([self.headerFieldType isEqualToString:@"title"]) {
        [((MainMenuViewController *)[((ContentHolderViewController *)[self.navigationController.viewControllers objectAtIndex:0]).mainNavigationController.viewControllers objectAtIndex:0]) changeHeader:self.headerTitle withType:self.headerFieldType];
    } else {
        [((MainMenuViewController *)[((ContentHolderViewController *)[self.navigationController.viewControllers objectAtIndex:0]).mainNavigationController.viewControllers objectAtIndex:0]) changeHeader:nil withType:self.headerFieldType];
    }
    [self.navigationController popViewControllerAnimated:YES];
}

-(NSMutableAttributedString *)promotionString:(NSDictionary *)dictionary {
    if([[dictionary objectForKey:@"unicredit_promotion"] isKindOfClass:[NSString class]] && ![(NSString *)[dictionary objectForKey:@"unicredit_promotion"] isEqualToString:@"0"]) {
        NSString *promotion = (NSString *)[self.placeDictionary objectForKey:@"unicredit_promotion"];
        NSString *forString = [Utils translate:@"For"];
        NSString *currency = [Utils translate:@"Currency"];
        
        NSMutableAttributedString *promotionString = [[NSMutableAttributedString alloc] initWithString:[NSString stringWithFormat:@"%@ %@ 1 %@", promotion, forString, currency]];
        [promotionString addAttribute:NSFontAttributeName value:[UIFont fontWithName:@"PTSans-Bold" size:20] range:NSMakeRange(0, promotion.length)];
        [promotionString addAttribute:NSFontAttributeName value:[UIFont fontWithName:@"PTSans-Regular" size:14] range:NSMakeRange(promotion.length + 1, forString.length)];
        [promotionString addAttribute:NSFontAttributeName value:[UIFont fontWithName:@"PTSans-Bold" size:20] range:NSMakeRange(promotion.length + forString.length + 2, 1)];
        [promotionString addAttribute:NSFontAttributeName value:[UIFont fontWithName:@"PTSans-Regular" size:14] range:NSMakeRange(promotionString.length - currency.length, 3)];
        
        return promotionString;
    } else {
        return [[NSMutableAttributedString alloc] initWithString:@""];
    }
}

#pragma mark - Navigation

- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    if([[segue destinationViewController] isKindOfClass:[GalleryViewController class]]) {
        NSMutableArray<UIImage *> *imagesArray = [[NSMutableArray alloc] initWithCapacity:0];
        for(UIView *subview in self.imagesScrollView.subviews) {
            if([subview isKindOfClass:[UIImageView class]]) {
                [imagesArray addObject:((UIImageView *)subview).image];
            }
        }
        GalleryViewController *galleryController = (GalleryViewController *)[segue destinationViewController];
        galleryController.imagesArray = imagesArray;
        galleryController.currentPage = self.currentPage;
        galleryController.onDismiss = ^(int currentPage) {
            [self.imagesScrollView setContentOffset:CGPointMake(currentPage * self.imagesScrollView.frame.size.width, 0)];
            self.currentPage = currentPage;
        };
    }
}


@end
