//
//  ProfileViewController.m
//  plus
//
//  Created by Виктор Георгиев on 9/6/16.
//  Copyright © 2016 Outsourcing IT Services Ltd. All rights reserved.
//

#import "ProfileViewController.h"
#import "LoginViewController.h"

#import "URLRequests.h"
#import "Utils.h"

@class LoginViewController;

@interface ProfileViewController ()

@property (weak, nonatomic) IBOutlet UIImageView *profileImageView;
@property (weak, nonatomic) IBOutlet UIImageView *profileNameImageView;
@property (weak, nonatomic) IBOutlet UIImageView *profileAddressImageView;
@property (weak, nonatomic) IBOutlet UIImageView *profileBirthdayImageView;
@property (weak, nonatomic) IBOutlet UIImageView *profileFavouritesImageView;

@property (weak, nonatomic) IBOutlet UILabel *profileNameLabel;
@property (weak, nonatomic) IBOutlet UILabel *profileAddressLabel;
@property (weak, nonatomic) IBOutlet UILabel *profileBirthdayLabel;
@property (weak, nonatomic) IBOutlet UILabel *profileFavouritesCountLabel;

@property (weak, nonatomic) IBOutlet NSLayoutConstraint *favouritesViewHeightConstraint;

@end

@implementation ProfileViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    
    self.profileImageView.layer.borderColor = [UIColor colorWithRed:70 / 255.0 green:173 / 255.0 blue:198 / 255.0 alpha:1].CGColor;
    self.profileImageView.layer.borderWidth = 5;
    
    int estimatedHeight = [UIScreen mainScreen].bounds.size.height - 537;
    if(estimatedHeight < 130) {
        self.favouritesViewHeightConstraint.constant = 130;
    } else {
        self.favouritesViewHeightConstraint.constant = estimatedHeight;
    }
    
    if([FBSDKAccessToken currentAccessToken] != nil) {
        [self loadProfile];
    }
}

-(void)loadProfile {
    FBSDKGraphRequest *graphRequest = [[FBSDKGraphRequest alloc] initWithGraphPath:@"me" parameters:@{@"fields" : @"id,name,first_name,last_name,birthday,email,location,picture.type(large)"} tokenString:[FBSDKAccessToken currentAccessToken].tokenString version:nil HTTPMethod:nil];
    [graphRequest startWithCompletionHandler:^(FBSDKGraphRequestConnection *connection, id result, NSError *error) {
        if(error == nil) {
            NSDictionary *facebookDictionary = (NSDictionary *)result;
            UIImage *image = [URLRequests loadImage:[[[facebookDictionary objectForKey:@"picture"] objectForKey:@"data"] objectForKey:@"url"] withFolder:@"Facebook" receive:^(UIImage *image) {
                self.profileImageView.image = image;
            }];
            self.profileImageView.image = image;
            self.profileNameLabel.text = [facebookDictionary objectForKey:@"name"];
            self.profileAddressLabel.text = [[facebookDictionary objectForKey:@"location"] objectForKey:@"name"];
            NSDateFormatter *dateFormatter = [[NSDateFormatter alloc] init];
            dateFormatter.dateFormat = @"MM/dd/yyyy";
            NSDate *birthDate = [dateFormatter dateFromString:[facebookDictionary objectForKey:@"birthday"]];
            dateFormatter.dateFormat = @"dd.MM.yyyy";
            self.profileBirthdayLabel.text = [dateFormatter stringFromDate:birthDate];

            [URLRequests setUser:[Utils currentLanguage] facebookId:[facebookDictionary objectForKey:@"id"] email:[facebookDictionary objectForKey:@"email"] firstname:[facebookDictionary objectForKey:@"first_name"] lastname:[facebookDictionary objectForKey:@"last_name"] receive:^(NSData *data) {
                NSError *error;
                NSDictionary *dictionary = [NSJSONSerialization JSONObjectWithData:data options:0 error:&error];
                int userId = [[dictionary objectForKey:@"id"] intValue];
                [[NSUserDefaults standardUserDefaults] setInteger:userId forKey:@"userId"];
                [[NSUserDefaults standardUserDefaults] synchronize];
                self.profileFavouritesCountLabel.text = [NSString stringWithFormat:@"%@", [dictionary objectForKey:@"count"]];
            }];
            
            [UIView animateWithDuration:1 animations:^{
                self.profileImageView.alpha = 1;
                self.profileNameLabel.alpha = 1;
                self.profileNameImageView.alpha = 1;
                self.profileAddressLabel.alpha = 1;
                self.profileAddressImageView.alpha = 1;
                self.profileBirthdayLabel.alpha = 1;
                self.profileBirthdayImageView.alpha = 1;
                self.profileFavouritesCountLabel.alpha = 1;
                self.profileFavouritesImageView.alpha = 1;
            }];
        }
    }];
}

-(void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    
    if([FBSDKAccessToken currentAccessToken] == nil) {
        self.view.alpha = 0;
        [self performSegueWithIdentifier:@"LoginSegue" sender:self];
    }
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
}

- (IBAction)onLogoutPressed:(id)sender {
    FBSDKLoginManager *login = [[FBSDKLoginManager alloc] init];
    [login logOut];
    [self performSegueWithIdentifier:@"LoginSegue" sender:self];
}

- (IBAction)onShowFavouritesPressed:(id)sender {
    [self.navigationController.viewControllers.firstObject performSegueWithIdentifier:@"CatalogueSegue" sender:self];
}

- (IBAction)onBackPressed:(id)sender {
    [self.navigationController popViewControllerAnimated:YES];
}


#pragma mark - Navigation

- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    if([[segue destinationViewController] isKindOfClass:[LoginViewController class]]) {
        LoginViewController *loginViewController = [segue destinationViewController];
        loginViewController.onDismiss = ^() {
            self.view.alpha = 1;
            [self loadProfile];
        };
    }
}


@end
