//
//  ShoppingCenterViewController.m
//  plus
//
//  Created by Виктор Георгиев on 9/9/16.
//  Copyright © 2016 Outsourcing IT Services Ltd. All rights reserved.
//

#import "ShoppingCenterViewController.h"
#import "GalleryViewController.h"
#import "PlaceViewController.h"
#import "CatalogueViewController.h"
#import "WebViewController.h"

#import "ShoppingCenterPlaceCollectionViewCell.h"

#import "URLRequests.h"
#import "Utils.h"
#import "PinAnnotation.h"
#import "InfoAnnotationView.h"
#import <FBSDKShareKit/FBSDKShareKit.h>

@interface ShoppingCenterViewController ()

@property (strong, nonatomic) NSDictionary *shoppingCenterDictionary;

@property (assign, nonatomic) CLLocationCoordinate2D shoppingCenterCoordinate;

@property (weak, nonatomic) IBOutlet MKMapView *mapView;



@property (weak, nonatomic) IBOutlet UIScrollView *shoppingCenterScrollView;
@property (weak, nonatomic) IBOutlet UIScrollView *imagesScrollView;

@property (weak, nonatomic) IBOutlet UIButton *leftArrowButton;
@property (weak, nonatomic) IBOutlet UIButton *rightArrowButton;
@property (weak, nonatomic) IBOutlet UIButton *websiteButton;
@property (weak, nonatomic) IBOutlet UIButton *facebookButton;
@property (weak, nonatomic) IBOutlet UIButton *phoneButton;
@property (weak, nonatomic) IBOutlet UIButton *mobilePhoneButton;
@property (weak, nonatomic) IBOutlet UIButton *addToFavouritesButton;
@property (weak, nonatomic) IBOutlet UIButton *shareButton;
@property (weak, nonatomic) IBOutlet UIButton *mapControlButton;

@property (weak, nonatomic) IBOutlet UILabel *addressLabel;
@property (weak, nonatomic) IBOutlet UILabel *distanceLabel;
@property (weak, nonatomic) IBOutlet UILabel *descriptionLabel;

@property (weak, nonatomic) IBOutlet UIImageView *addressImageView;
@property (weak, nonatomic) IBOutlet UIImageView *distanceImageView;

@property (weak, nonatomic) IBOutlet NSLayoutConstraint *addressImageViewVerticalConstraint;
@property (weak, nonatomic) IBOutlet NSLayoutConstraint *addressLabelVerticalConstraint;
@property (weak, nonatomic) IBOutlet NSLayoutConstraint *distanceLabelWidthConstraint;
@property (weak, nonatomic) IBOutlet NSLayoutConstraint *distanceImageViewVerticalConstraint;
@property (weak, nonatomic) IBOutlet NSLayoutConstraint *distanceLabelVerticalConstraint;
@property (weak, nonatomic) IBOutlet NSLayoutConstraint *websiteButtonHeightConstraint;
@property (weak, nonatomic) IBOutlet NSLayoutConstraint *websiteButtonVerticalConstraint;
@property (weak, nonatomic) IBOutlet NSLayoutConstraint *facebookButtonHeightConstraint;
@property (weak, nonatomic) IBOutlet NSLayoutConstraint *facebookButtonVerticalConstraint;
@property (weak, nonatomic) IBOutlet NSLayoutConstraint *phoneButtonVerticalConstraint;
@property (weak, nonatomic) IBOutlet NSLayoutConstraint *phoneButtonHeightConstraint;
@property (weak, nonatomic) IBOutlet NSLayoutConstraint *mobilePhoneButtonVerticalConstraint;
@property (weak, nonatomic) IBOutlet NSLayoutConstraint *mobilePhoneButtonHeightConstraint;
@property (weak, nonatomic) IBOutlet NSLayoutConstraint *descriptionLabelHeightConstraint;
@property (weak, nonatomic) IBOutlet NSLayoutConstraint *mapViewHeightConstraint;
@property (weak, nonatomic) IBOutlet NSLayoutConstraint *mapViewLeadingConstraint;
@property (weak, nonatomic) IBOutlet NSLayoutConstraint *mapViewTrailingConstraint;
@property (weak, nonatomic) IBOutlet NSLayoutConstraint *mapViewBottomConstraint;
@property (weak, nonatomic) IBOutlet NSLayoutConstraint *collectionViewHeightConstraint;
@property (weak, nonatomic) IBOutlet NSLayoutConstraint *collectionViewBottomConstraint;

@property (strong, nonatomic) NSNumber *placeId;

@property (assign, nonatomic) int imagesCount;
@property (assign, nonatomic) int currentPage;
@property (assign, nonatomic) int followed;
@property (assign, nonatomic) int shoppingCenterScrollViewOffset;

@property (assign, nonatomic) CGFloat collectionViewConstraint;

@property (assign, nonatomic) BOOL isMapOpened;

@end

@implementation ShoppingCenterViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    
    [URLRequests getShoppingCenter:self.shoppingCenterId.intValue receive:^(NSData *data) {
        NSError *error;
        self.shoppingCenterDictionary = [[NSJSONSerialization JSONObjectWithData:data options:0 error:&error] objectAtIndex:0];
        [self loadContent];
    }];
    
    [URLRequests getShoppingCenterPlaces:self.shoppingCenterId.intValue receive:^(NSData *data) {
        NSError *error;
        self.shoppingCenterPlacesArray = [[NSJSONSerialization JSONObjectWithData:data options:0 error:&error] objectForKey:@"data"];
        long count = self.shoppingCenterPlacesArray.count;
        if(count % 2 == 0) {
            self.collectionViewHeightConstraint.constant = count / 2  * 187;
        } else {
            self.collectionViewHeightConstraint.constant = (count + 1) / 2  * 187;
        }
        self.collectionViewConstraint = self.collectionViewHeightConstraint.constant;
        [self.shoppingCenterPlacesCollectionView reloadData];
    }];
    
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
}

-(void)loadContent {
    self.isMapOpened = NO;
    self.shoppingCenterScrollViewOffset = 0;
    
    self.imagesCount = 0;
    self.currentPage = 0;
    
//shopping center gallery
    NSString *imageUrl = [[self.shoppingCenterDictionary objectForKey:@"logo"] objectForKey:@"image"];
    NSString *folder = [self.shoppingCenterDictionary objectForKey:@"name"];
    folder = [folder stringByReplacingOccurrencesOfString:@" " withString:@"_"];
    if(imageUrl != nil) {
        UIImage *image = [URLRequests loadImage:imageUrl withFolder:nil receive:^(UIImage *image) {
            [self setImage:image];
        }];
        if(image != nil) {
            [self setImage:image];
        }
    }
    
    BOOL shouldSetDefaultImage = YES;
    if([self.shoppingCenterDictionary objectForKey:@"images"]) {
        NSArray<NSString *> *imagesArray = (NSArray<NSString *> *)[self.shoppingCenterDictionary objectForKey:@"images"];
        int imageIndex = 1;
        for(NSDictionary *imageDictionary in imagesArray) {
            shouldSetDefaultImage = NO;
            UIImage *image = [URLRequests loadImage:[imageDictionary objectForKey:@"image"] withFolder:folder receive:^(UIImage *image) {
                [self setImage:image];
            }];
            imageIndex++;
            if(image != nil) {
                [self setImage:image];
            }
        }
    }
    
    if(self.imagesCount == 0 && imageUrl == nil && shouldSetDefaultImage) {
        [self setImage:[UIImage imageNamed:@"no-logo"]];
        self.imagesScrollView.backgroundColor = [Utils colorFromHexString:@"#E6E7E8"];
    }
    
//shopping center address
    NSString *addressString = [[NSString alloc] init];
    NSDictionary *addressDictionary = [self.shoppingCenterDictionary objectForKey:@"address"];
    NSString *city = [addressDictionary objectForKey:@"city"];
    if(city != nil) {
        addressString = [addressString stringByAppendingString:[addressDictionary objectForKey:@"city"]];
    }
    NSString *address = [addressDictionary objectForKey:@"address1"];
    if(address != nil) {
        if(city != nil) {
            addressString = [addressString stringByAppendingString:@", "];
        }
        addressString = [addressString stringByAppendingString:[addressDictionary objectForKey:@"address1"]];
    }
    self.addressLabel.text = addressString;
    self.addressImageView.hidden = NO;
    
//shopping center distance
    CLLocation *shoppingCenterLocation = [[CLLocation alloc] initWithLatitude:[[self.shoppingCenterDictionary objectForKey:@"lat"] floatValue] longitude:[[self.shoppingCenterDictionary objectForKey:@"lng"] floatValue]];
    CLLocationDistance distance = [[Utils getLocation] distanceFromLocation:shoppingCenterLocation];
    if(distance < 1000) {
        self.distanceLabel.text = [NSString stringWithFormat:@"%d %@", (int)distance, [Utils translate:@"Meter"]];
    } else {
        self.distanceLabel.text = [NSString stringWithFormat:@"%.2f %@", distance / 1000, [Utils translate:@"Kilometer"]];
    }
    CGFloat labelWidth = [self.distanceLabel.text boundingRectWithSize:CGSizeMake(CGFLOAT_MAX, 20) options:NSStringDrawingUsesLineFragmentOrigin attributes: @{NSFontAttributeName : self.distanceLabel.font} context:nil].size.width;
    self.distanceLabelWidthConstraint.constant = labelWidth;
    self.distanceLabel.text = self.distanceLabel.text;
    self.distanceImageView.hidden = NO;
    
//place website button
    labelWidth = [[UIScreen mainScreen] bounds].size.width;
    NSString *website = [NSString stringWithFormat:@"%@", [self.shoppingCenterDictionary objectForKey:@"website"]];
    CGFloat labelHeight = [website boundingRectWithSize:CGSizeMake(labelWidth - 50, CGFLOAT_MAX) options:NSStringDrawingUsesLineFragmentOrigin attributes: @{NSFontAttributeName : self.websiteButton.titleLabel.font} context:nil].size.height;
    self.websiteButtonHeightConstraint.constant = labelHeight;
    if(website != nil && ![website isEqualToString:@""] && ![website isEqualToString:@"0"]) {
        [self.websiteButton setTitle:website forState:UIControlStateNormal];
        self.websiteButton.hidden = NO;
    } else {
        self.websiteButtonVerticalConstraint.constant = 0;
        self.websiteButtonHeightConstraint.constant = 0;
    }

//shopping center facebook button
    NSString *facebook = [NSString stringWithFormat:@"%@", [self.shoppingCenterDictionary objectForKey:@"facebook"]];
    labelHeight = [facebook boundingRectWithSize:CGSizeMake(labelWidth - 50, CGFLOAT_MAX) options:NSStringDrawingUsesLineFragmentOrigin attributes: @{NSFontAttributeName : self.facebookButton.titleLabel.font} context:nil].size.height;
    self.facebookButtonHeightConstraint.constant = labelHeight;
    if(facebook != nil && ![facebook isEqualToString:@""] && ![facebook isEqualToString:@"0"]) {
        [self.facebookButton setTitle:facebook forState:UIControlStateNormal];
        self.facebookButton.hidden = NO;
    } else {
        self.facebookButtonVerticalConstraint.constant = 0;
        self.facebookButtonHeightConstraint.constant = 0;
    }
    
//shopping center phone button
    NSString *phone = [NSString stringWithFormat:@"%@", [self.shoppingCenterDictionary objectForKey:@"phone"]];
    if(phone != nil && ![phone isEqualToString:@""] && ![phone isEqualToString:@"0"]) {
        [self.phoneButton setTitle:phone forState:UIControlStateNormal];
        self.phoneButton.hidden = NO;
    } else {
        self.phoneButtonVerticalConstraint.constant = 0;
        self.phoneButtonHeightConstraint.constant = 0;
    }
    
//shopping center mobile phone button
    phone = [NSString stringWithFormat:@"%@", [self.shoppingCenterDictionary objectForKey:@"mobile_phone"]];
    if(phone != nil && ![phone isEqualToString:@""] && ![phone isEqualToString:@"0"]) {
        [self.mobilePhoneButton setTitle:phone forState:UIControlStateNormal];
        self.mobilePhoneButton.hidden = NO;
    } else {
        self.mobilePhoneButtonVerticalConstraint.constant = 0;
        self.mobilePhoneButtonHeightConstraint.constant = 0;
    }
    
//shopping center description
    NSString *description = [NSString stringWithFormat:@"%@", [self.shoppingCenterDictionary objectForKey:@"description"]];
    labelHeight = [description boundingRectWithSize:CGSizeMake(labelWidth - 50, CGFLOAT_MAX) options:NSStringDrawingUsesLineFragmentOrigin attributes: @{NSFontAttributeName : self.descriptionLabel.font} context:nil].size.height;
    if(description != nil && ![description isEqualToString:@""]) {
        self.descriptionLabelHeightConstraint.constant = labelHeight;
        self.descriptionLabel.text = description;
    } else {
        self.descriptionLabelHeightConstraint.constant = 0;
    }
    
//shopping center add to favourites button
    if(((NSNumber *)[self.shoppingCenterDictionary objectForKey:@"followed"]).intValue == 0) {
        self.followed = 0;
        [self.addToFavouritesButton setTitle:[Utils translate:@"Follow"] forState:UIControlStateNormal];
    } else {
        self.followed = 1;
        [self.addToFavouritesButton setTitle:[Utils translate:@"Unfollow"] forState:UIControlStateNormal];
    }
    
//shopping center share button
    [self.shareButton setTitle:[Utils translate:@"Share"] forState:UIControlStateNormal];
    
//shopping center map annotation view
    PinAnnotation *annotation = [[PinAnnotation alloc] init];
    self.shoppingCenterCoordinate = CLLocationCoordinate2DMake([[self.shoppingCenterDictionary objectForKey:@"lat"] doubleValue], [[self.shoppingCenterDictionary objectForKey:@"lng"] doubleValue]);
    [annotation setCoordinate:self.shoppingCenterCoordinate];
    [self.mapView addAnnotation:annotation];
    MKMapCamera *camera = [MKMapCamera cameraLookingAtCenterCoordinate:self.shoppingCenterCoordinate fromEyeCoordinate:self.shoppingCenterCoordinate eyeAltitude:15000];
    double latitude = camera.centerCoordinate.latitude;
    double longitude = camera.centerCoordinate.longitude;
    if(latitude > -90 && latitude < 90 && longitude > - 180 && longitude < 180) {
        [self.mapView setCamera:camera animated:NO];
    }
    
//shopping center animation
    self.addressImageViewVerticalConstraint.constant = 30;
    self.addressLabelVerticalConstraint.constant = 30;
    self.distanceImageViewVerticalConstraint.constant = 30;
    self.distanceLabelVerticalConstraint.constant = 30;
    [self.view layoutIfNeeded];
    [UIView animateWithDuration:0.5 delay:0 options:UIViewAnimationOptionCurveLinear animations:^{
        self.addressImageViewVerticalConstraint.constant = 20;
        self.addressLabelVerticalConstraint.constant = 20;
        self.distanceImageViewVerticalConstraint.constant = 20;
        self.distanceLabelVerticalConstraint.constant = 20;
        self.addressImageView.alpha = 1;
        self.addressLabel.alpha = 1;
        self.distanceImageView.alpha = 1;
        self.distanceLabel.alpha = 1;
        self.websiteButton.alpha = 1;
        self.facebookButton.alpha = 1;
        self.phoneButton.alpha = 1;
        self.mobilePhoneButton.alpha = 1;
        self.addToFavouritesButton.alpha = 1;
        self.shareButton.alpha = 1;
        self.mapView.alpha = 1;
        self.mapControlButton.alpha = 1;
        self.shoppingCenterPlacesCollectionView.alpha = 1;
        [self.view layoutIfNeeded];
    } completion:^(BOOL finished) {
        
    }];
}

- (IBAction)onAddToFavouritesPressed:(id)sender {
    [URLRequests followPlace:self.followed withLocationId:[[self.shoppingCenterDictionary objectForKey:@"id"] intValue] receive:^(BOOL success) {
        if(success) {
            if(self.followed == 1) {
                self.followed = 0;
                [self.addToFavouritesButton setTitle:[Utils translate:@"Follow"] forState:UIControlStateNormal];
                UIViewController *viewController = self.navigationController.viewControllers.firstObject;
                if([viewController isKindOfClass:[CatalogueViewController class]]) {
                    for(ShoppingCenter *shoppingCenter in ((CatalogueViewController *)viewController).shoppingCentersArray) {
                        if(shoppingCenter.shoppingCenterId.intValue == self.shoppingCenterId.intValue) {
                            shoppingCenter.shoppingCenterFollowed = [NSNumber numberWithInt:self.followed];
                            break;
                        }
                    }
                    [((CatalogueViewController *)viewController).catalogueCollectionView reloadData];
                }
            } else {
                self.followed = 1;
                [self.addToFavouritesButton setTitle:[Utils translate:@"Unfollow"] forState:UIControlStateNormal];
                UIViewController *viewController = self.navigationController.viewControllers.firstObject;
                if([viewController isKindOfClass:[CatalogueViewController class]]) {
                    for(ShoppingCenter *shoppingCenter in ((CatalogueViewController *)viewController).shoppingCentersArray) {
                        if(shoppingCenter.shoppingCenterId.intValue == self.shoppingCenterId.intValue) {
                            shoppingCenter.shoppingCenterFollowed = [NSNumber numberWithInt:self.followed];
                            break;
                        }
                    }
                    [((CatalogueViewController *)viewController).catalogueCollectionView reloadData];
                }
            }
        }
    }];
}

- (IBAction)onSharePressed:(id)sender {
    if([FBSDKAccessToken currentAccessToken] != nil) {
        FBSDKShareLinkContent *content = [[FBSDKShareLinkContent alloc] init];
        content.contentURL = [NSURL URLWithString:@"http://www.unicreditbulbank.bg/bg/Individual_Clients/Plus_programme/About/index.htm"];
        content.contentTitle = [self.shoppingCenterDictionary objectForKey:@"name"];
        content.contentDescription = [NSString stringWithFormat:[Utils translate:@"Share description"], [self.shoppingCenterDictionary objectForKey:@"name"]];
        FBSDKShareDialog *shareDialog = [[FBSDKShareDialog alloc] init];
        shareDialog.fromViewController = self;
        shareDialog.shareContent = content;
        [shareDialog show];
    } else {
        NSArray *itemsToShare = [NSArray arrayWithObjects:[self.shoppingCenterDictionary objectForKey:@"name"], [NSString stringWithFormat:[Utils translate:@"Share description"], [self.shoppingCenterDictionary objectForKey:@"name"]], [NSURL URLWithString:@"http://www.unicreditbulbank.bg/bg/Individual_Clients/Plus_programme/About/index.htm"], nil];
        
        UIActivityViewController *activityVC = [[UIActivityViewController alloc] initWithActivityItems:itemsToShare applicationActivities:nil];
        activityVC.excludedActivityTypes = @[UIActivityTypePrint, UIActivityTypeCopyToPasteboard, UIActivityTypeAssignToContact, UIActivityTypeSaveToCameraRoll, UIActivityTypePostToWeibo];
        
        [self presentViewController:activityVC animated:YES completion:nil];
    }
}

- (IBAction)openWebView:(id)sender {
    WebViewController *webViewController = [[UIStoryboard storyboardWithName:@"Main" bundle:nil] instantiateViewControllerWithIdentifier:@"WebViewController"];
    NSURL *url = [NSURL URLWithString:((UIButton *)sender).titleLabel.text];
    NSURLRequest *request = [NSURLRequest requestWithURL:url];
    webViewController.request = request;
    webViewController.isFromMainDrawer = YES;
    [self.navigationController pushViewController:webViewController animated:YES];
}

- (IBAction)openDialer:(id)sender {
    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:[NSString stringWithFormat:@"Do you want to dial this number: %@ ?", ((UIButton *)sender).titleLabel.text] message:@"" preferredStyle:UIAlertControllerStyleAlert];
    [alertController addAction:[UIAlertAction actionWithTitle:@"No" style:UIAlertActionStyleDefault handler:^(UIAlertAction *action) {
        [self dismissViewControllerAnimated:YES completion:^{
        }];
    }]];
    
    [alertController addAction:[UIAlertAction actionWithTitle:@"Yes" style:UIAlertActionStyleCancel handler:^(UIAlertAction *action) {
        NSString *number = @"tel:";
        number = [number stringByAppendingString:((UIButton *)sender).titleLabel.text];
        number = [number stringByReplacingOccurrencesOfString:@" " withString:@""];
        NSURL *url = [NSURL URLWithString:number];
        if([[UIApplication sharedApplication] canOpenURL:url]) {
            [[UIApplication sharedApplication] openURL:url];
        }
    }]];
    
    [self presentViewController:alertController animated:YES completion:nil];
}

-(MKAnnotationView *) mapView:(MKMapView *)mapView viewForAnnotation:(id <MKAnnotation>) annotation {
    if(annotation == self.mapView.userLocation) {
        return nil;
    }
    
    static NSString *const pinIdentifier = @"mapPin";
    InfoAnnotationView *annotationView = nil;
    annotationView = (InfoAnnotationView *)[mapView dequeueReusableAnnotationViewWithIdentifier:pinIdentifier];
    if(annotationView == nil) {
        annotationView = [[InfoAnnotationView alloc] initWithAnnotation:annotation reuseIdentifier:pinIdentifier];
    }
    annotationView.annotation = annotation;
    
    UIImage *image = [UIImage imageNamed:@"malls-pin"];
    image = [Utils imageWithImage:image scaledToSize:CGSizeMake(40, 80)];
    annotationView.image = image;

    return annotationView;
}

- (NSInteger)collectionView:(UICollectionView *)collectionView numberOfItemsInSection:(NSInteger)section {
    return self.shoppingCenterPlacesArray.count;
}

- (CGSize)collectionView:(UICollectionView *)collectionView layout:(UICollectionViewLayout *)collectionViewLayout sizeForItemAtIndexPath:(NSIndexPath *)indexPath {
    CGRect screenBounds = [[UIScreen mainScreen] bounds];
    CGSize size = CGSizeMake((screenBounds.size.width - 45) / 2, 177);
    return size;
}

- (UICollectionViewCell *)collectionView:(UICollectionView *)collectionView cellForItemAtIndexPath:(NSIndexPath *)indexPath {
    ShoppingCenterPlaceCollectionViewCell *cell = (ShoppingCenterPlaceCollectionViewCell *)[collectionView dequeueReusableCellWithReuseIdentifier:@"Cell" forIndexPath:indexPath];
    cell.shoppingCenterViewController = self;
    
    NSDictionary *place = (NSDictionary *)[self.shoppingCenterPlacesArray objectAtIndex:indexPath.row];
    
    NSString *imageUrl = [[place objectForKey:@"logo"] objectForKey:@"image"];
    
    if(imageUrl != nil && ![imageUrl isEqualToString:@""] && ![imageUrl isEqualToString:@"null"]) {
        UIImage *image = [URLRequests loadImage:imageUrl withFolder:nil receive:^(UIImage *image) {
            for(NSIndexPath *indexPathForImage in [collectionView indexPathsForVisibleItems]) {
                if([[[[self.shoppingCenterPlacesArray objectAtIndex:indexPathForImage.row] objectForKey:@"logo"] objectForKey:@"image"] isEqualToString:imageUrl]) {
                    ((ShoppingCenterPlaceCollectionViewCell *)[collectionView cellForItemAtIndexPath:indexPathForImage]).placePlaceImageView.image = image;
                }
            }
        }];
        cell.placePlaceImageView.image = image;
    } else {
        cell.placePlaceImageView.image = [UIImage imageNamed:@"no-logo"];
        cell.placePlaceImageView.backgroundColor = [Utils colorFromHexString:@"E6E7E8"];
    }
    
    cell.placeNameLabel.text = [place objectForKey:@"name"];
    
    cell.placeCategoryLabel.text = [[[place objectForKey:@"categories"] objectAtIndex:0] objectForKey:@"name"];
    
    CLLocation *brandLocation = [[CLLocation alloc] initWithLatitude:[[place objectForKey:@"lat"] floatValue] longitude:[[place objectForKey:@"lng"] floatValue]];
    CLLocationDistance distance = [[Utils getLocation] distanceFromLocation:brandLocation];
    if(distance < 1000) {
        cell.placeDistanceLabel.text = [NSString stringWithFormat:@"%d %@", (int)distance, [Utils translate:@"Meter"]];
    } else {
        cell.placeDistanceLabel.text = [NSString stringWithFormat:@"%.2f %@", distance / 1000, [Utils translate:@"Kilometer"]];
    }
    CGFloat labelWidth = [cell.placeDistanceLabel.text boundingRectWithSize:CGSizeMake(CGFLOAT_MAX, 20) options:NSStringDrawingUsesLineFragmentOrigin attributes: @{NSFontAttributeName : cell.placeDistanceLabel.font} context:nil].size.width;
    cell.placeDistanceLabelWidthConstraint.constant = labelWidth;
    
    cell.locationId = [place objectForKey:@"id"];
    if([[place objectForKey:@"followed"] intValue] == 0) {
        [cell.placeFollowButton setImage:[UIImage imageNamed:@"add_to_favs"] forState:UIControlStateNormal];
        cell.followed = 0;
    } else {
        [cell.placeFollowButton setImage:[UIImage imageNamed:@"remove_from_favs"] forState:UIControlStateNormal];
        cell.followed = 1;
    }

    
    return cell;
}

- (void)collectionView:(UICollectionView *)collectionView didSelectItemAtIndexPath:(NSIndexPath *)indexPath {
    self.placeId = [NSNumber numberWithInteger:[[[self.shoppingCenterPlacesArray objectAtIndex:indexPath.item] objectForKey:@"id"] integerValue]];
    [self performSegueWithIdentifier:@"PlaceSegue" sender:self];
}

-(void)setImage:(UIImage *)image {
    CGRect frame;
    frame.origin.x = self.imagesScrollView.frame.size.width * self.imagesCount;
    frame.origin.y = 0;
    frame.size = self.imagesScrollView.frame.size;
    
    UIImageView *imageView = [[UIImageView alloc] initWithFrame:frame];
    imageView.image = image;
    imageView.contentMode = UIViewContentModeScaleAspectFit;
    
    UITapGestureRecognizer *imageTap = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(openGallery)];
    imageTap.numberOfTapsRequired = 1;
    imageTap.numberOfTouchesRequired = 1;
    
    imageView.userInteractionEnabled = YES;
    [imageView addGestureRecognizer:imageTap];
    
    [self.imagesScrollView addSubview:imageView];
    
    self.imagesCount++;
    self.imagesScrollView.contentSize = CGSizeMake(self.imagesScrollView.frame.size.width * self.imagesCount, self.imagesScrollView.frame.size.height);
    
    if(self.imagesCount <= 1) {
        self.leftArrowButton.hidden = YES;
        self.leftArrowButton.userInteractionEnabled = NO;
        self.rightArrowButton.hidden = YES;
        self.rightArrowButton.userInteractionEnabled = NO;
    } else {
        self.leftArrowButton.hidden = NO;
        self.leftArrowButton.userInteractionEnabled = YES;
        self.rightArrowButton.hidden = NO;
        self.rightArrowButton.userInteractionEnabled = YES;
    }
}

-(void)openGallery {
    [self performSegueWithIdentifier:@"GallerySegue" sender:self];
}

- (void)scrollViewDidScroll:(UIScrollView *)scrollView {
    if(scrollView == self.imagesScrollView) {
        self.currentPage = (scrollView.contentOffset.x + scrollView.frame.size.width / 2) / scrollView.frame.size.width;
    } else {
        self.shoppingCenterScrollViewOffset = scrollView.contentOffset.y;
    }
}

- (IBAction)onLeftArrowPressed:(id)sender {
    if(self.currentPage > 0) {
        self.currentPage--;
        [UIView animateWithDuration:0.3 animations:^{
            [self.imagesScrollView setContentOffset:CGPointMake(self.currentPage * self.imagesScrollView.frame.size.width, 0)];
        }];
    }
}

- (IBAction)onRightArrowPressed:(id)sender {
    if(self.currentPage < self.imagesCount - 1) {
        self.currentPage++;
        [UIView animateWithDuration:0.3 animations:^{
            [self.imagesScrollView setContentOffset:CGPointMake(self.currentPage * self.imagesScrollView.frame.size.width, 0)];
        }];
    }
}

- (IBAction)mapControl:(id)sender {
    CGFloat screenHeight = [[UIScreen mainScreen] bounds].size.height;
    if(!self.isMapOpened) {
        self.shoppingCenterScrollView.scrollEnabled = NO;
        [self.view layoutIfNeeded];
        [UIView animateWithDuration:0.3 delay:0.0 options:UIViewAnimationOptionCurveLinear animations:^{
            self.shoppingCenterScrollView.contentOffset = CGPointMake(self.shoppingCenterScrollView.contentOffset.x, self.shoppingCenterScrollView.contentSize.height);
            self.mapViewHeightConstraint.constant = screenHeight - 50;
            self.mapViewLeadingConstraint.constant = 0;
            self.mapViewTrailingConstraint.constant = 0;
            self.mapViewBottomConstraint.constant = 0;
            self.collectionViewHeightConstraint.constant = 0;
            self.collectionViewBottomConstraint.constant = 0;
            CATransition *fade = [CATransition animation];
            [self.mapControlButton.layer addAnimation:fade forKey:@"fade"];
            [self.mapControlButton setImage:[UIImage imageNamed:@"shrink_map"] forState:UIControlStateNormal];
            [self.view layoutIfNeeded];
        } completion:^(BOOL finished) {
            self.mapView.scrollEnabled = YES;
            self.mapView.zoomEnabled = YES;
            self.mapView.showsUserLocation = YES;
            ((CatalogueViewController *)[self.navigationController.viewControllers objectAtIndex:0]).catalogueControlViewController.backButton.userInteractionEnabled = NO;
        }];
        self.isMapOpened = YES;
    } else {
        MKMapCamera *camera = [MKMapCamera cameraLookingAtCenterCoordinate:self.shoppingCenterCoordinate fromEyeCoordinate:self.shoppingCenterCoordinate eyeAltitude:self.mapView.camera.altitude];
        double latitude = camera.centerCoordinate.latitude;
        double longitude = camera.centerCoordinate.longitude;
        if(latitude > -90 && latitude < 90 && longitude > - 180 && longitude < 180) {
            [self.mapView setCamera:camera animated:YES];
        }
        [self.view layoutIfNeeded];
        [UIView animateWithDuration:0.3 delay:0.0 options:UIViewAnimationOptionCurveLinear animations:^{
            self.shoppingCenterScrollView.contentOffset = CGPointMake(self.shoppingCenterScrollView.contentOffset.x, self.shoppingCenterScrollViewOffset - self.collectionViewConstraint);
            self.mapViewHeightConstraint.constant = 135;
            self.mapViewLeadingConstraint.constant = 15;
            self.mapViewTrailingConstraint.constant = 15;
            self.mapViewBottomConstraint.constant = 20;
            self.collectionViewHeightConstraint.constant = self.collectionViewConstraint;
            self.collectionViewBottomConstraint.constant = 20;
            CATransition *fade = [CATransition animation];
            [self.mapControlButton.layer addAnimation:fade forKey:@"fade"];
            [self.mapControlButton setImage:[UIImage imageNamed:@"expand_map"] forState:UIControlStateNormal];
            [self.view layoutIfNeeded];
        } completion:^(BOOL finished) {
            self.shoppingCenterScrollView.scrollEnabled = YES;
            self.mapView.scrollEnabled = NO;
            self.mapView.zoomEnabled = NO;
            self.mapView.showsUserLocation = NO;
            ((CatalogueViewController *)[self.navigationController.viewControllers objectAtIndex:0]).catalogueControlViewController.backButton.userInteractionEnabled = YES;
        }];
        self.isMapOpened = NO;
    }
}


#pragma mark - Navigation

- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    if([[segue destinationViewController] isKindOfClass:[GalleryViewController class]]) {
        NSMutableArray<UIImage *> *imagesArray = [[NSMutableArray alloc] initWithCapacity:0];
        for(UIView *subview in self.imagesScrollView.subviews) {
            if([subview isKindOfClass:[UIImageView class]]) {
                [imagesArray addObject:((UIImageView *)subview).image];
            }
        }
        GalleryViewController *galleryController = (GalleryViewController *)[segue destinationViewController];
        galleryController.imagesArray = imagesArray;
        galleryController.currentPage = self.currentPage;
        galleryController.onDismiss = ^(int currentPage) {
            [self.imagesScrollView setContentOffset:CGPointMake(currentPage * self.imagesScrollView.frame.size.width, 0)];
            self.currentPage = currentPage;
        };
        galleryController.isFromMainDrawer = YES;
    } else if([[segue destinationViewController] isKindOfClass:[PlaceViewController class]]) {
        PlaceViewController *placeViewController = (PlaceViewController *)[segue destinationViewController];
        placeViewController.placeId = self.placeId;
        placeViewController.headerTitle = @"";
        placeViewController.headerFieldType = @"";
        placeViewController.oldTitle = [self.shoppingCenterDictionary objectForKey:@"name"];
        placeViewController.isFromMainDrawer = YES;
    }
}


@end
