//
//  SingleBrandViewController.m
//  plus
//
//  Created by Виктор Георгиев on 8/29/16.
//  Copyright © 2016 Outsourcing IT Services Ltd. All rights reserved.
//

#define NEW 0
#define PROMOTIONS 1
#define FAVOURITES 2
#define SHOPPING_CENTERS 3

#import "AppDelegate.h"
#import <FBSDKCoreKit/FBSDKCoreKit.h>
#import <FBSDKShareKit/FBSDKShareKit.h>

#import "SingleBrandViewController.h"
#import "MainMenuViewController.h"
#import "ContentHolderViewController.h"
#import "GalleryViewController.h"
#import "PlaceViewController.h"
#import "WebViewController.h"
#import "NotificationsViewController.h"
#import "NotificationsHolderViewController.h"
#import "BrandsViewController.h"
#import "CatalogueViewController.h"

#import "PromotionTableViewCell.h"
#import "PlaceWithoutPromoTableViewCell.h"
#import "PlaceWithPromoTableViewCell.h"

#import "ItemCategory.h"
#import "Filter.h"
#import "URLRequests.h"
#import "Utils.h"
#import "PinAnnotation.h"
#import "InfoAnnotationView.h"
#import "Promotion.h"
#import "NewPlace.h"
#import "ShoppingCenter.h"

@interface SingleBrandViewController ()

@property (strong, nonatomic) NSDictionary *brandDictionary;

@property (assign, nonatomic) CLLocationCoordinate2D placeCoordinate;

@property (strong, nonatomic) ItemCategory *category;

@property (strong, nonatomic) Filter *filter;

@property (weak, nonatomic) IBOutlet UIScrollView *brandScrollView;
@property (weak, nonatomic) IBOutlet UIScrollView *imagesScrollView;

@property (weak, nonatomic) IBOutlet UITableView *placesTableView;

@property (weak, nonatomic) IBOutlet UIButton *backButton;
@property (weak, nonatomic) IBOutlet UIButton *websiteButton;
@property (weak, nonatomic) IBOutlet UIButton *phoneButton;
@property (weak, nonatomic) IBOutlet UIButton *mobilePhoneButton;
@property (weak, nonatomic) IBOutlet UIButton *addToFavouritesButton;
@property (weak, nonatomic) IBOutlet UIButton *shareButton;
@property (weak, nonatomic) IBOutlet UIButton *mapControlButton;
@property (weak, nonatomic) IBOutlet UIButton *leftArrowButton;
@property (weak, nonatomic) IBOutlet UIButton *rightArrowButton;

@property (weak, nonatomic) IBOutlet UIImageView *categoryImageView;

@property (weak, nonatomic) IBOutlet UILabel *brandNameLabel;
@property (weak, nonatomic) IBOutlet UILabel *categoryLabel;

@property (weak, nonatomic) IBOutlet NSLayoutConstraint *categoryImageViewVerticalConstraint;
@property (weak, nonatomic) IBOutlet NSLayoutConstraint *categoryLabelVerticalConstraint;
@property (weak, nonatomic) IBOutlet NSLayoutConstraint *websiteButtonVerticalConstraint;
@property (weak, nonatomic) IBOutlet NSLayoutConstraint *websiteButtonHeightConstraint;
@property (weak, nonatomic) IBOutlet NSLayoutConstraint *phoneButtonVerticalConstraint;
@property (weak, nonatomic) IBOutlet NSLayoutConstraint *phoneButtonHeightConstraint;
@property (weak, nonatomic) IBOutlet NSLayoutConstraint *mobilePhoneButtonVerticalConstraint;
@property (weak, nonatomic) IBOutlet NSLayoutConstraint *mobilePhoneButtonHeightConstraint;
@property (weak, nonatomic) IBOutlet NSLayoutConstraint *placesTableViewHeightConstraint;
@property (weak, nonatomic) IBOutlet NSLayoutConstraint *mapViewHeightConstraint;
@property (weak, nonatomic) IBOutlet NSLayoutConstraint *mapViewLeadingConstraint;
@property (weak, nonatomic) IBOutlet NSLayoutConstraint *mapViewTrailingConstraint;
@property (weak, nonatomic) IBOutlet NSLayoutConstraint *mapViewBottomConstraint;

@property (strong, nonatomic) NSArray<NSDictionary *> *promotionsArray;
@property (strong, nonatomic) NSArray<NSDictionary *> *placesArray;

@property (strong, nonatomic) NSMutableArray<NSNumber *> *promotionsTableViewRows;

@property (weak, nonatomic) IBOutlet MKMapView *mapView;

@property (strong, nonatomic) NSNumber *locationId;

@property (assign, nonatomic) int imagesCount;
@property (assign, nonatomic) int currentPage;
@property (assign, nonatomic) int followed;

@property (assign, nonatomic) float brandScrollViewOffset;

@property (assign, nonatomic) BOOL initialLaunch;
@property (assign, nonatomic) BOOL isMapOpened;
@property (assign, nonatomic) BOOL hasSelectedAnnotationView;

@end

@implementation SingleBrandViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    
    if(self.isFromMainDrawer) {
        self.backButton.hidden = YES;
        self.backButton.userInteractionEnabled = NO;
    }
    
    self.initialLaunch = YES;
    
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(changeLanguage:) name:@"LanguageNotification" object:nil];
}

-(void)viewDidAppear:(BOOL)animated {
    [super viewDidAppear:animated];
    
    if(self.initialLaunch) {
        if(!self.isFromMainDrawer) {
            CGRect rect = self.backButton.bounds;
            UIBezierPath *maskPath = [UIBezierPath bezierPathWithRoundedRect:rect byRoundingCorners:UIRectCornerTopRight cornerRadii:CGSizeMake(10, 10)];
            CAShapeLayer *maskLayer = [CAShapeLayer layer];
            maskLayer.fillColor = [Utils colorFromHexString:@"#46ADC6"].CGColor;
            maskLayer.frame = rect;
            maskLayer.path = maskPath.CGPath;
            [self.backButton.layer insertSublayer:maskLayer atIndex:0];
        }
        
        [URLRequests getBrand:self.brandId.intValue receive:^(NSData *data) {
            NSError *error;
            self.brandDictionary = [NSJSONSerialization JSONObjectWithData:data options:0 error:&error];
            
            if(!self.isFromMainDrawer) {
                [((MainMenuViewController *)[((ContentHolderViewController *)[self.navigationController.viewControllers objectAtIndex:0]).mainNavigationController.viewControllers objectAtIndex:0]) changeHeader:[self.brandDictionary objectForKey:@"name"] withType:@"title"];
            } else {
                if([self.navigationController.viewControllers.firstObject isKindOfClass:[NotificationsViewController class]]) {
                    NotificationsHolderViewController *notificationsHolderViewController = (NotificationsHolderViewController *)((NotificationsViewController *)[self.navigationController.viewControllers objectAtIndex:0]).notificationsHolderViewController;
                    notificationsHolderViewController.headerLabel.text = [self.brandDictionary objectForKey:@"name"];
                    notificationsHolderViewController.headerLabel.textColor = [UIColor whiteColor];
                    notificationsHolderViewController.headerImageView.hidden = YES;
                    notificationsHolderViewController.headerNotificationCountView.hidden = YES;
                    notificationsHolderViewController.headerLabelCenterConstraint.constant = 0;
                    
                }
            }
            
            self.brandCategoryId = (NSNumber *)[self.brandDictionary objectForKey:@"category_id"];
            
            for(ItemCategory *itemCategory in [Utils getCategories]) {
                if(itemCategory.itemId.intValue == self.brandCategoryId.intValue) {
                    self.category = itemCategory;
                    break;
                } else {
                    BOOL found = NO;
                    for(ItemCategory *itemSubcategory in itemCategory.itemSubcategories) {
                        if(itemSubcategory.itemId.intValue == self.brandCategoryId.intValue) {
                            self.category = itemSubcategory;
                            found = YES;
                            break;
                        }
                    }
                    
                    if(found) {
                        break;
                    }
                }
            }
            
            self.promotionsArray = (NSArray<NSDictionary *> *)[self.brandDictionary objectForKey:@"promotions"];
            int promotionTableViewHeight = 0;
            for(NSDictionary *promotion in self.promotionsArray ) {
                CGFloat labelHeight = [[promotion objectForKey:@"title"] boundingRectWithSize:CGSizeMake([UIScreen mainScreen].bounds.size.width - 50, CGFLOAT_MAX) options:NSStringDrawingUsesLineFragmentOrigin attributes: @{NSFontAttributeName : [UIFont fontWithName:@"PTSans-Regular" size:16]} context:nil].size.height;
                promotionTableViewHeight += labelHeight + 77;
            }
            self.promotionsTableViewHeightConstraint.constant = promotionTableViewHeight;
            
            self.placesArray = (NSArray<NSDictionary *> *)[self.brandDictionary objectForKey:@"locations"];
            int placesTableViewHeight = 0;
            for(NSDictionary *place in self.placesArray) {
                NSString *address =[place objectForKey:@"city"];
                address = [address stringByAppendingFormat:@", %@", [place objectForKey:@"address1"]];
                CGFloat height = [address boundingRectWithSize:CGSizeMake([UIScreen mainScreen].bounds.size.width - 82, CGFLOAT_MAX) options:NSStringDrawingUsesLineFragmentOrigin attributes:@{NSFontAttributeName : [UIFont fontWithName:@"PTSans-Regular" size:16]} context:nil].size.height;
                if([place objectForKey:@"promotions"]) {
                    height += 99;
                    for(NSDictionary *promo in [place objectForKey:@"promotions"]) {
                        height += [[promo objectForKey:@"title"] boundingRectWithSize:CGSizeMake([UIScreen mainScreen].bounds.size.width - 82, CGFLOAT_MAX) options:NSStringDrawingUsesLineFragmentOrigin attributes:@{NSFontAttributeName : [UIFont fontWithName:@"PTSans-Regular" size:16]} context:nil].size.height + 36;
                    }
                    placesTableViewHeight += height;
                } else {
                    height += 83;
                    placesTableViewHeight += height;
                }
            }
            self.placesTableViewHeightConstraint.constant = placesTableViewHeight;
            
            [self loadContent];
        }];
        
        self.initialLaunch = NO;
    }
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
}

-(void)loadContent {
    self.imagesCount = 0;
    self.isMapOpened = NO;
    self.currentPage = 0;
    
//brand gallery
    NSString *imageUrl = [self.brandDictionary objectForKey:@"image"];
    NSString *folder = [self.brandDictionary objectForKey:@"name"];
    folder = [folder stringByReplacingOccurrencesOfString:@" " withString:@"_"];
    if(imageUrl != nil) {
        UIImage *image = [URLRequests loadImage:imageUrl withFolder:nil receive:^(UIImage *image) {
            [self setImage:image];
        }];
        if(image != nil) {
            [self setImage:image];
        }
    }
    
    NSArray<NSString *> *imagesArray = (NSArray<NSString *> *)[self.brandDictionary objectForKey:@"images"];
    int imageIndex = 1;
    for(NSString *imageUrl in imagesArray) {
        UIImage *image = [URLRequests loadImage:imageUrl withFolder:folder receive:^(UIImage *image) {
            [self setImage:image];
        }];
        imageIndex++;
        if(image != nil) {
            [self setImage:image];
        }
    }
    self.imagesScrollView.hidden = NO;
    
    if(self.imagesCount == 0 && imageUrl == nil && imagesArray.count == 0) {
        //self.brandNameLabel.text = [self.brandDictionary objectForKey:@"name"];
        //self.brandNameLabel.hidden = NO;
        self.brandNameLabel.hidden = YES;
        [self setImage:[UIImage imageNamed:@"no-logo"]];
        self.imagesScrollView.backgroundColor = [Utils colorFromHexString:@"#E6E7E8"];
    } else {
        self.brandNameLabel.hidden = YES;
    }
    
//brand category
    UIImage *image = [URLRequests loadImage:self.category.itemCategoryPinIcon withFolder:nil receive:^(UIImage *image) {
        self.categoryImageView.image = image;
    }];
    self.categoryImageView.image = image;
    
    NSFetchRequest *request = [NSFetchRequest fetchRequestWithEntityName:@"Filter"];
    request.predicate = [NSPredicate predicateWithFormat:@"filterId == %d", self.category.itemId.intValue];
    NSManagedObjectContext *managedObjectContext = ((AppDelegate *)[UIApplication sharedApplication].delegate).managedObjectContext;
    NSError *error;
    NSArray<Filter *> *filtersArray = (NSArray<Filter *> *)[managedObjectContext executeFetchRequest:request error:&error];
    if(filtersArray.count == 1) {
        self.filter = [filtersArray objectAtIndex:0];
        if([[Utils currentLanguage] isEqualToString:@"bg"]) {
            self.categoryLabel.text = self.filter.filterBgName;
        } else {
            self.categoryLabel.text = self.filter.filterEnName;
        }
        self.categoryLabel.textColor = [Utils colorFromHexString:self.category.itemCategoryPinColor];
    }
    
//brand website button
    CGFloat labelWidth = [[UIScreen mainScreen] bounds].size.width;
    NSString *website = [NSString stringWithFormat:@"%@", [self.brandDictionary objectForKey:@"website"]];
    CGFloat labelHeight = [website boundingRectWithSize:CGSizeMake(labelWidth - 50, CGFLOAT_MAX) options:NSStringDrawingUsesLineFragmentOrigin attributes: @{NSFontAttributeName : self.websiteButton.titleLabel.font} context:nil].size.height;
    self.websiteButtonHeightConstraint.constant = labelHeight;
    if(website != nil && ![website isEqualToString:@""] && ![website isEqualToString:@"0"]) {
        [self.websiteButton setTitle:website forState:UIControlStateNormal];
        self.websiteButton.hidden = NO;
    } else {
        self.websiteButtonVerticalConstraint.constant = 0;
        self.websiteButtonHeightConstraint.constant = 0;
    }
    
//brand phone button
    if([self.brandDictionary objectForKey:@"phone"] != nil) {
        NSString *phone = [NSString stringWithFormat:@"%@", [self.brandDictionary objectForKey:@"phone"]];
        if(![phone isEqualToString:@""] && ![phone isEqualToString:@"0"]) {
            [self.phoneButton setTitle:phone forState:UIControlStateNormal];
            self.phoneButton.hidden = NO;
        } else {
            self.phoneButtonVerticalConstraint.constant = 0;
            self.phoneButtonHeightConstraint.constant = 0;
        }
    } else {
        self.phoneButtonVerticalConstraint.constant = 0;
        self.phoneButtonHeightConstraint.constant = 0;
    }
    
//brand mobile phone button
    if([self.brandDictionary objectForKey:@"mobile_phone"] != nil) {
        NSString *phone = [NSString stringWithFormat:@"%@", [self.brandDictionary objectForKey:@"mobile_phone"]];
        if(![phone isEqualToString:@""] && ![phone isEqualToString:@"0"]) {
            [self.mobilePhoneButton setTitle:phone forState:UIControlStateNormal];
            self.mobilePhoneButton.hidden = NO;
        } else {
            self.mobilePhoneButtonVerticalConstraint.constant = 0;
            self.mobilePhoneButtonHeightConstraint.constant = 0;
        }
    } else {
        self.mobilePhoneButtonVerticalConstraint.constant = 0;
        self.mobilePhoneButtonHeightConstraint.constant = 0;
    }
    
//brand promotion
    self.promotionsTableViewRows = [[NSMutableArray alloc] initWithCapacity:self.promotionsArray.count];
    for(int i = 0; i < self.promotionsArray.count; i++) {
        [self.promotionsTableViewRows addObject:[NSNumber numberWithInt:0]];
    }
    [self.promotionsTableView reloadData];
    
//brand add to favourites button
    if(((NSNumber *)[self.brandDictionary objectForKey:@"followed"]).intValue == 0) {
        self.followed = 0;
        [self.addToFavouritesButton setTitle:[Utils translate:@"Follow"] forState:UIControlStateNormal];
    } else {
        self.followed = 1;
        [self.addToFavouritesButton setTitle:[Utils translate:@"Unfollow"] forState:UIControlStateNormal];
    }
    
//brand share button
    [self.shareButton setTitle:[Utils translate:@"Share"] forState:UIControlStateNormal];
    
//brand places
    [self.placesTableView reloadData];
    
//brand map annotation view
    for(NSDictionary *place in self.placesArray) {
        PinAnnotation *annotation = [[PinAnnotation alloc] init];
        annotation.locationId = [place objectForKey:@"id"];
        self.placeCoordinate = CLLocationCoordinate2DMake([[place objectForKey:@"lat"] doubleValue], [[place objectForKey:@"lng"] doubleValue]);
        [annotation setCoordinate:self.placeCoordinate];
        [self.mapView addAnnotation:annotation];
    }
    if(self.mapView.annotations.count == 1) {
        if(self.placeCoordinate.latitude != 0 && self.placeCoordinate.longitude != 0) {
            MKMapCamera *camera = [MKMapCamera cameraLookingAtCenterCoordinate:self.placeCoordinate fromEyeCoordinate:self.placeCoordinate eyeAltitude:15000];
            double latitude = camera.centerCoordinate.latitude;
            double longitude = camera.centerCoordinate.longitude;
            if(latitude > -90 && latitude < 90 && longitude > - 180 && longitude < 180) {
                [self.mapView setCamera:camera animated:NO];
            }
        }
    } else {
        [self.mapView showAnnotations:self.mapView.annotations animated:YES];
    }
    
    
//brand animation
    self.categoryImageViewVerticalConstraint.constant = 40;
    self.categoryLabelVerticalConstraint.constant = 40;
    [self.view layoutIfNeeded];
    [UIView animateWithDuration:0.5 delay:0 options:UIViewAnimationOptionCurveLinear animations:^{
        self.categoryImageViewVerticalConstraint.constant = 30;
        self.categoryLabelVerticalConstraint.constant = 30;
        self.categoryImageView.alpha = 1;
        self.categoryLabel.alpha = 1;
        self.websiteButton.alpha = 1;
        self.phoneButton.alpha = 1;
        self.mobilePhoneButton.alpha = 1;
        self.addToFavouritesButton.alpha = 1;
        self.shareButton.alpha = 1;
        //self.mapView.alpha = 1;
        //self.expandMapButton.alpha = 1;
        [self.view layoutIfNeeded];
    } completion:^(BOOL finished) {
        
    }];
}

- (IBAction)openWebView:(id)sender {
    WebViewController *webViewController = [[UIStoryboard storyboardWithName:@"Main" bundle:nil] instantiateViewControllerWithIdentifier:@"WebViewController"];
    NSURL *url = [NSURL URLWithString:((UIButton *)sender).titleLabel.text];
    NSURLRequest *request = [NSURLRequest requestWithURL:url];
    webViewController.request = request;
    webViewController.isFromMainDrawer = NO;
    [self.navigationController pushViewController:webViewController animated:YES];
}

- (IBAction)openDialer:(id)sender {
    UIAlertController *alertController = [UIAlertController alertControllerWithTitle:[NSString stringWithFormat:@"Do you want to dial this number: %@ ?", ((UIButton *)sender).titleLabel.text] message:@"" preferredStyle:UIAlertControllerStyleAlert];
    [alertController addAction:[UIAlertAction actionWithTitle:@"No" style:UIAlertActionStyleDefault handler:^(UIAlertAction *action) {
        [self dismissViewControllerAnimated:YES completion:^{
        }];
    }]];
    
    [alertController addAction:[UIAlertAction actionWithTitle:@"Yes" style:UIAlertActionStyleCancel handler:^(UIAlertAction *action) {
        NSString *number = @"tel:";
        number = [number stringByAppendingString:((UIButton *)sender).titleLabel.text];
        number = [number stringByReplacingOccurrencesOfString:@" " withString:@""];
        NSURL *url = [NSURL URLWithString:number];
        if([[UIApplication sharedApplication] canOpenURL:url]) {
            [[UIApplication sharedApplication] openURL:url];
        }
    }]];
    
    [self presentViewController:alertController animated:YES completion:nil];
}

-(MKAnnotationView *) mapView:(MKMapView *)mapView viewForAnnotation:(id <MKAnnotation>) annotation {
    if(annotation == self.mapView.userLocation) {
        return nil;
    }
    
    static NSString *const pinIdentifier = @"mapPin";
    InfoAnnotationView *annotationView = nil;
    annotationView = (InfoAnnotationView *)[mapView dequeueReusableAnnotationViewWithIdentifier:pinIdentifier];
    if(annotationView == nil) {
        annotationView = [[InfoAnnotationView alloc] initWithAnnotation:annotation reuseIdentifier:pinIdentifier];
    }
    annotationView.annotation = annotation;
    
    if(self.category.itemId.intValue != -1) {
        UIImage *image = [URLRequests loadImage:self.category.itemMapIcon withFolder:nil receive:^(UIImage *image) {
        }];
        if(image != nil) {
            image = [Utils imageWithImage:image scaledToSize:CGSizeMake(40, 80)];
            annotationView.image = image;
        }
    }
    
    return annotationView;
}

- (void)mapView:(MKMapView *)mapView didSelectAnnotationView:(MKAnnotationView *)view {
    if(view.annotation != mapView.userLocation) {
        self.locationId = ((PinAnnotation *)view.annotation).locationId;
        self.hasSelectedAnnotationView = YES;
        [self performSegueWithIdentifier:@"PlaceSegue" sender:self];
    }
}

- (IBAction)mapControl:(id)sender {
    CGFloat screenHeight = [[UIScreen mainScreen] bounds].size.height;
    if(!self.isMapOpened) {
        self.brandScrollView.scrollEnabled = NO;
        [self.view layoutIfNeeded];
        [UIView animateWithDuration:0.3 delay:0.0 options:UIViewAnimationOptionCurveLinear animations:^{
            self.brandScrollView.contentOffset = CGPointMake(self.brandScrollView.contentOffset.x, self.brandScrollView.frame.size.height + (screenHeight - 50));
            self.mapViewHeightConstraint.constant = screenHeight - 50;
            self.mapViewLeadingConstraint.constant = 0;
            self.mapViewTrailingConstraint.constant = 0;
            self.mapViewBottomConstraint.constant = 0;
            CATransition *fade = [CATransition animation];
            [self.mapControlButton.layer addAnimation:fade forKey:@"fade"];
            [self.mapControlButton setImage:[UIImage imageNamed:@"shrink_map"] forState:UIControlStateNormal];
            self.backButton.alpha = 0;
            [self.view layoutIfNeeded];
        } completion:^(BOOL finished) {
            self.mapView.scrollEnabled = YES;
            self.mapView.zoomEnabled = YES;
            self.backButton.userInteractionEnabled = NO;
            self.mapView.showsUserLocation = YES;
        }];
        self.isMapOpened = YES;
    } else {
        MKMapCamera *camera = [MKMapCamera cameraLookingAtCenterCoordinate:self.placeCoordinate fromEyeCoordinate:self.placeCoordinate eyeAltitude:15000];
        double latitude = camera.centerCoordinate.latitude;
        double longitude = camera.centerCoordinate.longitude;
        if(latitude > -90 && latitude < 90 && longitude > - 180 && longitude < 180) {
            [self.mapView setCamera:camera animated:YES];
        }
        [self.view layoutIfNeeded];
        [UIView animateWithDuration:0.3 delay:0.0 options:UIViewAnimationOptionCurveLinear animations:^{
            self.brandScrollView.contentOffset = CGPointMake(self.brandScrollView.contentOffset.x, self.brandScrollViewOffset);
            self.mapViewHeightConstraint.constant = 135;
            self.mapViewLeadingConstraint.constant = 15;
            self.mapViewTrailingConstraint.constant = 15;
            self.mapViewBottomConstraint.constant = 25;
            CATransition *fade = [CATransition animation];
            [self.mapControlButton.layer addAnimation:fade forKey:@"fade"];
            [self.mapControlButton setImage:[UIImage imageNamed:@"expand_map"] forState:UIControlStateNormal];
            self.backButton.alpha = 1;
            [self.view layoutIfNeeded];
        } completion:^(BOOL finished) {
            self.brandScrollView.scrollEnabled = YES;
            self.mapView.scrollEnabled = NO;
            self.mapView.zoomEnabled = NO;
            self.backButton.userInteractionEnabled = YES;
            self.mapView.showsUserLocation = NO;
        }];
        self.isMapOpened = NO;
    }
}

-(void)setImage:(UIImage *)image {
    CGRect frame;
    frame.origin.x = self.imagesScrollView.frame.size.width * self.imagesCount;
    frame.origin.y = 0;
    frame.size = self.imagesScrollView.frame.size;
    
    UIImageView *imageView = [[UIImageView alloc] initWithFrame:frame];
    imageView.image = image;
    imageView.contentMode = UIViewContentModeScaleAspectFit;
    if([self.brandDictionary objectForKey:@"color"]) {
        imageView.backgroundColor = [Utils colorFromHexString:[NSString stringWithFormat:@"#%@", [self.brandDictionary objectForKey:@"color"]]];
    }
    
    UITapGestureRecognizer *imageTap = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(openGallery)];
    imageTap.numberOfTapsRequired = 1;
    imageTap.numberOfTouchesRequired = 1;
    
    imageView.userInteractionEnabled = YES;
    [imageView addGestureRecognizer:imageTap];
    
    [self.imagesScrollView addSubview:imageView];
    
    self.imagesCount++;
    self.imagesScrollView.contentSize = CGSizeMake(self.imagesScrollView.frame.size.width * self.imagesCount, self.imagesScrollView.frame.size.height);
    
    if(self.imagesCount <= 1) {
        self.leftArrowButton.hidden = YES;
        self.leftArrowButton.userInteractionEnabled = NO;
        self.rightArrowButton.hidden = YES;
        self.rightArrowButton.userInteractionEnabled = NO;
    } else {
        self.leftArrowButton.hidden = NO;
        self.leftArrowButton.userInteractionEnabled = YES;
        self.rightArrowButton.hidden = NO;
        self.rightArrowButton.userInteractionEnabled = YES;
    }
}

-(int)changeRowHeight:(long)index withCurrentHeight:(int)currentHeight {
    int height = 0;
    if([self.promotionsTableViewRows objectAtIndex:index].intValue == 0) {
        [self.promotionsTableViewRows setObject:[NSNumber numberWithInt:1] atIndexedSubscript:index];
        if([[self.promotionsArray objectAtIndex:index] objectForKey:@"content"]) {
            CGFloat labelHeight = [[[self.promotionsArray objectAtIndex:index] objectForKey:@"content"] boundingRectWithSize:CGSizeMake([UIScreen mainScreen].bounds.size.width - 50, CGFLOAT_MAX) options:NSStringDrawingUsesLineFragmentOrigin attributes: @{NSFontAttributeName : [UIFont fontWithName:@"PTSans-Regular" size:14]} context:nil].size.height;
            height += labelHeight;
        }
        if(((NSArray *)[[self.promotionsArray objectAtIndex:index] objectForKey:@"images"]).count > 0) {
            UIImage *image = [URLRequests loadImage:[[[self.promotionsArray objectAtIndex:index] objectForKey:@"images"] objectAtIndex:0] withFolder:nil receive:nil];
            CGFloat width = [UIScreen mainScreen].bounds.size.width - 30;
            CGFloat imageWidth = image.size.width;
            CGFloat scale = width / imageWidth;
            height += image.size.height * scale;
        }
        height += currentHeight;
    } else {
        [self.promotionsTableViewRows setObject:[NSNumber numberWithInt:0] atIndexedSubscript:index];
        CGFloat labelHeight = [[[self.promotionsArray objectAtIndex:index] objectForKey:@"title"] boundingRectWithSize:CGSizeMake([UIScreen mainScreen].bounds.size.width - 50, CGFLOAT_MAX) options:NSStringDrawingUsesLineFragmentOrigin attributes: @{NSFontAttributeName : [UIFont fontWithName:@"PTSans-Regular" size:16]} context:nil].size.height;
        height = labelHeight + 77;
    }
    
    return height;
}

-(CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath {
    if(tableView == self.promotionsTableView) {
        if([self.promotionsTableViewRows objectAtIndex:indexPath.row].intValue == 0) {
            CGFloat labelHeight = [[[self.promotionsArray objectAtIndex:indexPath.row] objectForKey:@"title"] boundingRectWithSize:CGSizeMake([UIScreen mainScreen].bounds.size.width - 50, CGFLOAT_MAX) options:NSStringDrawingUsesLineFragmentOrigin attributes: @{NSFontAttributeName : [UIFont fontWithName:@"PTSans-Regular" size:16]} context:nil].size.height;
            labelHeight = labelHeight + 77;
            return labelHeight;
        } else {
            int height = 0;
            CGFloat labelHeight = [[[self.promotionsArray objectAtIndex:indexPath.row] objectForKey:@"title"] boundingRectWithSize:CGSizeMake([UIScreen mainScreen].bounds.size.width - 50, CGFLOAT_MAX) options:NSStringDrawingUsesLineFragmentOrigin attributes: @{NSFontAttributeName : [UIFont fontWithName:@"PTSans-Regular" size:16]} context:nil].size.height;
            height += labelHeight + 87;
            if([[self.promotionsArray objectAtIndex:indexPath.row] objectForKey:@"content"]) {
                CGFloat labelHeight = [[[self.promotionsArray objectAtIndex:indexPath.row] objectForKey:@"content"] boundingRectWithSize:CGSizeMake([UIScreen mainScreen].bounds.size.width - 50, CGFLOAT_MAX) options:NSStringDrawingUsesLineFragmentOrigin attributes: @{NSFontAttributeName : [UIFont fontWithName:@"PTSans-Regular" size:14]} context:nil].size.height;
                height += labelHeight;
            }
            if(((NSArray *)[[self.promotionsArray objectAtIndex:indexPath.row] objectForKey:@"images"]).count > 0) {
                UIImage *image = [URLRequests loadImage:[[[self.promotionsArray objectAtIndex:indexPath.row] objectForKey:@"images"] objectAtIndex:0] withFolder:nil receive:nil];
                CGFloat width = [UIScreen mainScreen].bounds.size.width - 30;
                CGFloat imageWidth = image.size.width;
                CGFloat scale = width / imageWidth;
                height += image.size.height * scale;
            }
            return height;
        }
    } else if(tableView == self.placesTableView) {
        NSDictionary *singlePlaceDictionary = [self.placesArray objectAtIndex:indexPath.row];
        if([singlePlaceDictionary objectForKey:@"promotions"]) {
            NSString *address =[singlePlaceDictionary objectForKey:@"city"];
            address = [address stringByAppendingFormat:@", %@", [singlePlaceDictionary objectForKey:@"address1"]];
            CGFloat height = [address boundingRectWithSize:CGSizeMake([UIScreen mainScreen].bounds.size.width - 82, CGFLOAT_MAX) options:NSStringDrawingUsesLineFragmentOrigin attributes:@{NSFontAttributeName : [UIFont fontWithName:@"PTSans-Regular" size:16]} context:nil].size.height;
            height += 99;
            NSArray<NSDictionary *> *promotionsArray = (NSArray<NSDictionary *> *)[singlePlaceDictionary objectForKey:@"promotions"];
            for(NSDictionary *promo in promotionsArray) {
                height += [[promo objectForKey:@"title"] boundingRectWithSize:CGSizeMake([UIScreen mainScreen].bounds.size.width - 82, CGFLOAT_MAX) options:NSStringDrawingUsesLineFragmentOrigin attributes:@{NSFontAttributeName : [UIFont fontWithName:@"PTSans-Regular" size:16]} context:nil].size.height + 36;
            }
            return height;
        } else {
            NSString *address =[singlePlaceDictionary objectForKey:@"city"];
            address = [address stringByAppendingFormat:@", %@", [singlePlaceDictionary objectForKey:@"address1"]];
            CGFloat height = [address boundingRectWithSize:CGSizeMake([UIScreen mainScreen].bounds.size.width - 82, CGFLOAT_MAX) options:NSStringDrawingUsesLineFragmentOrigin attributes:@{NSFontAttributeName : [UIFont fontWithName:@"PTSans-Regular" size:16]} context:nil].size.height;
            height += 83;
            return height;
        }
    }
    return 0;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    if(tableView == self.promotionsTableView) {
        return self.promotionsArray.count;
    } else if(tableView == self.placesTableView) {
        return self.placesArray.count;
    }
    return 0;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    if(tableView == self.promotionsTableView) {
        PromotionTableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"Cell"];
        cell.singleBrandViewController = self;
        cell.index = indexPath.row;
        cell.promotionNameLabel.text = [[self.promotionsArray objectAtIndex:indexPath.row] objectForKey:@"title"];
        cell.promoLabel.text = [Utils translate:@"Promo"];
        NSDateFormatter *dateFormatter = [[NSDateFormatter alloc] init];
        dateFormatter.dateFormat = @"yyyy-MM-dd HH:mm:ss";
        NSDate *startingDate = [dateFormatter dateFromString:[[self.promotionsArray objectAtIndex:indexPath.row] objectForKey:@"starts"]];
        NSDate *endingDate = [dateFormatter dateFromString:[[self.promotionsArray objectAtIndex:indexPath.row] objectForKey:@"ends"]];
        if(startingDate != nil && endingDate != nil) {
            dateFormatter.dateFormat = @"dd.MM.yyyy";
            NSString *dateString = [dateFormatter stringFromDate:startingDate];
            NSString *promotionString = [NSString stringWithFormat:@"%@ %@", [Utils translate:@"From"], dateString];
            dateString = [dateFormatter stringFromDate:endingDate];
            promotionString = [promotionString stringByAppendingFormat:@" %@ %@", [Utils translate:@"To"], dateString];
            cell.promotionDuration.text = promotionString;
        } else {
            cell.promotionDuration.text = nil;
        }
        
        if(![[[self.promotionsArray objectAtIndex:indexPath.row] objectForKey:@"content"] isEqualToString:@""]) {
            cell.promotionDescriptionLabel.text = [[self.promotionsArray objectAtIndex:indexPath.row] objectForKey:@"content"];
        }
        
        if(((NSArray *)[[self.promotionsArray objectAtIndex:indexPath.row] objectForKey:@"images"]).count == 0) {
            cell.promotionImageViewHeightConstraint.constant = 0;
            cell.promotionImageViewVerticalConstraint.constant = 0;
        } else {
            UIImage *image = [URLRequests loadImage:[[[self.promotionsArray objectAtIndex:indexPath.row] objectForKey:@"images"] objectAtIndex:0] withFolder:nil receive:^(UIImage *image) {
                if(image != nil) {
                    CGFloat width = [UIScreen mainScreen].bounds.size.width - 30;
                    CGFloat imageWidth = image.size.width;
                    CGFloat scale = width / imageWidth;
                    cell.promotionImageViewHeightConstraint.constant = image.size.height * scale;
                    cell.promotionImageView.image = image;
                }
            }];
            if(image != nil) {
                CGFloat width = [UIScreen mainScreen].bounds.size.width - 30;
                CGFloat imageWidth = image.size.width;
                CGFloat scale = width / imageWidth;
                cell.promotionImageViewHeightConstraint.constant = image.size.height * scale;
                cell.promotionImageView.image = image;
            }
        }
        return cell;
    } else if(tableView == self.placesTableView) {
        NSDictionary *singlePlaceDictionary = [self.placesArray objectAtIndex:indexPath.row];
        if([singlePlaceDictionary objectForKey:@"promotions"]) {
            PlaceWithPromoTableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"PlaceWithPromo"];
            NSString *address =[singlePlaceDictionary objectForKey:@"city"];
            address = [address stringByAppendingFormat:@", %@", [singlePlaceDictionary objectForKey:@"address1"]];
            CGFloat height = [address boundingRectWithSize:CGSizeMake([UIScreen mainScreen].bounds.size.width - 82, CGFLOAT_MAX) options:NSStringDrawingUsesLineFragmentOrigin attributes:@{NSFontAttributeName : cell.placeLocationLabel.font} context:nil].size.height;
            height += 58;
            NSArray<NSDictionary *> *promotionsArray = (NSArray<NSDictionary *> *)[singlePlaceDictionary objectForKey:@"promotions"];
            for(NSDictionary *promo in promotionsArray) {
                height += [[promo objectForKey:@"title"] boundingRectWithSize:CGSizeMake([UIScreen mainScreen].bounds.size.width - 82, CGFLOAT_MAX) options:NSStringDrawingUsesLineFragmentOrigin attributes:@{NSFontAttributeName : [UIFont fontWithName:@"PTSans-Regular" size:16]} context:nil].size.height + 35;
            }
            cell.placeViewHeightConstraint.constant = height;
            if(address != nil && ![address isEqualToString:@""]) {
                cell.placeLocationLabel.text = address;
                cell.placeAddressImageView.hidden = NO;
            } else {
                cell.placeAddressImageView.hidden = YES;
            }
            CLLocation *placeLocation = [[CLLocation alloc] initWithLatitude:[[singlePlaceDictionary objectForKey:@"lat"] floatValue] longitude:[[singlePlaceDictionary objectForKey:@"lng"] floatValue]];
            CLLocationDistance distance = [[Utils getLocation] distanceFromLocation:placeLocation];
            if(distance < 1000) {
                cell.placeDistanceLabel.text = [NSString stringWithFormat:@"%d %@", (int)distance, [Utils translate:@"Meter"]];
            } else {
                cell.placeDistanceLabel.text = [NSString stringWithFormat:@"%.2f %@", distance / 1000, [Utils translate:@"Kilometer"]];
            }
            cell.promotionsArray = [singlePlaceDictionary objectForKey:@"promotions"];
            [cell.placePromotionsTableView reloadData];
            cell.placePromotionLabel.attributedText = [self promotionString:singlePlaceDictionary];
            cell.placePromotionViewWidthConstraint.constant = cell.placePromotionLabel.attributedText.length * 11;
            
            UITapGestureRecognizer *tap = [[UITapGestureRecognizer alloc] initWithTarget:self action:@selector(performSegue)];
            [cell.promotionsView addGestureRecognizer:tap];
            
            if([[singlePlaceDictionary objectForKey:@"new_location"] boolValue] == YES) {
                cell.placeNewPlaceView.hidden = NO;
            } else {
                cell.placeNewPlaceView.hidden = YES;
            }
            
            cell.selectionStyle = UITableViewCellSelectionStyleNone;
            return cell;
        } else {
            PlaceWithoutPromoTableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"PlaceWithoutPromo"];
            NSString *address =[singlePlaceDictionary objectForKey:@"city"];
            address = [address stringByAppendingFormat:@", %@", [singlePlaceDictionary objectForKey:@"address1"]];
            CGFloat height = [address boundingRectWithSize:CGSizeMake([UIScreen mainScreen].bounds.size.width - 82, CGFLOAT_MAX) options:NSStringDrawingUsesLineFragmentOrigin attributes:@{NSFontAttributeName : cell.placeLocationLabel.font} context:nil].size.height;
            height += 42;
            cell.placeViewHeightConstraint.constant = height;
            if(address != nil && ![address isEqualToString:@""]) {
                cell.placeLocationLabel.text = address;
                cell.placeAddressImageView.hidden = NO;
            } else {
                cell.placeAddressImageView.hidden = YES;
            }
            CLLocation *placeLocation = [[CLLocation alloc] initWithLatitude:[[singlePlaceDictionary objectForKey:@"lat"] floatValue] longitude:[[singlePlaceDictionary objectForKey:@"lng"] floatValue]];
            CLLocationDistance distance = [[Utils getLocation] distanceFromLocation:placeLocation];
            if(distance < 1000) {
                cell.placeDistanceLabel.text = [NSString stringWithFormat:@"%d %@", (int)distance, [Utils translate:@"Meter"]];
            } else {
                cell.placeDistanceLabel.text = [NSString stringWithFormat:@"%.2f %@", distance / 1000, [Utils translate:@"Kilometer"]];
            }
            cell.placePromotionLabel.attributedText = [self promotionString:singlePlaceDictionary];
            cell.placePromotionViewWidthConstraint.constant = cell.placePromotionLabel.attributedText.length * 11;
            
            if([[singlePlaceDictionary objectForKey:@"new_location"] boolValue] == YES) {
                cell.placeNewPlaceView.hidden = NO;
            } else {
                cell.placeNewPlaceView.hidden = YES;
            }
            
            cell.selectionStyle = UITableViewCellSelectionStyleNone;
        return cell;
        }
    }
    return nil;
}

-(void)performSegue {
    [self performSegueWithIdentifier:@"PlaceSegue" sender:self];
}

-(void)openGallery {
    [self performSegueWithIdentifier:@"GallerySegue" sender:self];
}

- (void)scrollViewDidScroll:(UIScrollView *)scrollView {
    if(scrollView == self.imagesScrollView) {
        self.currentPage = (scrollView.contentOffset.x + scrollView.frame.size.width / 2) / scrollView.frame.size.width;
    } else {
        self.brandScrollViewOffset = scrollView.contentOffset.y;
    }
}

- (IBAction)onLeftArrowPressed:(id)sender {
    if(self.currentPage > 0) {
        self.currentPage--;
        [UIView animateWithDuration:0.3 animations:^{
            [self.imagesScrollView setContentOffset:CGPointMake(self.currentPage * self.imagesScrollView.frame.size.width, 0)];
        }];
    }
}

- (IBAction)onRightArrowPressed:(id)sender {
    if(self.currentPage < self.imagesCount - 1) {
        self.currentPage++;
        [UIView animateWithDuration:0.3 animations:^{
            [self.imagesScrollView setContentOffset:CGPointMake(self.currentPage * self.imagesScrollView.frame.size.width, 0)];
        }];
    }
}

- (IBAction)addToFavourites:(id)sender {
    [URLRequests followBrand:self.followed withLocationId:self.brandId.intValue receive:^(BOOL success) {
        if(success) {
            if(self.followed == 1) {
                self.followed = 0;
                [self.addToFavouritesButton setTitle:[Utils translate:@"Follow"] forState:UIControlStateNormal];
                UIViewController *viewController = self.navigationController.viewControllers.firstObject;
                if([viewController isKindOfClass:[ContentHolderViewController class]]) {
                    BrandsViewController *brandsViewController = ((ContentHolderViewController *)viewController).brandsViewController;
                    for(Brand *brand in brandsViewController.brandsArray) {
                        if(brand.brandId.intValue == self.brandId.intValue) {
                            brand.isFollowed = NO;
                            break;
                        }
                    }
                    [brandsViewController.brandsCollectionView reloadData];
                } else if([viewController isKindOfClass:[CatalogueViewController class]]) {
                    CatalogueViewController *catalogueViewController = (CatalogueViewController *)viewController;
                    if(catalogueViewController.catalogueType == NEW) {
                        for(NewPlace *newPlace in catalogueViewController.newsArray) {
                            if(newPlace.placeId.intValue == self.brandId.intValue) {
                                newPlace.placeIsFollowed = NO;
                                break;
                            }
                        }
                    } else if(catalogueViewController.catalogueType == PROMOTIONS) {
                        for(Promotion *promotion in catalogueViewController.promotionsArray) {
                            if(promotion.promotionId.intValue == self.brandId.intValue) {
                                promotion.promotionFollowed = [NSNumber numberWithInt:self.followed];
                                break;
                            }
                        }
                    } else if(catalogueViewController.catalogueType == FAVOURITES) {
                        NSMutableArray *mutableArray = [catalogueViewController.newsArray mutableCopy];
                        for(NewPlace *newPlace in catalogueViewController.newsArray) {
                            if(newPlace.placeId.intValue == self.brandId.intValue) {
                                [mutableArray removeObjectAtIndex:[mutableArray indexOfObject:newPlace]];
                                catalogueViewController.newsArray = [mutableArray copy];
                                
                                BrandsViewController *brandsViewController = ((MainMenuViewController *)[catalogueViewController.catalogueControlViewController.navigationController.viewControllers objectAtIndex:0]).brandsViewController;
                                for(Brand *brand in brandsViewController.brandsArray) {
                                    if(brand.brandId.intValue == self.brandId.intValue) {
                                        brand.isFollowed = NO;
                                        break;
                                    }
                                }
                                [brandsViewController.brandsCollectionView reloadData];
                                break;
                            }
                        }
                    }
                    [catalogueViewController.catalogueCollectionView reloadData];
                }
            } else {
                self.followed = 1;
                [self.addToFavouritesButton setTitle:[Utils translate:@"Unfollow"] forState:UIControlStateNormal];
                UIViewController *viewController = self.navigationController.viewControllers.firstObject;
                if([viewController isKindOfClass:[ContentHolderViewController class]]) {
                    BrandsViewController *brandsViewController = ((ContentHolderViewController *)viewController).brandsViewController;
                    for(Brand *brand in brandsViewController.brandsArray) {
                        if(brand.brandId.intValue == self.brandId.intValue) {
                            brand.isFollowed = YES;
                            break;
                        }
                    }
                    [brandsViewController.brandsCollectionView reloadData];
                    
                } else if([viewController isKindOfClass:[CatalogueViewController class]]) {
                    CatalogueViewController *catalogueViewController = (CatalogueViewController *)viewController;
                    if(catalogueViewController.catalogueType == NEW) {
                        for(NewPlace *newPlace in catalogueViewController.newsArray) {
                            if(newPlace.placeId.intValue == self.brandId.intValue) {
                                newPlace.placeIsFollowed = YES;
                                break;
                            }
                        }
                    } else if(catalogueViewController.catalogueType == PROMOTIONS) {
                        for(Promotion *promotion in catalogueViewController.promotionsArray) {
                            if(promotion.promotionId.intValue == self.brandId.intValue) {
                                promotion.promotionFollowed = [NSNumber numberWithInt:self.followed];
                                break;
                            }
                        }
                    } else if(catalogueViewController.catalogueType == FAVOURITES) {
                        NSMutableArray *mutableArray = [catalogueViewController.newsArray mutableCopy];
                        for(NewPlace *newPlace in catalogueViewController.newsArray) {
                            if(newPlace.placeId.intValue == self.brandId.intValue) {
                                [mutableArray removeObjectAtIndex:[mutableArray indexOfObject:newPlace]];
                                catalogueViewController.newsArray = [mutableArray copy];
                                
                                BrandsViewController *brandsViewController = ((MainMenuViewController *)[catalogueViewController.catalogueControlViewController.navigationController.viewControllers objectAtIndex:0]).brandsViewController;
                                for(Brand *brand in brandsViewController.brandsArray) {
                                    if(brand.brandId.intValue == self.brandId.intValue) {
                                        brand.isFollowed = YES;
                                        break;
                                    }
                                }
                                [brandsViewController.brandsCollectionView reloadData];
                                break;
                            }
                        }
                    }
                    [catalogueViewController.catalogueCollectionView reloadData];
                }
                
            }
        }
    }];
}

- (IBAction)onSharePressed:(id)sender {
    if([FBSDKAccessToken currentAccessToken] != nil) {
        FBSDKShareLinkContent *content = [[FBSDKShareLinkContent alloc] init];
        content.contentURL = [NSURL URLWithString:@"http://www.unicreditbulbank.bg/bg/Individual_Clients/Plus_programme/About/index.htm"];
        content.contentTitle = [self.brandDictionary objectForKey:@"name"];
        content.contentDescription = [NSString stringWithFormat:[Utils translate:@"Share description"], [self.brandDictionary objectForKey:@"name"]];
        FBSDKShareDialog *shareDialog = [[FBSDKShareDialog alloc] init];
        shareDialog.fromViewController = self;
        shareDialog.shareContent = content;
        [shareDialog show];
    } else {
        NSArray *itemsToShare = [NSArray arrayWithObjects:[self.brandDictionary objectForKey:@"name"], [NSString stringWithFormat:[Utils translate:@"Share description"], [self.brandDictionary objectForKey:@"name"]], [NSURL URLWithString:@"http://www.unicreditbulbank.bg/bg/Individual_Clients/Plus_programme/About/index.htm"], nil];
        
        UIActivityViewController *activityVC = [[UIActivityViewController alloc] initWithActivityItems:itemsToShare applicationActivities:nil];
        activityVC.excludedActivityTypes = @[UIActivityTypePrint, UIActivityTypeCopyToPasteboard, UIActivityTypeAssignToContact, UIActivityTypeSaveToCameraRoll, UIActivityTypePostToWeibo];
        
        [self presentViewController:activityVC animated:YES completion:nil];
    }
}

-(void)changeLanguage:(NSNotification *)notification {
    if([[Utils currentLanguage] isEqualToString:@"bg"]) {
        self.categoryLabel.text = self.filter.filterBgName;
    } else {
        self.categoryLabel.text = self.filter.filterEnName;
    }
    self.categoryLabel.textColor = [Utils colorFromHexString:self.category.itemCategoryPinColor];
    
    [self.shareButton setTitle:[Utils translate:@"Share"] forState:UIControlStateNormal];
    
    [URLRequests getBrand:self.brandId.intValue receive:^(NSData *data) {
        NSError *error;
        self.brandDictionary = [NSJSONSerialization JSONObjectWithData:data options:0 error:&error];
        
        if(((NSNumber *)[self.brandDictionary objectForKey:@"followed"]).intValue == 0) {
            self.followed = 0;
            [self.addToFavouritesButton setTitle:[Utils translate:@"Follow"] forState:UIControlStateNormal];
        } else {
            self.followed = 1;
            [self.addToFavouritesButton setTitle:[Utils translate:@"Unfollow"] forState:UIControlStateNormal];
        }
        
        self.promotionsArray = (NSArray<NSDictionary *> *)[self.brandDictionary objectForKey:@"promotions"];
        int promotionTableViewHeight = 0;
        for(NSDictionary *promotion in self.promotionsArray ) {
            CGFloat labelHeight = [[promotion objectForKey:@"title"] boundingRectWithSize:CGSizeMake([UIScreen mainScreen].bounds.size.width - 50, CGFLOAT_MAX) options:NSStringDrawingUsesLineFragmentOrigin attributes: @{NSFontAttributeName : [UIFont fontWithName:@"PTSans-Regular" size:16]} context:nil].size.height;
            promotionTableViewHeight += labelHeight + 77;
        }
        self.promotionsTableViewHeightConstraint.constant = promotionTableViewHeight;
        
        self.placesArray = (NSArray<NSDictionary *> *)[self.brandDictionary objectForKey:@"locations"];
        int placesTableViewHeight = 0;
        for(NSDictionary *place in self.placesArray) {
            NSString *address =[place objectForKey:@"city"];
            address = [address stringByAppendingFormat:@", %@", [place objectForKey:@"address1"]];
            CGFloat height = [address boundingRectWithSize:CGSizeMake([UIScreen mainScreen].bounds.size.width - 82, CGFLOAT_MAX) options:NSStringDrawingUsesLineFragmentOrigin attributes:@{NSFontAttributeName : [UIFont fontWithName:@"PTSans-Regular" size:16]} context:nil].size.height;
            if([place objectForKey:@"promotions"]) {
                height += 99;
                for(NSDictionary *promo in [place objectForKey:@"promotions"]) {
                    height += [[promo objectForKey:@"title"] boundingRectWithSize:CGSizeMake([UIScreen mainScreen].bounds.size.width - 82, CGFLOAT_MAX) options:NSStringDrawingUsesLineFragmentOrigin attributes:@{NSFontAttributeName : [UIFont fontWithName:@"PTSans-Regular" size:16]} context:nil].size.height + 36;
                }
                placesTableViewHeight += height;
            } else {
                height += 83;
                placesTableViewHeight += height;
            }
        }
        self.placesTableViewHeightConstraint.constant = placesTableViewHeight;
        [self.placesTableView reloadData];
    }];
}

- (IBAction)onBackPressed:(id)sender {
    [((MainMenuViewController *)[((ContentHolderViewController *)[self.navigationController.viewControllers objectAtIndex:0]).mainNavigationController.viewControllers objectAtIndex:0]) changeHeader:nil withType:self.headerFieldType];
    [self.navigationController popViewControllerAnimated:YES];
}



-(NSMutableAttributedString *)promotionString:(NSDictionary *)dictionary {
    if([[dictionary objectForKey:@"unicredit_promotion"] isKindOfClass:[NSString class]] && ![(NSString *)[dictionary objectForKey:@"unicredit_promotion"] isEqualToString:@"0"]) {
        NSString *promotion = (NSString *)[dictionary objectForKey:@"unicredit_promotion"];
        NSString *forString = [Utils translate:@"For"];
        NSString *currency = [Utils translate:@"Currency"];
        
        NSMutableAttributedString *promotionString = [[NSMutableAttributedString alloc] initWithString:[NSString stringWithFormat:@"%@ %@ 1 %@", promotion, forString, currency]];
        [promotionString addAttribute:NSFontAttributeName value:[UIFont fontWithName:@"PTSans-Bold" size:16] range:NSMakeRange(0, promotion.length)];
        [promotionString addAttribute:NSFontAttributeName value:[UIFont fontWithName:@"PTSans-Regular" size:14] range:NSMakeRange(promotion.length + 1, forString.length)];
        [promotionString addAttribute:NSFontAttributeName value:[UIFont fontWithName:@"PTSans-Bold" size:16] range:NSMakeRange(promotion.length + forString.length + 2, 1)];
        [promotionString addAttribute:NSFontAttributeName value:[UIFont fontWithName:@"PTSans-Regular" size:14] range:NSMakeRange(promotionString.length - currency.length, 3)];
        
        return promotionString;
    } else {
        return [[NSMutableAttributedString alloc] initWithString:@""];
    }
}


#pragma mark - Navigation

- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    if([[segue destinationViewController] isKindOfClass:[GalleryViewController class]]) {
        NSMutableArray<UIImage *> *imagesArray = [[NSMutableArray alloc] initWithCapacity:0];
        for(UIView *subview in self.imagesScrollView.subviews) {
            if([subview isKindOfClass:[UIImageView class]]) {
                [imagesArray addObject:((UIImageView *)subview).image];
            }
        }
        GalleryViewController *galleryController = (GalleryViewController *)[segue destinationViewController];
        galleryController.imagesArray = imagesArray;
        galleryController.currentPage = self.currentPage;
        galleryController.onDismiss = ^(int currentPage) {
            [self.imagesScrollView setContentOffset:CGPointMake(currentPage * self.imagesScrollView.frame.size.width, 0)];
            self.currentPage = currentPage;
        };
    } else if([[segue destinationViewController] isKindOfClass:[PlaceViewController class]]) {
        NSNumber *item;
        if(self.hasSelectedAnnotationView) {
            item = self.locationId;
            self.hasSelectedAnnotationView = YES;
        } else {
            item = [[self.placesArray objectAtIndex:self.placesTableView.indexPathForSelectedRow.row] objectForKey:@"id"];
        }
        PlaceViewController *placeViewController = (PlaceViewController *)[segue destinationViewController];
        placeViewController.placeId = item;
        placeViewController.headerFieldType = @"title";
        placeViewController.headerTitle = [self.brandDictionary objectForKey:@"name"];
        placeViewController.oldTitle = [self.brandDictionary objectForKey:@"name"];
        placeViewController.isFromMainDrawer = self.isFromMainDrawer;
    }
}


@end
