//
//  SubcategoriesViewController.m
//  plus
//
//  Created by Виктор Георгиев on 7/13/16.
//  Copyright © 2016 Outsourcing IT Services Ltd. All rights reserved.
//

#define DURATION 0.3f

#import "AppDelegate.h"
#import "SubcategoriesViewController.h"
#import "SubcategoriesTableViewCell.h"
#import "CategoriesViewController.h"
#import "URLRequests.h"
#import "Utils.h"
#import "Filter.h"

@interface SubcategoriesViewController ()

@property (strong, nonatomic) NSManagedObjectContext *managedObjectContext;

@property (weak, nonatomic) IBOutlet UITableView *subCategoriesTableView;

@property (strong, nonatomic) SubcategoriesTableViewCell *subCategoryCell;

@property (weak, nonatomic) IBOutlet UIView *selectAllView;

@property (weak, nonatomic) IBOutlet UIImageView *headerImageView;
@property (weak, nonatomic) IBOutlet UIImageView *selectAllImageView;
@property (weak, nonatomic) IBOutlet UIImageView *selectAllCheckImageView;

@property (weak, nonatomic) IBOutlet UILabel *headerTitleLabel;
@property (weak, nonatomic) IBOutlet UILabel *selectAllLabel;
@property (weak, nonatomic) IBOutlet UILabel *clearAllLabel;

@property (strong, nonatomic) UIImage *checkboxImage;
@property (strong, nonatomic) UIImage *checkImage;

@property (assign, nonatomic) long selectedCount;

@property (strong, nonatomic) NSArray<ItemCategory *> *subCategoriesArray;
@property (strong, nonatomic) NSArray<Filter *> *filtersArray;

@property (weak, nonatomic) IBOutlet UIButton *showResultsButton;

@property (assign, nonatomic) long selectedRow;

@property (assign, nonatomic) BOOL hasSelectedRow;
@property (assign, nonatomic) BOOL isAllSelected;
@property (assign, nonatomic) BOOL hasChangedHeaderLabel;

@end

@implementation SubcategoriesViewController

- (void)viewDidLoad {
    [super viewDidLoad];
    self.selectedRow = INT_MAX;
    
    self.managedObjectContext = ((AppDelegate *)[UIApplication sharedApplication].delegate).managedObjectContext;
    NSError *error;
    NSFetchRequest *request = [NSFetchRequest fetchRequestWithEntityName:@"Filter"];
    request.predicate = [NSPredicate predicateWithFormat:@"parentId = %@ && filterId != -1", self.subCategory.itemId];
    self.filtersArray = (NSArray<Filter *> *)[self.managedObjectContext executeFetchRequest:request error:&error];
    
    self.subCategoriesArray = self.subCategory.itemSubcategories;
    
    self.headerTitleLabel.text = self.subCategory.itemName;
    UIImage * image = [URLRequests loadImage:self.subCategory.itemCategoryIcon withFolder:@"category" receive:^(UIImage *image) {
        self.headerImageView.image = image;
    }];
    self.headerImageView.image = image;
    
    self.selectAllLabel.text = [Utils translate:@"Select all"];
    self.clearAllLabel.text = [Utils translate:@"Clear all"];
    
    self.checkboxImage = [UIImage imageNamed:@"checkbox"];
    self.checkImage = [UIImage imageNamed:@"check_white"];
    
    request.predicate = [NSPredicate predicateWithFormat:@"parentId == %@ && filterIsSelected == %@", self.subCategory.itemId, @(YES)];
    self.selectedCount = [self.managedObjectContext executeFetchRequest:request error:&error].count;
    if(self.selectedCount == self.subCategoriesArray.count) {
        self.selectAllImageView.hidden = YES;
        self.selectAllCheckImageView.hidden = NO;
        self.hasChangedHeaderLabel = YES;
        self.isAllSelected = YES;
    } else {
        self.isAllSelected = NO;
    }
    
    
    [self.showResultsButton setTitle:[Utils translate:@"Show Results"] forState:UIControlStateNormal];
}

-(void)viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:animated];
    [((CategoriesViewController *)[self.navigationController.viewControllers objectAtIndex:0]).categoriesTableView reloadData];
    [((CategoriesViewController *)[self.navigationController.viewControllers objectAtIndex:0]) checkSelected];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return self.subCategoriesArray.count;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    SubcategoriesTableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"Cell" forIndexPath:indexPath];
    cell.selectionStyle = UITableViewCellSelectionStyleNone;
    ItemCategory *itemCategory = ((ItemCategory *)[self.subCategoriesArray objectAtIndex:indexPath.row]);
    cell.filterLabel.text = itemCategory.itemName;
    NSString *itemCategoryIcon = itemCategory.itemCategoryIcon;
    itemCategoryIcon = [itemCategoryIcon stringByReplacingOccurrencesOfString:@" " withString:@"%20"];
    UIImage *image = [URLRequests loadImage:itemCategoryIcon withFolder:nil receive:^(UIImage *image) {
        for(NSIndexPath *indexPathForImage in [self.subCategoriesTableView indexPathsForVisibleRows]) {
            if([[self.subCategoriesArray objectAtIndex:indexPathForImage.row].itemCategoryIcon isEqualToString:itemCategoryIcon]) {
                ((SubcategoriesTableViewCell *)[self.subCategoriesTableView cellForRowAtIndexPath:indexPathForImage]).filterImageView.image = image;
            }
        }
    }];
    cell.filterImageView.image = image;
    
    BOOL foundId = NO;
    for(Filter *filter in self.filtersArray) {
        if(filter.filterId.intValue == itemCategory.itemId.intValue) {
            if(filter.filterIsSelected.intValue == 1) {
                foundId = YES;
                break;
            }
        }
    }
    
    if(self.selectedRow != indexPath.row && self.selectedRow != -1) {
        if(foundId) {
            cell.filterCheckboxImageView.hidden = YES;
            cell.filterCheckImageView.hidden = NO;
            cell.isSelected = YES;
        } else {
            cell.filterCheckboxImageView.hidden = NO;
            cell.filterCheckImageView.hidden = YES;
            cell.isSelected = NO;
        }
    } else {
        if(foundId) {
            if(!cell.isSelected) {
                [self selectRowAnimation:cell];
            }
        } else {
            if(cell.isSelected) {
                [self deselectRowAnimation:cell];
            }
        }
    }
    
    return cell;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
    ItemCategory *item = [self.subCategoriesArray objectAtIndex:indexPath.row];
    for(Filter *filter in self.filtersArray) {
        if(filter.filterId.intValue == item.itemId.intValue) {
            if(filter.filterIsSelected.intValue == 1) {
                filter.filterIsSelected = [NSNumber numberWithInt:0];
                if(self.selectedCount > 0) {
                    self.selectedCount--;
                }
            } else {
                filter.filterIsSelected = [NSNumber numberWithInt:1];
                if(self.selectedCount < self.subCategoriesArray.count) {
                    self.selectedCount++;
                }
            }
            [((AppDelegate *)[UIApplication sharedApplication].delegate) saveContext];
            break;
        }
    }
    if(self.selectedCount == self.subCategoriesArray.count) {
        self.isAllSelected = YES;
        [self selectAllAnimation];
        self.hasChangedHeaderLabel = YES;
    } else {
        if(self.selectAllImageView.hidden == YES) {
            self.isAllSelected = NO;
            [self deselectAllAnimation];
            self.hasChangedHeaderLabel = NO;
        } else {
            self.selectAllImageView.image = self.checkboxImage;
        }
    }
    self.selectedRow = indexPath.row;
    self.hasSelectedRow = YES;
    self.subCategoryCell = [tableView cellForRowAtIndexPath:indexPath];
    [self.subCategoriesTableView reloadData];
}

- (IBAction)onSelectAllTap:(id)sender {
    if(!self.isAllSelected) {
        self.isAllSelected = YES;
        self.selectedCount = self.subCategoriesArray.count;
        [self selectAllAnimation];
        self.hasChangedHeaderLabel = YES;
        for(Filter *filter in self.filtersArray) {
            filter.filterIsSelected = [NSNumber numberWithInt:1];
        }
        [((AppDelegate *)[UIApplication sharedApplication].delegate) saveContext];
        self.selectedRow = -1;
        [self.subCategoriesTableView reloadData];
    }
}

- (IBAction)onClearAllTap:(id)sender {
    self.isAllSelected = NO;
    self.selectedCount = 0;
    if(self.hasChangedHeaderLabel) {
        [self deselectAllAnimation];
        self.hasChangedHeaderLabel = NO;
    }
    self.selectAllImageView.image = self.checkboxImage;
    for(Filter *filter in self.filtersArray) {
        filter.filterIsSelected = [NSNumber numberWithInt:0];
    }
    [((AppDelegate *)[UIApplication sharedApplication].delegate) saveContext];
    self.selectedRow = -1;
    [self.subCategoriesTableView reloadData];
}

- (IBAction)onShowResultsPressed:(id)sender {
    [((CategoriesViewController *)[self.navigationController.viewControllers objectAtIndex:0]) onShowResultsPressed:self];
}

- (IBAction)onBackPressed:(id)sender {
    [self.navigationController popViewControllerAnimated:YES];
}

- (void)animationDidStop:(CAAnimation *)theAnimation finished:(BOOL)flag {
    self.selectAllImageView.layer.anchorPoint = CGPointMake(0.5, 0.5);
    self.selectAllImageView.layer.position = CGPointMake(22 + self.selectAllImageView.frame.size.width / 2, self.selectAllImageView.superview.frame.size.height / 2);
    self.selectAllCheckImageView.layer.anchorPoint = CGPointMake(0.5, 0.5);
    self.selectAllCheckImageView.layer.position = CGPointMake(22 + self.selectAllImageView.frame.size.width / 2, self.selectAllImageView.superview.frame.size.height / 2);
}

-(void)selectAllAnimation {
    self.selectAllImageView.hidden = NO;
    self.selectAllCheckImageView.hidden = NO;
    
    self.selectAllImageView.layer.anchorPoint = CGPointMake(0.0, 0.5);
    self.selectAllImageView.layer.position = CGPointMake(22, self.selectAllView.frame.size.height / 2);
    
    CABasicAnimation *rotate = [CABasicAnimation animationWithKeyPath:@"transform.rotation"];
    rotate.delegate = self;
    rotate.fromValue = [self degreesToRadians:0];
    rotate.toValue = [self degreesToRadians:-45];
    rotate.duration = DURATION;
    rotate.repeatCount = 1;
    CABasicAnimation *fadeOut = [CABasicAnimation animationWithKeyPath:@"opacity"];
    fadeOut.fromValue = [NSNumber numberWithFloat:1];
    fadeOut.toValue = [NSNumber numberWithFloat:0];
    fadeOut.duration = DURATION;
    fadeOut.repeatCount = 1;
    [self.selectAllImageView.layer addAnimation:rotate forKey:@"rotate"];
    [self.selectAllImageView.layer addAnimation:fadeOut forKey:@"fadeOut"];
    
    self.selectAllCheckImageView.layer.anchorPoint = CGPointMake(0.0, 0.5);
    self.selectAllCheckImageView.layer.position = CGPointMake(22, self.selectAllView.frame.size.height / 2);
    
    CABasicAnimation *fadeIn = [CABasicAnimation animationWithKeyPath:@"opacity"];
    fadeIn.fromValue = [NSNumber numberWithFloat:0];
    fadeIn.toValue = [NSNumber numberWithFloat:1];
    fadeIn.duration = DURATION;
    fadeIn.repeatCount = 1;
    
    rotate.fromValue = [self degreesToRadians:45];
    rotate.toValue = [self degreesToRadians:0];
    rotate.duration = DURATION;
    rotate.repeatCount = 1;
    [self.selectAllCheckImageView.layer addAnimation:rotate forKey:@"rotate"];
    [self.selectAllCheckImageView.layer addAnimation:fadeIn forKey:@"fadeIn"];
    
    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, DURATION - 0.04 * NSEC_PER_SEC), dispatch_get_main_queue(), ^{
        self.selectAllImageView.hidden = YES;
    });
}

-(void)deselectAllAnimation {
    self.selectAllImageView.hidden = NO;
    self.selectAllCheckImageView.hidden = NO;
    
    self.selectAllImageView.layer.anchorPoint = CGPointMake(0.0, 0.5);
    self.selectAllImageView.layer.position = CGPointMake(22, self.selectAllImageView.superview.frame.size.height / 2);
    
    CABasicAnimation *rotate = [CABasicAnimation animationWithKeyPath:@"transform.rotation"];
    rotate.delegate = self;
    rotate.fromValue = [self degreesToRadians:-45];
    rotate.toValue = [self degreesToRadians:0];
    rotate.duration = DURATION;
    rotate.repeatCount = 1;
    CABasicAnimation *fadeOut = [CABasicAnimation animationWithKeyPath:@"opacity"];
    fadeOut.fromValue = [NSNumber numberWithFloat:0];
    fadeOut.toValue = [NSNumber numberWithFloat:1];
    fadeOut.duration = DURATION;
    fadeOut.repeatCount = 1;
    [self.selectAllImageView.layer addAnimation:rotate forKey:nil];
    [self.selectAllImageView.layer addAnimation:fadeOut forKey:nil];
    
    self.selectAllCheckImageView.layer.anchorPoint = CGPointMake(0.0, 0.5);
    self.selectAllCheckImageView.layer.position = CGPointMake(22, self.selectAllView.frame.size.height / 2);
    
    CABasicAnimation *fadeIn = [CABasicAnimation animationWithKeyPath:@"opacity"];
    fadeIn.fromValue = [NSNumber numberWithFloat:1];
    fadeIn.toValue = [NSNumber numberWithFloat:0];
    fadeIn.duration = DURATION;
    fadeIn.repeatCount = 1;
    
    rotate.fromValue = [self degreesToRadians:0];
    rotate.toValue = [self degreesToRadians:45];
    rotate.duration = DURATION;
    rotate.repeatCount = 1;
    [self.selectAllCheckImageView.layer addAnimation:rotate forKey:nil];
    [self.selectAllCheckImageView.layer addAnimation:fadeIn forKey:nil];
    
    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, DURATION - 0.04 * NSEC_PER_SEC), dispatch_get_main_queue(), ^{
        self.selectAllCheckImageView.hidden = YES;
    });
}

-(void)selectRowAnimation:(SubcategoriesTableViewCell *)cell {
    cell.filterCheckImageView.hidden = NO;
    cell.filterCheckboxImageView.hidden = NO;
    
    cell.filterCheckboxImageView.layer.anchorPoint = CGPointMake(0.0, 0.5);
    cell.filterCheckboxImageView.layer.position = CGPointMake(22, cell.frame.size.height / 2);
    
    CABasicAnimation *rotate = [CABasicAnimation animationWithKeyPath:@"transform.rotation"];
    rotate.delegate = self;
    rotate.fromValue = [self degreesToRadians:0];
    rotate.toValue = [self degreesToRadians:-45];
    rotate.duration = DURATION;
    rotate.repeatCount = 1;
    CABasicAnimation *fadeOut = [CABasicAnimation animationWithKeyPath:@"opacity"];
    fadeOut.fromValue = [NSNumber numberWithFloat:1];
    fadeOut.toValue = [NSNumber numberWithFloat:0];
    fadeOut.duration = DURATION;
    fadeOut.repeatCount = 1;
    [cell.filterCheckboxImageView.layer addAnimation:rotate forKey:@"rotate"];
    [cell.filterCheckboxImageView.layer addAnimation:fadeOut forKey:@"fadeOut"];
    
    cell.filterCheckImageView.layer.anchorPoint = CGPointMake(0.0, 0.5);
    cell.filterCheckImageView.layer.position = CGPointMake(22, cell.frame.size.height / 2);
    
    CABasicAnimation *fadeIn = [CABasicAnimation animationWithKeyPath:@"opacity"];
    fadeIn.fromValue = [NSNumber numberWithFloat:0];
    fadeIn.toValue = [NSNumber numberWithFloat:1];
    fadeIn.duration = DURATION;
    fadeIn.repeatCount = 1;
    
    rotate.fromValue = [self degreesToRadians:45];
    rotate.toValue = [self degreesToRadians:0];
    rotate.duration = DURATION;
    rotate.repeatCount = 1;
    [cell.filterCheckImageView.layer addAnimation:rotate forKey:@"rotate"];
    [cell.filterCheckImageView.layer addAnimation:fadeIn forKey:@"fadeIn"];
    
    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, DURATION - 0.04 * NSEC_PER_SEC), dispatch_get_main_queue(), ^{
        cell.filterCheckboxImageView.layer.anchorPoint = CGPointMake(0.5f, 0.5f);
        cell.filterCheckboxImageView.layer.position = CGPointMake(22 + cell.filterCheckboxImageView.frame.size.width / 2, cell.frame.size.height / 2);
        cell.filterCheckImageView.layer.anchorPoint = CGPointMake(0.5f, 0.5f);
        cell.filterCheckImageView.layer.position = CGPointMake(22 + cell.filterCheckboxImageView.frame.size.width / 2, cell.frame.size.height / 2);
        cell.filterCheckboxImageView.hidden = YES;
        cell.isSelected = YES;
    });
}

-(void)deselectRowAnimation:(SubcategoriesTableViewCell *)cell {
    cell.filterCheckImageView.hidden = NO;
    cell.filterCheckboxImageView.hidden = NO;
    
    cell.filterCheckboxImageView.layer.anchorPoint = CGPointMake(0.0, 0.5);
    cell.filterCheckboxImageView.layer.position = CGPointMake(22, cell.frame.size.height / 2);
    
    CABasicAnimation *rotate = [CABasicAnimation animationWithKeyPath:@"transform.rotation"];
    rotate.delegate = self;
    rotate.fromValue = [self degreesToRadians:-45];
    rotate.toValue = [self degreesToRadians:0];
    rotate.duration = DURATION;
    rotate.repeatCount = 1;
    CABasicAnimation *fadeOut = [CABasicAnimation animationWithKeyPath:@"opacity"];
    fadeOut.fromValue = [NSNumber numberWithFloat:0];
    fadeOut.toValue = [NSNumber numberWithFloat:1];
    fadeOut.duration = DURATION;
    fadeOut.repeatCount = 1;
    [cell.filterCheckboxImageView.layer addAnimation:rotate forKey:@"rotate"];
    [cell.filterCheckboxImageView.layer addAnimation:fadeOut forKey:@"fadeOut"];
    
    cell.filterCheckImageView.layer.anchorPoint = CGPointMake(0.0, 0.5);
    cell.filterCheckImageView.layer.position = CGPointMake(22, cell.frame.size.height / 2);
    
    CABasicAnimation *fadeIn = [CABasicAnimation animationWithKeyPath:@"opacity"];
    fadeIn.fromValue = [NSNumber numberWithFloat:1];
    fadeIn.toValue = [NSNumber numberWithFloat:0];
    fadeIn.duration = DURATION;
    fadeIn.repeatCount = 1;
    
    rotate.fromValue = [self degreesToRadians:0];
    rotate.toValue = [self degreesToRadians:45];
    rotate.duration = DURATION;
    rotate.repeatCount = 1;
    [cell.filterCheckImageView.layer addAnimation:rotate forKey:@"rotate"];
    [cell.filterCheckImageView.layer addAnimation:fadeIn forKey:@"fadeIn"];
    
    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, DURATION - 0.04 * NSEC_PER_SEC), dispatch_get_main_queue(), ^{
        cell.filterCheckboxImageView.layer.anchorPoint = CGPointMake(0.5f, 0.5f);
        cell.filterCheckboxImageView.layer.position = CGPointMake(22 + cell.filterCheckboxImageView.frame.size.width / 2, cell.frame.size.height / 2);
        cell.filterCheckImageView.layer.anchorPoint = CGPointMake(0.5f, 0.5f);
        cell.filterCheckImageView.layer.position = CGPointMake(22 + cell.filterCheckboxImageView.frame.size.width / 2, cell.frame.size.height / 2);
        cell.filterCheckImageView.hidden = YES;
        cell.isSelected = NO;
    });
}

- (NSNumber *)degreesToRadians:(CGFloat)degrees {
    return [NSNumber numberWithFloat:degrees * M_PI / 180];
}

/*
#pragma mark - Navigation

// In a storyboard-based application, you will often want to do a little preparation before navigation
- (void)prepareForSegue:(UIStoryboardSegue *)segue sender:(id)sender {
    // Get the new view controller using [segue destinationViewController].
    // Pass the selected object to the new view controller.
}
*/

@end
