//
//  Utils.m
//  plus
//
//  Created by Виктор Георгиев on 7/8/16.
//  Copyright © 2016 Outsourcing IT Services Ltd. All rights reserved.
//

#import "Utils.h"
#import <SystemConfiguration/SCNetworkReachability.h>

#import "LoadAnimationViewController.h"

@implementation Utils

static UINavigationController *navigationController;

static NSArray<ItemCategory *> *categoriesArray;

static CLLocation *lastKnownLocation;

+ (NSString *)currentLanguage {
    NSUserDefaults *userDefaults = [NSUserDefaults standardUserDefaults];
    return [[userDefaults objectForKey:@"AppleLanguages"] objectAtIndex:0];
}

+ (NSString *)translate:(NSString *)UtilsKey {
    NSArray *userDefaults = [[NSUserDefaults standardUserDefaults] arrayForKey:@"AppleLanguages"];
    NSString *UtilsLanguage = userDefaults[0];
    if(![UtilsLanguage isEqualToString:@"bg"] && ![UtilsLanguage isEqualToString:@"en"]) {
        UtilsLanguage = @"bg";
    }
    NSString * path = [[NSBundle mainBundle] pathForResource:UtilsLanguage ofType:@"lproj"];
    NSBundle * languageBundle = [NSBundle bundleWithPath:path];
    return [languageBundle localizedStringForKey:UtilsKey value:UtilsKey table:nil];
}

+ (UIColor *)colorFromHexString:(NSString *)hexString {
    NSString *cleanString = [hexString stringByReplacingOccurrencesOfString:@"#" withString:@""];
    if([cleanString length] == 3) {
        cleanString = [NSString stringWithFormat:@"%@%@%@%@%@%@",
                       [cleanString substringWithRange:NSMakeRange(0, 1)],[cleanString substringWithRange:NSMakeRange(0, 1)],
                       [cleanString substringWithRange:NSMakeRange(1, 1)],[cleanString substringWithRange:NSMakeRange(1, 1)],
                       [cleanString substringWithRange:NSMakeRange(2, 1)],[cleanString substringWithRange:NSMakeRange(2, 1)]];
    }
    if([cleanString length] == 6) {
        cleanString = [@"ff" stringByAppendingString:cleanString];
    }
    
    unsigned int baseValue;
    [[NSScanner scannerWithString:cleanString] scanHexInt:&baseValue];
    
    float alpha = ((baseValue >> 24) & 0xFF)/255.0f;
    float red = ((baseValue >> 16) & 0xFF)/255.0f;
    float green = ((baseValue >> 8) & 0xFF)/255.0f;
    float blue = ((baseValue >> 0) & 0xFF)/255.0f;
    
    
    return [UIColor colorWithRed:red green:green blue:blue alpha:alpha];
}

+ (UIImage *)imageWithImage:(UIImage *)image scaledToSize:(CGSize)newSize {
    UIGraphicsBeginImageContextWithOptions(newSize, NO, 0.0);
    [image drawInRect:CGRectMake(0, 0, newSize.width, newSize.height / 2)];
    UIImage *newImage = UIGraphicsGetImageFromCurrentImageContext();
    UIGraphicsEndImageContext();
    return newImage;
}

+ (void)setCategories:(NSArray<ItemCategory *> *)array {
    categoriesArray = array;
}

+ (NSArray<ItemCategory *> *)getCategories {
    return categoriesArray;
}

+ (void)saveLocation:(CLLocation *)location {
    lastKnownLocation = location;
}

+ (CLLocation *)getLocation {
    return lastKnownLocation;
}

+ (void)displayAnimation {
    if(navigationController == nil) {
        navigationController = (UINavigationController *)[[UIApplication sharedApplication].delegate window].rootViewController;
    }
    if(![navigationController.presentedViewController isKindOfClass:[LoadAnimationViewController class]]) {
        [navigationController.viewControllers.firstObject performSegueWithIdentifier:@"AnimationSegue" sender:navigationController.viewControllers.firstObject];
    }
}

+ (void)dismissAnimation {
    if([navigationController.presentedViewController isKindOfClass:[LoadAnimationViewController class]]) {
        [navigationController.viewControllers.firstObject dismissViewControllerAnimated:NO completion:nil];
    }
}

+ (BOOL)deviceHasInternetConneciton {
    SCNetworkReachabilityFlags flags;
    SCNetworkReachabilityRef address;
    address = SCNetworkReachabilityCreateWithName(NULL, "www.apple.com" );
    Boolean success = SCNetworkReachabilityGetFlags(address, &flags);
    CFRelease(address);
    
    bool canReach = success
    && !(flags & kSCNetworkReachabilityFlagsConnectionRequired)
    && (flags & kSCNetworkReachabilityFlagsReachable);
    
    return canReach;
}

@end
